; $Id: dcs_cryst_align.pro,v 1.68.2.5 2015/06/04 15:32:34 ymqiu Exp $
;#######################################################################
;
; NAME:
;  dcs_cryst_align
;
; PURPOSE:
;  This program plots crystal alignment files
;
; CATEGORY:
;  DCS data tools
;
; AUTHOR:
;  Yiming Qiu
;  NIST Center for Neutron Research
;  100 Bureau Drive, Gaithersburg, MD 20899-6102
;  United States
;  yiming.qiu@nist.gov
;  June, 2015
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;#######################################################################

@dm_macs_sequencer    ;for several MACS utilities
@dm_calc_projection   ;for dm_macs_lambda2correction

pro dcs_cryst_align_e2lambda_event,event
    widget_control,/hourglass
    widget_control,event.handler,get_uvalue=state

    ;catch and ignore all errors in this program
    catch, myerror
    if myerror ne 0 then begin
       ok = dialog_message(dialog_parent=state.tlb,!error_state.msg,/error,/center)
       catch,/cancel
       return
    end
    
    case event.id of
       state.eTxt:     dcs_cryst_align_e2lambda_update,state,/E
       state.lTxt:     dcs_cryst_align_e2lambda_update,state,/lambda
       state.a2Txt:    dcs_cryst_align_e2lambda_update,state,/A2
       state.plotBut:  DCSToolsPerformance,group_leader=event.top
       state.exitBut:  begin
                       widget_control,event.handler,/destroy
                       return
                       end
       else:
    endcase
 
    widget_control,event.handler,set_uvalue=state
end

pro dcs_cryst_align_e2lambda_update,state,E=E,lambda=lambda,A2=A2
    eng = !values.f_nan & wav = !values.f_nan  & atu = !values.f_nan ;initialization
    ;retrieve values from text boxes
    current_id = 0>(keyword_set(lambda)+2*keyword_set(A2))<2
    widget_control,([state.eTxt,state.lTxt,state.a2Txt])[current_id],get_value=str
    if n_elements(str) eq 0 then return
    if state.current[current_id] eq strtrim(str,2) then return
    tmp = dm_to_number(strsplit(str,' ,'+string(9b),/extract))
    ind0 = where(finite(tmp),count0)
    if count0 gt 0 then begin
       ind1 = where(tmp[ind0] gt 0,count1)
       if count1 gt 0 then begin
          if keyword_set(E) then begin
             eng = tmp[ind0[ind1]]
             wav = sqrt(81.8042/eng)
             atu = asin(wav/2/3.35416)/!dtor*2
             widget_control,state.lTxt,set_value=dm_to_string(wav,resolution=3,separator=', ')
          endif
          if keyword_set(lambda) then begin
             wav = tmp[ind0[ind1]]
             eng = 81.8042/wav/wav
             atu = asin(wav/2/3.35416)/!dtor*2
             widget_control,state.eTxt,set_value=dm_to_string(eng,resolution=3,separator=', ')
          endif
          if keyword_set(a2) then begin
             atu = tmp[ind0[ind1]]
             wav = sin(atu/2*!dtor)*3.35416*2
             eng = 81.8042/wav/wav
             widget_control,state.lTxt,set_value=dm_to_string(wav,resolution=3,separator=', ')
             widget_control,state.eTxt,set_value=dm_to_string(eng,resolution=3,separator=', ')
          endif
       endif
       if (state.instr eq 2) and (~keyword_set(A2)) then widget_control,state.a2Txt,set_value=dm_to_string(atu,res=3,separator=', ') ;MACS
    endif
    
    wav = wav[0] & eng = eng[0] & atu = atu[0] ;only the first one  
    if finite(eng) and finite(wav) then begin
       if (state.instr eq 1) then begin ;for DCS, figure out the energy resolution and flux 
          f_low  = interpol((*(state.flPtr))[1,*],(*(state.flPtr))[0,*],wav)/state.fmax*100.0
          f_med  = interpol((*(state.fmPtr))[1,*],(*(state.fmPtr))[0,*],wav)/state.fmax*100.0
          flux   = 'low:'+strtrim(string(0>(f_low)<100,format='(f6.2)'),2)+', '+$
                   'med:'+strtrim(string(0>(f_med)<100,format='(f6.2)'),2)
          widget_control,state.fxTxt,set_value=flux
          er_low = 13220./wav^2.947
          er_med = 3948./wav^2.656
          eresl  = 'low:'+strtrim(string(er_low,format='(f9.1)'),2)+', '+$
                   'med:'+strtrim(string(er_med,format='(f9.1)'),2)
          widget_control,state.erTxt,set_value=eresl 
       endif   
    endif else begin
       if (state.instr eq 1) then begin ;DCS
          widget_control,state.fxTxt,set_value=''
          widget_control,state.erTxt,set_value=''
       endif
       if keyword_set(E) then      widget_control,state.lTxt,set_value=''
       if keyword_set(lambda) then widget_control,state.eTxt,set_value=''
       if (state.instr eq 2) and (~keyword_set(A2)) then widget_control,state.a2Txt,set_value=''
    endelse
    if (state.instr eq 2) and keyword_set(A2) and ~finite(atu) then begin
       widget_control,state.lTxt,set_value=''
       widget_control,state.eTxt,set_value=''
    endif
    widget_control,state.eTxt,get_value=str & state.current[0] = strtrim(str,2)
    widget_control,state.lTxt,get_value=str & state.current[1] = strtrim(str,2)
    if state.instr eq 2 then begin
       widget_control,state.a2Txt,get_value=str & state.current[2] = strtrim(str,2)
    endif
end

pro dcs_cryst_align_e2lambda_Exit,tlb
    widget_control,tlb,get_uvalue=state
    ptr_free,state.flPtr,state.fmPtr
    widget_control,tlb,/destroy
end

;E to Lambda conversion
pro dcs_cryst_align_e2lambda,event,lambda=lambda,ei=ei,dcs=dcs,macs=macs
    state={group_leader:    0L, $   ;group leader
           tlb:             0L, $   ;top level base
           instr:           0b, $   ;flag for instrument type: 0-other 1-dcs 2-macs
           eTxt:            0L, $   ;E text box
           lTxt:            0L, $   ;lambda text box
           erTxt:           0L, $   ;E resolution text box
           fxTxt:           0L, $   ;flux text box
           a2Txt:           0L, $   ;A2 text box
           current:   strarr(3),$   ;save current E, lambda, A2 string to avoid unnecessary updates
           plotBut:         0L, $   ;start performance button
           exitBut:         0L, $   ;exit button
           fmax:            0e, $   ;maximum flux
           flPtr:    ptr_new(), $   ;low resl mode flux
           fmPtr:    ptr_new()  $   ;medium resl mode flux
          }
    registerName = 'dcs_cryst_align_e2lambda'
    if xregistered(registerName) then begin ;only allow one copy running
       id = LookupManagedWidget(registername)
       widget_control,id,iconify=0
       return
    endif
    state.instr = keyword_set(dcs)
    if keyword_set(macs) then state.instr = 2
    if n_elements(ei) eq 0 then ei = !values.f_nan
    if n_elements(lambda) eq 0 then lambda = !values.f_nan
    if n_elements(event) ne 0 then state.group_leader = event.top
    state.tlb = widget_base(title='Convert Energy to Wavelength',/col,kill_notify='dcs_cryst_align_e2lambda_Exit',tlb_frame_attr=1,group_leader=state.group_leader)
    row1        = widget_base(state.tlb,/row,/grid_layout)
    col1        = widget_base(row1,/col,/grid_layout)
    col2        = widget_base(row1,/col,/grid_layout)
    if (state.instr eq 2) then begin
    col3        = widget_base(row1,/col,/grid_layout)
    void        = widget_label(col3,value='A2('+string(176b)+')',/align_center)
    state.a2Txt = widget_text(col3,/editable,/all_events)
    void        = widget_label(state.tlb,value='A2 is calculated using PG (002) d-spacing of 3.35416 '+string('c5'xb)+'.',/align_center)
    endif
    void        = widget_label(col1,value='E (meV)',/align_center)
    state.eTxt  = widget_text(col1,/editable,/all_events,value=dm_to_string(ei))
    void        = widget_label(col2,value='Lambda ('+string('c5'xb)+')',/align_center)
    state.lTxt  = widget_text(col2,/editable,/all_events,value=dm_to_string(lambda))
    if (state.instr eq 1) then begin
    void        = widget_label(col1,value='FWHM ('+string('b5'xb)+'eV)',/align_center)
    state.erTxt = widget_text(col1)
    void        = widget_label(col2,value='Flux (% of maximum)',/align_center)
    state.fxTxt = widget_text(col2)
    endif
    row2        = widget_base(state.tlb,/row,/align_center)
    if (state.instr eq 1) then begin
    state.plotBut = widget_button(row2,value='  Plot  ',sensitive=(xregistered('dave',/noshow) or xregistered('wd_DAVETool',/noshow)))
    void          = widget_label(row2,value= '   ')
    endif
    state.exitBut = widget_button(row2,value='  Exit  ')
    ;dcs flux info
    state.flPtr = ptr_new([[1.25,357], [1.3,1799.4], [1.35,30630], [1.36,44460], [1.38,69780], [1.4,86595], [1.42,86820], [1.44,59370], [1.45,57570], [1.46,73200],$
                    [1.48,117900], [1.5,137498], [1.52,143280], [1.54,145380], [1.55,135750], [1.56,106530], [1.58,33600], [1.6,34440], [1.62,122010], [1.64,177630],$
                    [1.65,172500], [1.66,158280], [1.68,167760], [1.7,212850], [1.72,165270], [1.74,68310], [1.75,51450], [1.76,71100], [1.78,186870], [1.8,266385],$
                    [1.82,265980], [1.84,216420], [1.85,177360], [1.86,131010], [1.88,43860], [1.9,4959], [1.95,3192], [1.96,7602], [1.98,32730], [2,87915],$
                    [2.02,116220], [2.04,60900], [2.05,28993.5], [2.06,10188], [2.08,820.8], [2.1,167.22], [2.15,385.5], [2.2,196260], [2.25,541500], [2.3,720400],$
                    [2.4,803100], [2.5,875400], [2.6,905700], [2.7,912600], [2.8,902400], [2.9,898500], [3,901350], [3.1,888300], [3.2,876600], [3.25,855900],$
                    [3.27,826800], [3.3,760800], [3.33,390900], [3.35,346200], [3.38,731100], [3.4,831900], [3.42,839400], [3.45,846000], [3.5,861300], [3.6,870300],$
                    [3.7,877800], [3.8,859500], [3.85,844500], [3.9,832200], [3.95,807000], [3.98,781200], [4,763800], [4.03,734700], [4.05,787800], [4.08,850500],$
                    [4.1,872400], [4.15,857400], [4.2,847200], [4.3,828900], [4.4,798300], [4.5,765900], [4.6,738000], [4.62,719400], [4.65,716400], [4.67,746400],$
                    [4.7,780000], [4.72,777900], [4.75,772800], [4.78,766200], [4.8,763950], [5,721650], [5.5,645900], [6,558000], [6.3,508200], [6.4,487500],$
                    [6.45,466500], [6.5,453600], [6.55,405600], [6.6,297180], [6.65,33900], [6.7,24780], [6.75,279690], [6.8,384600], [6.85,402600], [6.9,413700],$
                    [6.95,406200], [7,404850], [7.5,349800], [8,292040], [9,199770], [10,142250], [12,69260], [14,34890], [15,25104], [16,17742],$
                    [18,9459], [20,5184]])
    state.fmPtr = ptr_new([[1.35,3894], [1.4,11083.5], [1.45,4354.5], [1.5,17070], [1.55,17355], [1.6,1409.55], [1.65,22560], [1.7,27450], [1.75,3330], [1.8,34275],$
                    [1.85,22425], [1.9,207.45], [1.95,322.2], [2,11304], [2.05,1801.5], [2.1,130.185], [2.15,150.9], [2.2,28005], [2.25,72900], [2.3,92160],$
                    [2.4,102165], [2.5,112905], [2.6,117765], [2.7,120540], [2.8,117645], [2.9,119475], [3,122535], [3.2,117090], [3.25,114390], [3.3,103335],$
                    [3.35,31950], [3.4,112305], [3.5,116445], [3.6,120810], [3.7,120810], [3.8,119025], [3.85,118320], [3.9,115035], [3.95,111540], [4,105735],$
                    [4.05,106050], [4.1,123960], [4.15,121905], [4.2,121185], [4.3,117795], [4.4,113085], [4.6,103245], [4.8,110460], [5,103703], [5.5,91095],$
                    [6,77842.5], [6.5,59820], [6.6,36780], [6.7,952.5], [6.8,49335], [6.9,53190], [7,54127.5], [7.5,43965], [8,36540], [9,24315],$
                    [10,16657.5], [12,7679.25], [14,3831], [16,1926], [18,1006.58], [20,554.55]])
    state.fmax = max((*(state.flPtr))[1,*])
    dm_center_kid,state.tlb,state.group_leader
    widget_control,state.tlb,/realize
    if finite(ei) then dcs_cryst_align_e2lambda_update,state,/e $
    else if finite(lambda) then dcs_cryst_align_e2lambda_update,state,/lambda
    widget_control,state.tlb,set_uvalue=state
    xmanager,registerName,state.tlb,cleanup='dcs_cryst_align_e2lambda_Exit',/no_block
end

;build context sensitive menu
pro dcs_cryst_align_csm,center,pState,csms=csms
    for i=0,1 do begin
        if n_elements(csms) ne 2 then csm = widget_base(center,/context_events,uname='contextevent'+dm_to_string(i),xpad=0,ypad=0,space=0) else csm=csms[i]
        void = widget_label(csm,value=' ',sensitive=0)
        mid  = widget_base(csm,/context_menu,uname='contextmenu')
        void = dm_widget_button(mid,value='Show Cursor Position',uname='optnshowcursor')
        void = dm_widget_button(mid,value='Show Fitting Parameters',uname='optnshowfit')
        void = dm_widget_button(mid,value='Fix Fitting Width',uname='optnfixwidth')
        fitm = widget_button(mid,value='Fitting Function',/menu,/separator)
        void = dm_widget_button(fitm,value='Gaussian',uname='fit_gaussian')
        void = dm_widget_button(fitm,value='Lorentzian',uname='fit_lorentzian',sensitive=(*pState).mpfit_exist)
        void = dm_widget_button(fitm,value='Moffat',uname='fit_moffat',sensitive=(*pState).mpfit_exist)
        void = dm_widget_button(fitm,value=(*pState).fitfunname[(*pState).fitfunchoice],uname='fitfun_3',/separator)
        void = dm_widget_button(fitm,value=(*pState).fitfunname[(*pState).fitfunchoice]+(['+C3','+C4'])[(*pState).fitfunchoice eq 2],uname='fitfun_4')
        void = dm_widget_button(fitm,value=(*pState).fitfunname[(*pState).fitfunchoice]+(['+C3+C4*x','+C4+C5*x'])[(*pState).fitfunchoice eq 2],uname='fitfun_5')
        void = dm_widget_button(fitm,value=(*pState).fitfunname[(*pState).fitfunchoice]+(['+C3+C4*x+C5*x^2','+C4+C5*x+C6*x^2'])[(*pState).fitfunchoice eq 2],uname='fitfun_6')
    endfor
end

;calculate the fitting line data
pro dcs_cryst_align_fitline,pState,params,x,xx=xx,yy=yy,fwhm=fwhm,xmin=xmin,xmax=xmax
    nx = 150>(n_elements(x))
    if (n_elements(xmin) eq 0) or (n_elements(xmax) eq 0) then xmax = max(x,min=xmin)
    if n_elements(fwhm) eq 0 then begin
       if (*pState).fitfunchoice eq 0 then fwhm = params[2]*sqrt(8.*alog(2)) else fwhm = params[2]*2.0
    endif
    x0 = xmin+findgen(nx)/(nx-1.0)*(xmax-xmin+2)-1
    is_moffat = ((*pState).fitfunchoice eq 2)
    if params[2] ne 0 then begin
       if fwhm lt xmax-xmin then begin
          xx = params[1]-4*params[2]+findgen(nx)/(nx-1.0)*8*params[2]
          id = where(x0 lt params[1]-4*params[2],count)
          if count gt 0 then xx = [x0[id],xx]
          id = where(x0 gt params[1]+4*params[2],count)
          if count gt 0 then xx = [xx,x0[id]]
       endif else xx = x0
       case (*pState).fitfunchoice of
          0:  yy = params[0]*exp(-0.5*((xx-params[1])/params[2])^2)
          1:  yy = params[0]/(1.+((xx-params[1])/params[2])^2)
          2:  yy = params[0]/(1.+((xx-params[1])/params[2])^2)^params[3]
       endcase
       if (*pState).fitfunnterm ge 4 then yy = yy+params[3+is_moffat]
       if (*pState).fitfunnterm ge 5 then yy = yy+params[4+is_moffat]*xx
       if (*pState).fitfunnterm ge 6 then yy = yy+params[5+is_moffat]*xx^2
    endif else begin
       xx = x0
       yy = fltarr(n_elements(x0))
       if (*pState).fitfunnterm ge 4 then yy = yy+params[3]
    endelse 
end

;plotting
pro dcs_cryst_align_plot,pState,dataonly=dataonly,xs=xs,ys=ys,dys=dys,params=params,sigma=sigma,fixedwidth=fixedwidth,peaktime=peaktime
    if (*pState).fit then (*pState).plotWin->getproperty,xran=oldxran ;save the xrange
    if n_elements(*(*pState).yPtr) eq 0 then return ;no data
    tit    = (*pState).title  
    xtit   = (*(*pState).xtit)[(*pState).xaxis]
    ytit   = 'Neutron Counts'
    n_file = n_elements((*(*pState).xPtr)[0,*]) 
    n_det  = n_elements(*(*pState).detPtr)
    t_axis = ((*pState).xaxis eq (n_elements(*(*pState).xtit)-1))
    textra = (t_axis and ((*pState).ftype ne 'WAND'))*(*pState).tchoice
    case (*pState).ftype of 
         'DCS':   begin
              ytit = ytit+'/'+dm_to_string((*pState).duration,res=1)+' Sec'+(['','s'])[(*pState).duration ge 1.5]
              if n_elements(*(*pState).bgPtr) ne 0 then dcsbgfactor = (*pState).dcs_monitor[0]/(*pState).dcs_monitor[1]
              end
         'MACS':  begin
              xA4 = stregex(xtit,'A4',/fold_case,/boolean)  ;Is x axis A4?
              yMn = ((*(*pState).detPtr)[0] eq -1)          ;Is y axis monitor?
              yPt = ((*(*pState).detPtr)[0] eq -2)          ;Is detector PTAI?
              iA4 = (where(stregex((*(*pState).xtit),'A4',/fold_case,/boolean)))[0]
              ikn = iA4+2 & iPt = iA4+1
              mon = reform((*(*pState).xPtr)[n_elements((*(*pState).xPtr)[*,0])-1,*])
              tim = reform((*(*pState).xPtr)[n_elements((*(*pState).xPtr)[*,0])-2,*])
              ief = 2*((*pState).macs_eff-1)                ;detector efficiency index
              macs_ydat = (*(*pState).yPtr)
              macs_yerr = (*(*pState).dyPtr)
              timestamp = reform((*(*pState).xPtr)[n_elements((*(*pState).xPtr)[*,0])-4,*])
              if (*pState).macs_normchoice eq 0 then monfac = mon[0]/(1>mon) $  ;monitor normalization
              else monfac = tim[0]/tim
              for j=0L,n_elements(monfac)-1 do begin
                  macs_ydat[*,*,j] = macs_ydat[*,*,j]*monfac[j]
                  macs_yerr[*,*,j] = macs_yerr[*,*,j]*monfac[j]
              endfor
              if (*pState).macs_eff ne 0 then begin         ;apply detector efficiency
                 for i=0L,n_file-1 do begin
                     for j=0,19 do begin
                         macs_ydat[0,j,i] = macs_ydat[0,j,i]*((*pState).macs_deteff)[ief,j]
                         macs_ydat[1,j,i] = macs_ydat[1,j,i]*((*pState).macs_deteff)[ief+1,j]
                         macs_yerr[0,j,i] = macs_yerr[0,j,i]*((*pState).macs_deteff)[ief,j]
                         macs_yerr[1,j,i] = macs_yerr[1,j,i]*((*pState).macs_deteff)[ief+1,j]
                     endfor  
                 endfor
                 macs_ydat[0,20,*] = macs_ydat[0,20,*]*((*pState).macs_deteff)[ief,(*(*pState).xPtr)[iPt,*]-1]
                 macs_ydat[1,20,*] = macs_ydat[1,20,*]*((*pState).macs_deteff)[ief+1,(*(*pState).xPtr)[iPt,*]-1]
                 macs_yerr[0,20,*] = macs_yerr[0,20,*]*((*pState).macs_deteff)[ief,(*(*pState).xPtr)[iPt,*]-1]
                 macs_yerr[1,20,*] = macs_yerr[1,20,*]*((*pState).macs_deteff)[ief+1,(*(*pState).xPtr)[iPt,*]-1]
              endif
              if (*pState).macs_lamb2 then begin            ;apply monitor lambda/2 correction
                 iA2  = (where(stregex((*(*pState).xtit),'A2',/fold_case,/boolean)))[0] 
                 Ei   = reform(81.8042/((2*(*pState).macs_maspacing[0]*sin(!dtor*(*(*pState).xPtr)[iA2+2,*]/2))^2))
                 cfx  = reform((*(*pState).xPtr)[n_elements((*(*pState).xPtr)[*,0])-3,*])
                 mon1 = dm_macs_lambda2correction(Ei,mon,cfx)
                 fac  = mon/(1>(mon1))  
                 for i=0L,n_file-1 do begin
                     macs_ydat[*,*,i] = macs_ydat[*,*,i]*fac[i]
                     macs_yerr[*,*,i] = macs_yerr[*,*,i]*fac[i]
                 endfor
              endif
              titind = stregex(tit,'\(2th=([0-9.]+)',/subexpr,length=titlen)
              if titlen[1] gt 0 then tittth = dm_to_number(strmid(tit,titind[1],titlen[1])) else begin
                 titind = stregex(tit,'\(<2th>=([0-9.]+)',/subexpr,length=titlen)
                 if titlen[1] gt 0 then tittth = dm_to_number(strmid(tit,titind[1],titlen[1]))
              endelse
              if (*pState).duration lt 0 then $
                 ytit = ytit+'/'+dm_to_string(-(*pState).duration,/int)+' Sec'+(['s',''])[(*pState).duration ge -1] $
              else $
                 ytit = ytit+'/mon='+dm_to_string((*pState).duration,/exp)
              end
         else:
    endcase
    showindvdet = (*pState).indvdet
    if (strpos(tit,'det=all') ne -1) and ((*pState).ftype ne 'MACS') then showindvdet = 0b 
    if strpos(tit,'det=upper') ne -1   then showindvdet = 0b 
    if strpos(tit,'det=central') ne -1 then showindvdet = 0b 
    if strpos(tit,'det=lower') ne -1   then showindvdet = 0b 
    if keyword_set(dataonly)           then showindvdet = 0b 
    hidelegend = (n_elements(*(*pState).detPtr) eq 1) or (~showindvdet)
    colrs = (*pState).plotWin->getcolor(/list,linecolor=colr0)  ;colr0 is the first line color
    colrs = colrs[0:n_elements(colrs)-2]                        ;remove the user defined color
    (*pState).plotWin->getproperty,textpos=textpos,textfsize=textfsize
    if n_elements(textpos) eq 2 then (*pState).textposfsize[0:1] = textpos
    if n_elements(textfsize) eq 1 then (*pState).textposfsize[2] = textfsize
    for ii = 0L, 0>(showindvdet*n_det-1) do begin
        x = reform((*(*pState).xPtr)[(*pState).xaxis+textra,*])
        if (*pState).ftype eq 'MACS' then begin
           if ((*pState).xaxis gt iA4) or (((*pState).xaxis eq iA4) and (~yPt)) then x = reform((*(*pState).xPtr)[(*pState).xaxis+2+textra,*])
        endif
        y = fltarr(n_file) & dy = y
        if n_elements(dcsbgfactor) ne 0 then begin  ;DCS empty can subtraction
           y_bg = total((*(*pState).bgPtr)[*(*pState).detPtr],/nan)
           ind =  where((*(*pState).bgPtr)[*(*pState).detPtr] eq 0,count)
           tmp_yerr = (*(*pState).dbgPtr)[*(*pState).detPtr]
           if count ne 0 then begin
              zero_bgerror = min(abs(tmp_yerr[ind]))
              tmp_yerr[ind] = 0
           endif
           dy_bg = sqrt(total(tmp_yerr^2,/nan))
           if (dy_bg eq 0) and (count ne 0) then dy_bg = zero_bgerror
        endif
        for i=0,n_file-1 do begin
            if showindvdet then begin
               if (*pState).ftype eq 'MACS' then begin
                  if ~yMn and ~yPt then begin
                     if (*pState).y_typ eq 2 then begin ;spec+diff
                        y[i]  = macs_ydat[0,(*(*pState).detPtr)[ii],i]+macs_ydat[1,(*(*pState).detPtr)[ii],i]
                        dy[i] = sqrt((macs_yerr[0,(*(*pState).detPtr)[ii],i])^2+(macs_yerr[1,(*(*pState).detPtr)[ii],i])^2)
                     endif else begin                   ;spec or diff
                        y[i]  = macs_ydat[(*pState).y_typ,(*(*pState).detPtr)[ii],i]
                        dy[i] = macs_yerr[(*pState).y_typ,(*(*pState).detPtr)[ii],i]
                     endelse
                     if xA4 then x[i] = x[i]-76+(*(*pState).tthPtr)[(*(*pState).detPtr)[ii]]
                  endif
               endif else begin
                  y[i]  = (*(*pState).yPtr)[(*(*pState).detPtr)[ii],i]
                  dy[i] = (*(*pState).dyPtr)[(*(*pState).detPtr)[ii],i]
                  if n_elements(dcsbgfactor) ne 0 then begin  ;DCS empty can subtraction
                     y[i]  = y[i]-dcsbgfactor*(*(*pState).bgPtr)[(*(*pState).detPtr)[ii]]
                     dy[i] = sqrt((dy[i])^2+(dcsbgfactor*(*(*pState).dbgPtr)[(*(*pState).detPtr)[ii]])^2)
                  endif
               endelse
            endif else begin
               if (*pState).ftype eq 'MACS' then begin
                  if ~yMn and ~yPt and ~xA4 then begin
                     ind = where(macs_ydat[*,*(*pState).detPtr,i] eq 0,count)
                     if count ne 0 then begin
                        tmp_yerr   = macs_yerr[*,*(*pState).detPtr,i]
                        zero_error = min(abs(tmp_yerr[ind]))
                        tmp_yerr[ind] = 0.0
                        macs_yerr[*,*(*pState).detPtr,i] = temporary(tmp_yerr)
                     endif
                     if (*pState).y_typ eq 2 then begin ;spec+diff
                        y[i]  = total(macs_ydat[0,*(*pState).detPtr,i],/nan)+total(macs_ydat[1,*(*pState).detPtr,i],/nan)
                        dy[i] = sqrt(total((macs_yerr[0,*(*pState).detPtr,i])^2,/nan)+total((macs_yerr[1,*(*pState).detPtr,i])^2,/nan))
                     endif else begin                   ;spec or diff
                        y[i]  = total(macs_ydat[(*pState).y_typ,*(*pState).detPtr,i],/nan)
                        dy[i] = sqrt(total((macs_yerr[(*pState).y_typ,*(*pState).detPtr,i])^2,/nan))
                     endelse
                     if count ne 0 and dy[i] eq 0 then dy[i] = zero_error
                  endif
               endif else begin
                  ind = where((*(*pState).yPtr)[*(*pState).detPtr,i] eq 0,count)
                  tmp_yerr = (*(*pState).dyPtr)[*(*pState).detPtr,i]
                  if count ne 0 then begin
                     zero_error = min(abs(tmp_yerr[ind]))
                     tmp_yerr[ind] = 0.0
                  endif
                  y[i]  = total((*(*pState).yPtr)[*(*pState).detPtr,i],/nan)
                  dy[i] = sqrt(total(tmp_yerr^2,/nan))
                  if count ne 0 and dy[i] eq 0 then dy[i] = zero_error
                  if n_elements(dcsbgfactor) ne 0 then begin  ;DCS empty can subtraction
                     y[i]  = y[i]-dcsbgfactor*y_bg
                     dy[i] = sqrt((dy[i])^2+dcsbgfactor^2*dy_bg^2)
                  endif
               endelse
            endelse
        endfor
        if keyword_set(yMn) then begin
           y  = mon
           dy = sqrt(mon)
           if xA4 then begin
              x  = (fltarr(20)+1)#x
              timestamp = (fltarr(20)+1)#timestamp
              for i=0,19 do begin
                  x[i,*] = x[i,*]-76+(*(*pState).tthPtr)[i]
              endfor
              y  = (fltarr(20)+1)#y
              dy = (fltarr(20)+1)#dy
           endif
        endif 
        if keyword_set(yPt) then begin
           if (*pState).y_typ eq 2 then begin ;spec+diff
              y  = reform(macs_ydat[0,20,*]+macs_ydat[1,20,*])
              dy = sqrt((reform(macs_yerr[0,20,*]))^2+(reform(macs_yerr[1,20,*]))^2)        
           endif else begin                   ;spec or diff
              y  = reform(macs_ydat[(*pState).y_typ,20,*])
              dy = reform(macs_yerr[(*pState).y_typ,20,*])
           endelse
        endif 
        if ~showindvdet and keyword_set(xA4) then begin ;special case of MACS A4
           if ~yMn and ~yPt then begin
              x = (fltarr(n_det)+1)#x
              timestamp = (fltarr(20)+1)#timestamp
              for i=0,n_det-1 do begin
                  x[i,*] = x[i,*]-76+(*(*pState).tthPtr)[(*(*pState).detPtr)[i]]
              endfor
              x  = reform(x)
              if (*pState).y_typ eq 2 then begin ;spec+diff
                 y  = reform(macs_ydat[0,*(*pState).detPtr,*]+macs_ydat[1,*(*pState).detPtr,*])
                 dy = sqrt(reform(macs_yerr[0,*(*pState).detPtr,*])^2+reform(macs_yerr[1,*(*pState).detPtr,*])^2)
              endif else begin                   ;spec or diff
                 y  = reform(macs_ydat[(*pState).y_typ,*(*pState).detPtr,*])
                 dy = reform(macs_yerr[(*pState).y_typ,*(*pState).detPtr,*])
              endelse
           endif
        endif
        macsptai = (((*pState).ftype eq 'MACS') and (n_elements(*(*pState).detPtr) eq 1) and ((*(*pState).detPtr)[0] eq -2))
        if t_axis and (n_file gt 1) then begin  ;combine data with same T_set
           case (*pState).ftype of 
                'DCS':  T_set = reform((*(*pState).xPtr)[n_elements((*(*pState).xPtr)[*,0])-2,*])
                'MACS': T_set = reform((*(*pState).xPtr)[n_elements((*(*pState).xPtr)[*,0])-5,*])
             else:      T_set = round(x*100.0)/100.0
           endcase
           tmp = T_set  
           uniq_val = tmp[uniq(tmp,sort(tmp))]
           dm_step_bin,0,T_set,ydat=x,uniq_val=uniq_val  
           dm_step_bin,0,tmp,ydat=y,yerr=dy,uniq_val=uniq_val,/dataminzero
        endif else if macsptai then begin
           x = round(temporary(x)*1d3,/L64)/1d3
           uniq_val = x[uniq(x,sort(x))]
           dm_step_bin,0,x,ydat=y,yerr=dy,uniq_val=uniq_val,/dataminzero
        endif
        lsty = (['no line',(['solid','dashed'])[(ii/15) mod 2]])[~hidelegend]
        if showindvdet then begin
           if (*pState).ftype eq 'DCS' then begin
              if (*(*pState).detPtr)[ii] eq 913 or (*(*pState).detPtr)[ii] eq 914 then begin
                 legd = (['FC2','FC3'])[(*(*pState).detPtr)[ii]-913]
              endif else begin
                 legd = dm_to_string((*(*pState).detPtr)[ii]+1)+([' L',' C',' U'])[dm_mask_detbanks(detnum=(*(*pState).detPtr)[ii])+1]
              endelse 
           endif else legd = dm_to_string((*(*pState).detPtr)[ii]+1)
        endif else legd = ''
        if ii eq 0 then begin 
           xs  = x
           ys  = y
           dys = dy
           if ~keyword_set(dataonly) then $
              (*pState).plotWin->setproperty,xdat=x,ydat=y,yerr=dy,/nodraw,xtit=xtit,ytit=ytit,psym='circle',hidelegend=hidelegend,linestyle=lsty,color=colr0,legend=legd
        endif else begin
           xs  = [xs,x]
           ys  = [ys,y]
           dys = [dys,dy]
           if ~keyword_set(dataonly) then $
              (*pState).plotWin->add_plot,x,y,yerr=dy,psym='circle',linestyle=lsty,color=colrs[ii mod n_elements(colrs)],legend=legd
        endelse
    endfor
    if keyword_set(dataonly) then return
    xmax = max(xs,min=xmin) 
    ymax = max(ys+dys,imax) & imax = imax[0]
    ymin = min(ys-dys)
    xran = [xmin,xmax]+[-0.05,0.05]*(xmax-xmin+abs(xmax+1)/10.)
    yran = [ymin,ymax]+[-0.07,0.07]*(ymax-ymin)
    if (*pState).fit and (n_elements(y) gt ((*pState).fitfunnterm+((*pState).fitfunchoice eq 2))) and ((~(*pState).indvdet) or (n_elements(*(*pState).detPtr) eq 1)) then begin
       if total(finite(oldxran,/nan)) eq 0 then begin
          index = where((x ge oldxran[0]) and (x le oldxran[1]),count)
          if count gt ((*pState).fitfunnterm+((*pState).fitfunchoice eq 2)) then begin
             x = x[index]
             y = y[index]
             dy = dy[index]
             if n_elements(timestamp) ne 0 then timestamp = timestamp[index]
          endif
       endif
       index = where(dy le 0,count)
       if count ne 0 then dy[index] = 1
       index = sort(x) & x = x[index] & y = y[index] & dy = dy[index]
       if n_elements(timestamp) ne 0 then timestamp = timestamp[index]
       is_moffat = ((*pState).fitfunchoice eq 2)
       if (*pState).fixfitwidth then begin
          if (n_elements(fixedwidth) eq 0) then begin
             widget_control,widget_info((*pState).tlb,find_by_uname='fixwidth'),get_value=fixedwidth
             fixedwidth = dm_to_number(fixedwidth)
          endif
       endif else fixedwidth = !values.f_nan
       dm_gaussfit,y,x=x,params=params,lorentzian=((*pState).fitfunchoice eq 1),moffat=is_moffat,nterms=(*pState).fitfunnterm+is_moffat,measure_errors=dy,chisq=chisq,sigma=sigma,/auto,fwhm=fwhm,fixedwidth=fixedwidth
       if (*pState).fixfitwidth then begin
          if ~finite(fixedwidth) then begin
             wid = widget_info((*pState).tlb,find_by_uname='fixwidth')
             if wid ne 0 then widget_control,wid,set_value=dm_to_string(params[2])
          endif
       endif
       if total(finite(params)) eq n_elements(params) then begin
          dcs_cryst_align_fitline,pState,params,x,xx=xx,yy=yy,fwhm=fwhm,xmin=xmin,xmax=xmax
          unit = ' '
          if strmid(xtit,0,1,/reverse_offset) eq ')' then begin
             pos = strpos(xtit,'(',/reverse_search)
             if pos ne -1 then unit = unit+strmid(xtit,pos+1,strlen(xtit)-pos-2) 
          endif
          (*pState).plotWin->add_plot,xx,yy,linestyle='solid',color=(['red','cyan'])[total(abs([255,0,0]-colr0)) eq 0],legend=(['gaussian','lorentzian','moffat'])[(*pState).fitfunchoice]+' fit',layer=0,thick=2
          text = 'peak!l !n='+dm_to_string(params[1],resolution=2)+unit+'!Cfwhm='+dm_to_string(fwhm,resolution=3)+unit
          if ((*pState).ftype eq 'MACS') and (n_elements(tittth) ne 0) then begin
             tmp = min(abs(x-params[1]),ind)
             if ~xA4 then tittth = mean((*(*pState).xPtr)[ikn,ind[0]])-76.0+tittth else tittth = mean((*(*pState).xPtr)[ikn,*])-76.0+tittth 
             tit = strmid(tit,0,titind[1])+dm_to_string(tittth,res=3)+strmid(tit,titind[1]+titlen[1])
             (*pState).tth[1] = tittth
          endif
          if ((*pState).xaxis eq 0) and finite((*pState).tth[(*pState).ftype eq 'MACS']) then begin
             tth = (*pState).tth[(*pState).ftype eq 'MACS'] 
             if (*pState).ftype eq 'WAND' then tth = -tth  ;WAND
             if tth ge 0 then a2 = 90.-tth/2. else a2 = -(90.+tth/2.)
             if (*pState).ftype eq 'MACS' then a2 = 90-a2  ;MACS
             text = text+'!C'+(*(*pState).psiname)[(where(*(*pState).fname eq (*pState).ftype))[0]]+' reset to '+dm_to_string(a2,resolution=3)+string('b0'xb)              
          endif
          if (*pState).showfit then begin
             res1 = (1>(ceil(2-alog10(min(abs(sigma[where(sigma ne 0)]))+1e-7))))
             res2 = (1>(ceil(2-alog10(abs(chisq)+1e-7))))
             text = text+'!C!Dparams=['+dm_to_string(params,sep=', ')+']!C!Derror=['+dm_to_string(sigma,sep=', ',res=res1)+']!C!D\chi!s!e2!r!ired!d='+dm_to_string(chisq,res=res2)+'!n'
             if (n_elements(timestamp) ne 0) and (params[1] ge x[0]) and (params[1] le x[n_elements(x)-1]) then begin
                ind = (where(x ge params[1]))[0]
                if ind eq 0 then timestamp = timestamp[0] else timestamp = interpol(timestamp[ind-1:ind],x[ind-1:ind],params[1])
                text = text+'!C!Dpeak time: '+dm_to_string(timestamp,/date,/est)
                if arg_present(peaktime) then peaktime = timestamp
             endif
          endif
          (*pState).plotWin->add_text,text,(*pState).textposfsize[0],(*pState).textposfsize[1],fontsize=(*pState).textposfsize[2]
       endif
    endif else begin
       if ((*pState).ftype eq 'MACS') and (n_elements(tittth) ne 0) then begin
          tittth = mean((*(*pState).xPtr)[ikn,*])-76.0+tittth 
          tit = strmid(tit,0,titind[1])+dm_to_string(tittth,res=3)+strmid(tit,titind[1]+titlen[1])
          (*pState).tth[1] = tittth
       endif
    endelse
    if ~(*pState).showfname then filename = '' else begin
       filename = (*pState).filename[0]
       if n_elements(*(*pState).bgPtr) ne 0 then filename = '('+filename+')-('+(*pState).filename[1]+')'
    endelse
    (*pState).plotWin->setproperty,xran=xran,yran=yran,/nodraw,title=tit,cornertxt=filename
    (*pState).plotWin->draw
end

;calculate the 2theta angle from hkl
function dcs_cryst_align_2theta1,Ei,hkl,dir_latt_parm,Ef=Ef
    q = dcs_cryst_align_qlength1(hkl,dir_latt_parm)
    return,dcs_cryst_align_2theta2(Ei,q,Ef=Ef)
end

;calculate the 2theta angle from q&E
function dcs_cryst_align_2theta2,Ei,q,Ef=Ef
    if n_elements(Ef) eq 0 then Ef = Ei
    ki = 2*!dpi/sqrt((81.8042d)/Ei)     
    kf = 2*!dpi/sqrt((81.8042d)/Ef)  
    if q gt ki+kf or q lt abs(ki-kf) then return,!values.f_nan else return,acos((ki^2+kf^2-q^2)/2./ki/kf)/!dpi*180.0
end

;calculate the angle between hkl1 and hkl2
function dcs_cryst_align_angle,hkl1,hkl2,dir_latt_parm
    abc_r = dm_basis_r(dir_latt_parm)
    q1 = abc_r#hkl1
    q2 = abc_r#hkl2
    return,acos(q1##transpose(q2)/norm(q1)/norm(q2))/!dpi*180.0
end

;calculate the lattice parameter from two theta and hkl assuming a=b=c
function dcs_cryst_align_latta,Ei,tth,hkl,abg
    qlen = dcs_cryst_align_qlength1(hkl,[1.0,1.0,1.0,abg])
    return,qlen*sqrt((81.8042d)/Ei)/4./!dpi/sin(abs(tth)/2*!dpi/180.0)
end

;calculate q length from direct crystal lattice parameters
function dcs_cryst_align_qlength1,hkl,cryst
    ddtor   = !dpi/(180d)
    sinabg  = sin(cryst[3:5]*ddtor)
    cosabg  = cos(cryst[3:5]*ddtor)
    v       = cryst[0]*cryst[1]*cryst[2]*sqrt((1d)+(2d)*cosabg[0]*cosabg[1]*cosabg[2]-total(cosabg[0:2]^2))
    rec_abc = (2d)*!dpi/v*[cryst[1]*cryst[2]*sinabg[0],cryst[0]*cryst[2]*sinabg[1],cryst[0]*cryst[1]*sinabg[2]]
    tmp     = (4d)*!dpi*!dpi/v/v*cryst[0]*cryst[1]*cryst[2]
    aadotbb = tmp*cryst[2]*(cosabg[0]*cosabg[1]-cosabg[2])
    bbdotcc = tmp*cryst[0]*(cosabg[1]*cosabg[2]-cosabg[0])
    ccdotaa = tmp*cryst[1]*(cosabg[2]*cosabg[0]-cosabg[1])
    return, sqrt(total((hkl*rec_abc)^2)+(2d)*(hkl[0]*hkl[1]*aadotbb+hkl[1]*hkl[2]*bbdotcc+hkl[2]*hkl[0]*ccdotaa))
end

;calculate q length from Ei, Ef and 2theta
function dcs_cryst_align_qlength2,Ei,tth,Ef=Ef
    if n_elements(Ef) eq 0 then Ef = Ei
    ki = (2.*!dpi)/sqrt(81.8042d/Ei)
    kf = (2.*!dpi)/sqrt(81.8042d/Ef)
    return,sqrt(ki^2+kf^2-2*ki*kf*cos(tth*!dpi/180.0))
end

;change file type
pro dcs_cryst_align_changeftype,event,init=init,pState=pState,det_info=det_info,x_info=x_info
    if keyword_set(init) then begin
       *(*pState).fname   = ['DCS','MACS','WAND']
       *(*pState).fextn   = ['.dcs','.ng0,.bt9','.wand']
       *(*pState).psiname = ['A2','A3','Psi']
    endif else begin
       uname = widget_info(event.id,/uname)
       if strmid(uname,0,5) ne 'ftype' then return
       widget_control,event.handler,get_uvalue=pState
       ftype = strmid(uname,6) 
       if (*pState).ftype eq ftype then return else (*pState).ftype = ftype
       ftypemenu = widget_info((*pState).tlb,find_by_uname='ftypeMenu')
       checked   = widget_info(ftypemenu,find_by_uname='ftype_'+(*pState).ftype)
       for i=0,n_elements(*(*pState).fname)-1 do begin
           if (*(*pState).fname)[i] ne (*pState).ftype then begin
              tmp = widget_info(ftypemenu,find_by_uname='ftype_'+(*(*pState).fname)[i])
              unchecked = (n_elements(unchecked) eq 0)?tmp:([unchecked,tmp])
           endif
       endfor
       dm_toggle_menubut,check=checked,uncheck=unchecked
       widget_control,widget_info((*pState).tlb,find_by_uname='optnbitflip'),sensitive=((*pState).ftype eq 'DCS')
       widget_control,widget_info((*pState).tlb,find_by_uname='optnapplylamb2'),sensitive=((*pState).ftype eq 'MACS')
       widget_control,widget_info((*pState).tlb,find_by_uname='tempMenu'),sensitive=((*pState).ftype ne 'WAND')
       optnmenu = widget_info((*pState).mbar,find_by_uname='optnMenu')
       toolmenu = widget_info((*pState).mbar,find_by_uname='toolMenu')
       backmenu = widget_info((*pState).mbar,find_by_uname='backgroundMenu')
       kidlimitBut = widget_info(toolmenu,find_by_uname='kidlimitbut')
       if (*pstate).ftype eq 'MACS' then begin  ;MACS
          normmenu = widget_button(optnmenu,value='Intensity Normalization Choice',uname='normMenu',/menu)
          void = dm_widget_button(normmenu,value='Divided by Monitor Count',uname='norm_monitor',set_button=((*pState).macs_normchoice eq 0))
          void = dm_widget_button(normmenu,value='Divided by Counting Time',uname='norm_time',set_button=((*pState).macs_normchoice eq 1))
          macseffmenu = dm_widget_button(optnmenu,value='Apply Detector Efficiency',set_button=((*pState).macs_eff ne 0),uname='macseffMenu',/menu)
          void = dm_widget_button(macseffmenu,value='From File',uname='macs_eff_file',set_button=((*pState).macs_eff eq 1))
          void = dm_widget_button(macseffmenu,value='Specify...',uname='macs_eff_spec',set_button=((*pState).macs_eff eq 2))
          void = dm_widget_button(macseffmenu,value='None',uname='macs_eff_none',set_button=((*pState).macs_eff eq 0),sensitive=((*pState).macs_eff ne 0),/separator)
          if kidlimitBut eq 0 then begin
             void = widget_button(toolmenu,value='Kidney Angle Limit',uname='kidlimitbut')
             void = widget_button(toolmenu,value='Fit and Plot Instrument Alignment Files',uname='macsalignment',/separator)
             void = widget_button(toolmenu,value='Fit and Plot Monochromator Alignment Files',uname='macsalignmonchr')
             void = widget_button(toolmenu,value='Fit and Plot Analyzer Alignment Files',uname='macsalignanalyzer')
             void = widget_button(toolmenu,value='Fit and Plot Kidney Alignment File',uname='macsalignkidney')
             void = widget_button(toolmenu,value='Calculate Flipping Ratio',uname='macsflipratio',/separator)
          endif
       endif else begin    
          normmenu = widget_info(optnmenu,find_by_uname='normMenu')
          macseffmenu = widget_info(optnmenu,find_by_uname='macseffMenu')  
          if normmenu ne 0 then widget_control,normmenu,/destroy       
          if macseffmenu ne 0 then widget_control,macseffmenu,/destroy       
          if kidlimitBut ne 0 then begin
             widget_control,kidlimitBut,/destroy
             widget_control,widget_info(toolmenu,find_by_uname='macsalignment'),/destroy
             widget_control,widget_info(toolmenu,find_by_uname='macsalignmonchr'),/destroy
             widget_control,widget_info(toolmenu,find_by_uname='macsalignanalyzer'),/destroy
             widget_control,widget_info(toolmenu,find_by_uname='macsalignkidney'),/destroy
             widget_control,widget_info(toolmenu,find_by_uname='macsflipratio'),/destroy
          endif
       endelse
       if ((*pState).ftype eq 'DCS') and (backmenu eq 0) then begin
          if backmenu eq 0 then begin
             backmenu = widget_button((*pState).mbar,value='Background',/menu,uname='backgroundMenu')
             void     = widget_button(backmenu,value='Load Empty Can File(s)',uname='loadbgfile')
             void     = widget_button(backmenu,value='Clear Background',uname='clearbgfile',/separator,sensitive=0)
          endif
       endif else if backmenu ne 0 then widget_control,backmenu,/destroy
    endelse
    
    ;initialize the instrument info
    (*pState).t_chan  = 1
    (*pState).e_ran   = [!values.f_nan,!values.f_nan]
    (*pState).tth     = [!values.f_nan,!values.f_nan]
    (*pState).xaxis   = 0
    (*pState).fit     = 0
    (*pState).textposfsize = [0.05,0.93,14] ;set them to default every time the file type is changed
    case (*pState).ftype of   ;temperature needs to be the last entry of xtit
       'DCS':begin     ;DCS
         alldetPtr    = dm_load_detpos(/dcs)
         det_info     = ['All','Specify','Central Bank','Upper Bank','Lower Bank','FC2','FC3']
         x_info       = ['Rotation','Height','X-translation','Y-translation','X-tilt','Y-tilt','File Index','Temperature']
         *(*pState).xtit = ['Rotation (\deg)','Height (mm)','X-translation (mm)','Y-translation (mm)','X-tilt (\deg)','Y-tilt (\deg)','File Index','T (K)']
         (*pState).tch_tit = 'tchan='+(['all','elastic','specify'])[(*pState).t_chan]
         end
       'MACS':begin    ;MACS
         alldetPtr    = dm_load_detpos(/macs) 
         det_info     = ['All','Specify','Monitor','PTAI']
         x_info       = ['H','K','L','E','A3','A4','A1','A2','A5','A6','DFM','MonRot','MonTrans','DFMDTS','MBTSlide','MonBlade'+string(1+indgen(21),format='(i02)'),'AnalyzerTheta'+string(1+indgen(20),format='(i02)'),$
                        'Smpl'+['X','Y','Z','LTilt','UTilt'],'DMBT','Focus','Beta1','Beta2','VBAH','VBAV','MagField','Temperature']
         *(*pState).xtit = ['H','K','L','E (meV)',['A3','A4','A1','A2','A5','A6','DFM','MonRot']+' (\deg)',['MonTrans','DFMDTS']+' (mm)','MBTSlide (cm)','MonBlade'+string(1+indgen(21),format='(i02)')+' (\deg)',$
                        'AnalyzerTheta'+string(1+indgen(20),format='(i02)')+' (\deg)',['X','Y','Z']+' (mm)',['Lower','Upper']+'-tilt (\deg)','DMBT (mm)','Focus',['Beta1','Beta2']+' (\deg)',$
                        ['VBAH','VBAV']+' (mm)','\mu!i0!nH (T)','T (K)']
         (*pState).tch_tit = ''
         end  
       'WAND':begin    ;WAND
         alldetPtr    = dm_load_detpos(/wand) 
         det_info     = ['All','Specify']
         x_info       = ['Psi','Temperature']
         *(*pState).xtit = ['Psi (\deg)','T (K)']
         (*alldetPtr).two_theta  = (*alldetPtr).two_theta*(-1) 
         (*(*pState).infoPtr)[0] = 1.46 ;fixed wavelength
         (*pState).tch_tit = ''
         end
       else:
    endcase
    iftype            = (where(*(*pState).fname eq (*pState).ftype))[0]
    *(*pState).tthPtr = (*alldetPtr).two_theta & ptr_free,alldetPtr
    (*pState).ndet    = n_elements(*(*pState).tthPtr)
    *(*pState).detPtr = indgen((*pState).ndet)  ;default all detectors
    (*pState).det_tit = (['',(['spec ','diff ','spec+diff '])[(*pState).y_typ]])[(*pState).ftype eq 'MACS']+'det=all'
    (*pState).title   = (*pState).det_tit
    if strlen((*pState).tch_tit) ne 0 then (*pState).title=(*pState).title+', '+(*pState).tch_tit
    
    ;construct detector droplist
    if (*pState).ftype eq 'DCS' then begin ;DCS det bank info
       tmp   = string(*(*pState).tthPtr,format='(f8.2)')
       tmp   = string(indgen((*pState).ndet)+1,format='(i3)')+tmp+string('b0'xb)
       index = dm_mask_detbanks(0)
       tmp[index] = tmp[index]+'  c' & index=-1  ;central bank
       index = dm_mask_detbanks(1)
       tmp[index] = tmp[index]+'  u' & index=-1  ;upper bank
       index = dm_mask_detbanks(-1)
       tmp[index] = tmp[index]+'  l' & index=-1  ;lower bank
    endif else begin
       tmp   = string(indgen((*pState).ndet)+1,format='(i'+dm_to_string(ceil(alog10((*pState).ndet)))+')')+'   '+dm_to_string(*(*pState).tthPtr,res=3)+string('b0'xb)
    endelse
    det_info = [det_info,tmp]
    
    if ~keyword_set(init) then begin
       widget_control,(*pState).tlb,tlb_set_title=(*pState).ftype+([' Single Crystal',''])[(*pState).ftype eq 'MACS']+' Alignment Utility'
       widget_control,widget_info((*pState).tlb,find_by_uname='calca2but'),set_value='Calculate '+(*(*pState).psiname)[iftype]+' Reset Value'
       widget_control,widget_info((*pState).tlb,find_by_uname='calca2rot'),set_value='Calculate '+(*(*pState).psiname)[iftype]+' Rotation Angle'
       dm_set_droplist,widget_info((*pState).tlb,find_by_uname='detlist'),select=0,set_value=det_info
       widget_control,widget_info((*pState).tlb,find_by_uname='timelabel'),set_value=(['Time Channel','Det. Type'])[(*pState).ftype eq 'MACS'],sensitive=((*pState).ftype ne 'WAND')
       dm_set_droplist,widget_info((*pState).tlb,find_by_uname='timelist'),select=([(*pState).t_chan,(*pState).y_typ])[(*pState).ftype eq 'MACS'],$
          set_value=([['All','Elastic','Specify',' '],['SPEC','DIFF','SPEC+DIFF','Monitor']])[0:([2,3])[(*pState).ftype eq 'MACS'],((*pState).ftype eq 'MACS')],sensitive=((*pState).ftype ne 'WAND')
       dm_set_droplist,widget_info((*pState).tlb,find_by_uname='xaxislist'),select=(*pState).xaxis,set_value=x_info       
       ptr_free,(*pState).xPtr,(*pState).yPtr,(*pState).dyPtr,(*pState).bgPtr,(*pState).dbgPtr
       (*pState).xPtr   = ptr_new(/allocate_heap)  
       (*pState).yPtr   = ptr_new(/allocate_heap)
       (*pState).dyPtr  = ptr_new(/allocate_heap)
       (*pState).bgPtr  = ptr_new(/allocate_heap)
       (*pState).dbgPtr = ptr_new(/allocate_heap)
       (*pState).filesel->set_filter,(*(*pState).fextn)[iftype]
       (*pState).plotWin->erase
    endif 
end

pro dcs_cryst_align_setfilehistory,pState,dir,files,choose=choose,ftpobj=ftpobj
    (*pState).fileSel->getproperty,ftpserver=ftpserver
    if n_elements(choose) eq 1 then begin
       ftype = strmid((*((*pState).file_hist[choose]))[0],6)
       if ftype ne (*pState).ftype then begin
          wid = widget_info((*pState).tlb,find_by_uname='ftype_'+ftype)
          dcs_cryst_align_changeftype,{WIDGET_BUTTON,ID:wid,TOP:(*pState).tlb,HANDLER:(*pState).tlb,SELECT:1}
       endif
       path = (*((*pState).file_hist[choose]))[2]
       tmp = stregex(path,'ftp://'+ftpserver+'(.+)',/subexpr,/extract)
       if strlen(tmp[1]) gt 0 then path=tmp[1]
       (*pState).filesel->set_path,path,ftp=(strlen(tmp[1]) gt 0)
       (*pState).filesel->getproperty,dFiles=dFiles
       for i=3,n_elements(*((*pState).file_hist[choose]))-1 do begin
           ind = where(dFiles eq (*((*pState).file_hist[choose]))[i],count)
           if count ne 0 then begin
              if n_elements(selected) eq 0 then selected=ind $
              else selected=[selected,ind]
           endif
       endfor
       if n_elements(selected) ne 0 then (*pState).filesel->set_file,selected,/highlight
       return
    endif
    num = n_elements((*pState).file_hist)    ;capacity
    nfiles = n_elements(files)
    if (n_elements(dir) ne 0) and (nfiles ne 0) then begin
       if obj_valid(ftpobj) then dir1 = 'ftp://'+ftpserver+dir else dir1 = dir
       info = files[0] & if nfiles gt 1 then info = info+' to '+files[nfiles-1]
       for i=0,(*pState).n_history-1 do begin  ;check if this entry already exists, if existed move to the top
           if dir1 ne (*((*pState).file_hist[i]))[2] then continue
           if n_elements(files) ne n_elements(*((*pState).file_hist[i]))-3 then continue
           same = 1b
           for j=0, n_elements(files)-1 do begin
               if files[j] ne (*((*pState).file_hist[i]))[3+j] then same=0b
           endfor
           if same then begin ;entry exists, move that entry to the top
              if i ne 0 then begin
                 (*pState).file_hist[0:i] = shift((*pState).file_hist[0:i],1)
                 for j=0,i do widget_control,widget_info((*pState).tlb,find_by_uname='fhist_'+dm_to_string(j,/int)),set_value=dm_to_string(j,/int)+' '+(*((*pState).file_hist[j]))[1]
              endif
              return
           endif
       endfor
       if (*pState).n_history lt num then begin ;empty room available
          if (*pState).n_history ne 0 then $
             (*pState).file_hist[1:(*pState).n_history] = (*pState).file_hist[0:(*pState).n_history-1]
          (*pState).file_hist[0] = ptr_new(['ftype_'+(*pState).ftype,info,dir1,files])
          (*pState).n_history = (*pState).n_history+1
       endif else begin                    ;full
          ptr_free,(*pState).file_hist[num-1]
          (*pState).file_hist[1:num-1] = (*pState).file_hist[0:num-2]
          (*pState).file_hist[0] = ptr_new(['ftype_'+(*pState).ftype,info,dir1,files])
       endelse
    endif
    histmenu = widget_info((*pState).tlb,find_by_uname='fhistMenu')
    widget_control,histmenu,sensitive=((*pState).n_history gt 0)
    widget_control,widget_info((*pState).tlb,find_by_uname='clearHistory'),sensitive=((*pState).n_history gt 0)
    for i=0,num-1 do begin
        tmp = widget_info(histmenu,find_by_uname='fhist_'+dm_to_string(i,/int))
        if tmp ne 0 then widget_control,tmp,/destroy
    endfor
    for i=0,(*pState).n_history-1 do begin
        void = widget_button(histmenu,value=dm_to_string(i,/int)+' '+(*((*pState).file_hist[i]))[1],uname='fhist_'+dm_to_string(i))
    endfor
end

pro dcs_cryst_align_event,event,nofilehistory=nofilehistory
    compile_opt IDL2    ;,strictarrsubs
                        ;idl2=defint32,strictarr
                        ;strictarr:   [] to index array
                        ;strictarrsubs: error when out-of-range indices,IDL5.6 or after
    WIDGET_CONTROL,/HOURGLASS
    widget_control,event.handler,get_uvalue=pState

    if (*pState).enotebook then $
       IF DAVE_SET_FOCUS(EVENT) THEN RETURN

    widget_type = widget_info(event.id,/type)
    case widget_type of
       8:    tmp_value = widget_info(event.id,/droplist_select)  ;droplist
       12:   tmp_value = event.index                             ;combobox
       else: tmp_value = 0
    endcase

    ;catch and ignore all errors in this program
    catch, myerror
    if myerror ne 0 then begin
       ok = dialog_message(dialog_parent=(*pState).tlb,!error_state.msg,/error,/center)
       catch,/cancel
       if obj_valid(mesg) then obj_destroy,mesg
       return
    end
    
    tab    = string(9b)
    dgsym  = '!e !n\deg' ;string('b0'xb)
    uname  = strlowcase(widget_info(event.id,/uname))
    type   = strlowcase(tag_names(event,/structure))
    iftype = (where(*(*pState).fname eq (*pState).ftype))[0]
    psiname = (*(*pState).psiname)[iftype]
    case strmid(type,0,7) of
        'dm_plot': uname = type
        'dm_file': uname = type
        else:
    endcase
    
    ind = where(['loadbut','loadbgfile','macsalignment','macsalignmonchr','macsalignanalyzer','macsalignkidney','macsflipratio'] eq uname,count)
    if count ne 0 then begin
       date_str = strmid(dm_to_string(dm_to_number(systime(),/date)),0,8)  ;date string
       colrs = (*pState).plotWin->getcolor(/list,linecolor=colr0)  ;colr0 is the first line color
       (*pState).filesel->getproperty,path=open_path,file=open_file,ftpobj=ftpobj,ftpbufferdir=ftpbufferdir,sep=sep,dFiles=dFiles
       n_files = n_elements(open_file)
       if n_files le 0 then begin
          name = ([(*pState).ftype,'empty can','instrument alignment','monochromator alignment','two analyzer alignment','kidney center alignment','two flipping ratio'])[ind[0]]+' file'+(['s',''])[ind[0] eq 5]
          ok = dialog_message('Please select '+name+' first.',/error,dialog_parent=event.handler,/center)
          return
       endif else begin
          if (ind[0] eq 4) and (n_files ne 2) then begin
             ok = dialog_message('Please select two analyzer alignment files'+(['.',' only.'])[n_files gt 2],/error,dialog_parent=event.handler,/center)
             return 
          endif
          if (ind[0] eq 6) and (n_files ne 2) then begin
             ok = dialog_message('Please select two flipping ratio files'+(['.',' only.'])[n_files gt 2],/error,dialog_parent=event.handler,/center)
             return
          endif
       endelse
       selected = lonarr(n_files)
       if ~keyword_set(nofilehistory) then dcs_cryst_align_setfilehistory,pState,open_path,open_file,ftpobj=ftpobj
       tmp = file_basename(open_file,strmid(open_file[0],strpos(open_file[0],'.')))
       if n_elements(open_file) gt 1 then filename = tmp[0]+'-'+tmp[n_elements(tmp)-1] else filename = tmp[0]
    endif
    if stregex(uname,'^fit[_f]',/boolean) then begin
       funcid = [widget_info((*pState).tlb,find_by_uname='fit_gaussian'),widget_info((*pState).tlb,find_by_uname='fit_lorentzian'),widget_info((*pState).tlb,find_by_uname='fit_moffat')]
       termid = [widget_info((*pState).tlb,find_by_uname='fitfun_3'),widget_info((*pState).tlb,find_by_uname='fitfun_4'),widget_info((*pState).tlb,find_by_uname='fitfun_5'),$
                 widget_info((*pState).tlb,find_by_uname='fitfun_6')]
    endif
    
    if uname eq 'contextevent0' or uname eq 'contextevent1' then uname = 'contextevent'
    case uname of
       'tlb':    widget_control,event.id,scr_xsize=(*pState).geom[0],scr_ysize=(*pState).geom[1]
       'contextevent': if ~(*pState).fixfitwidth then begin
          menuid = widget_info(event.id,find_by_uname='contextmenu')
          dm_set_button,widget_info(menuid,find_by_uname='optnshowcursor'),(*pState).showcursor,onstring='Show Cursor Position',offstring='Hide Cursor Position'   
          dm_set_button,widget_info(menuid,find_by_uname='optnshowfit'),(*pState).showfit,onstring='Show Fitting Parameters',offstring='Hide Fitting Parameters'
          dm_set_button,widget_info(menuid,find_by_uname='optnfixwidth'),(*pState).fixfitwidth,onstring='Fix Fitting Width',offstring='Not Fix Fitting Width'
          funcid = [widget_info(menuid,find_by_uname='fit_gaussian'),widget_info(menuid,find_by_uname='fit_lorentzian'),widget_info(menuid,find_by_uname='fit_moffat')]
          termid = [widget_info(menuid,find_by_uname='fitfun_3'),widget_info(menuid,find_by_uname='fitfun_4'),widget_info(menuid,find_by_uname='fitfun_5'),widget_info(menuid,find_by_uname='fitfun_6')]
          dm_toggle_menubut,check=funcid[(*pState).fitfunchoice],uncheck=funcid[((*pState).fitfunchoice+[1,2]) mod 3]
          dm_toggle_menubut,check=termid[(*pState).fitfunnterm-3],uncheck=termid[((*pState).fitfunnterm-3+[1,2,3]) mod 4]
          funstr = (['C0*exp[-(x-C1)^2/(2*C2^2)]','C0/(1+(x-C1)^2/C2^2)','C0/(1+(x-C1)^2/C2^2)^C3'])[(*pState).fitfunchoice]
          if (*pState).fitfunchoice eq 2 then extstr = ['','+C4','+C4+C5*x','+C4+C5*x+C6*x^2'] else extstr = ['','+C3','+C3+C4*x','+C3+C4*x+C5*x^2']
          for i=0,3 do widget_control,termid[i],set_value=funstr+extstr[i]
          widget_control,termid[3],sensitive=(((*pState).fitfunchoice eq 0) and (~(*pState).fixfitwidth))
          widget_displaycontextmenu,event.id,event.x,event.y,menuid   
          end
       'dm_plot_showcpos': begin
          (*pState).showcursor = event.status
          dm_set_button,widget_info((*pState).tlb,find_by_uname='optnshowcursor'),(*pState).showcursor,onstring='Show Cursor Position',offstring='Hide Cursor Position'
         end 
       'loadbut': begin
          (*pState).fit = 0
          e_range = (*pState).e_ran
          case (*pState).ftype of 
              'DCS': begin   ;DCS
                 dm_load_dcsdata,open_path,open_file,ang=x,qty=y,dqty=dy,t_chan=(*pState).t_chan,e_range=e_range,monrate=monitor,$
                      parent=(*pState).tlb,duration=duration,/raw,eadjust=1,montype=1,/fissionchamber,bitflip=(*pState).bitflip,ftpobj=ftpobj,ftpbufferdir=ftpbufferdir  
                 (*pState).dcs_monitor[0] = monitor
                 end
              'MACS': begin  ;MACS
                 for i=0L,n_files-1L do begin
                     file = open_path+sep+open_file[i]
                     if i eq 0 then $
                        mesg =  obj_new('dm_progress',title='Loading data',message='Loading '+file,group_leader=(*pState).tlb) $
                     else $
                        mesg->update,message='loading '+file,title='Loading...  '+strtrim(string(i+1),2)+'/'+strtrim(string(n_files),2)
                     if obj_valid(ftpobj) then begin
                        ok = ftpobj->GetFileContent(file,localfilename=ftpbufferdir+path_sep()+'ftptmp'+open_file[i])
                        file = ftpbufferdir+path_sep()+'ftptmp'+open_file[i]
                     endif
                     ;read the macs file
                     dm_load_macs,file,data,colinfo,maspacing=maspacing,deteff=deteff,header=header,latticeparm=latticeparm,latticeori=latticeori,error=error,group_leader=(*pState).tlb
                     (*pState).macs_deteff[0:1,*] = deteff
                     (*pState).macs_maspacing = maspacing
                     if total(finite((*pState).macs_deteff[2:3,*],/nan)) ne 0 then (*pState).macs_deteff[2:3,*] = deteff
                     if obj_valid(ftpobj) then file_delete,file,/ALLOW_NONEXISTENT,/NOEXPAND_PATH,/QUIET
                     if keyword_set(error) then begin
                        if obj_valid(mesg) then obj_destroy,mesg
                        if n_elements(x) eq 0 then return else break
                     endif
                     if n_elements(data) eq 0 then continue
                     n_data  = n_elements(data[*,0])
                     ind_qty = lonarr(2,21)
                     for j=1,20 do begin
                         tmp = string(j,format='(i02)')
                         ind_qty[0,j-1] = (where(colinfo eq 'spec'+tmp))[0]
                         ind_qty[1,j-1] = (where(colinfo eq 'diff'+tmp))[0]
                     endfor
                     ind_qty[0,20] = (where(colinfo eq 'spec'))[0]
                     ind_qty[1,20] = (where(colinfo eq 'diff'))[0]
                     ind_hs = (where(colinfo eq 'h',cnt_hs))[0]
                     ind_ks = (where(colinfo eq 'k',cnt_ks))[0]
                     ind_ls = (where(colinfo eq 'l',cnt_ls))[0]
                     ind_en = (where(colinfo eq 'e',cnt_en))[0]
                     ind_a1 = (where(colinfo eq 'a1',cnt_a1))[0]
                     ind_a2 = (where(colinfo eq 'a2',cnt_a2))[0]
                     ind_a3 = (where(colinfo eq 'a3',cnt_a3))[0]
                     ind_a4 = (where(colinfo eq 'a4',cnt_a4))[0]
                     ind_a5 = (where(colinfo eq 'a5',cnt_a5))[0]
                     ind_a6 = (where(colinfo eq 'a6',cnt_a6))[0]
                     ind_pt = (where(colinfo eq 'ptai',cnt_pt))[0]
                     ind_df = (where(colinfo eq 'dfm',cnt_df))[0]
                     ind_mr = (where(colinfo eq 'monrot',cnt_mr))[0]
                     ind_mt = (where(colinfo eq 'montrans',cnt_mt))[0]
                     ind_ds = (where(colinfo eq 'dfmdts',cnt_ds))[0]
                     ind_mb = (where(colinfo eq 'mbtslide',cnt_mb))[0]
                     ind_bs = lonarr(21)
                     ind_as = lonarr(20)
                     for j=1,21 do begin
                         tmp = string(j,format='(i02)')
                         ind_bs[j-1] = (where(colinfo eq 'monblade'+tmp,cnt_bs))[0]
                         if j le 20 then ind_as[j-1] = (where(colinfo eq 'analyzertheta'+tmp,cnt_as))[0]
                     endfor
                     ind_sx = (where(colinfo eq 'smplx',cnt_sx))[0]
                     ind_sy = (where(colinfo eq 'smply',cnt_sy))[0]
                     ind_sz = (where(colinfo eq 'smplz',cnt_sz))[0]
                     ind_sl = (where(colinfo eq 'smplltilt',cnt_sl))[0]
                     ind_su = (where(colinfo eq 'smplutilt',cnt_su))[0]
                     ind_dm = (where(colinfo eq 'dmbt',cnt_dm))[0]
                     ind_fc = (where(colinfo eq 'focus',cnt_fc))[0]
                     ind_b1 = (where(colinfo eq 'beta1',cnt_b1))[0]
                     ind_b2 = (where(colinfo eq 'beta2',cnt_b2))[0]
                     ind_vh = (where(colinfo eq 'vbah',cnt_vh))[0]
                     ind_vv = (where(colinfo eq 'vbav',cnt_vv))[0]
                     ind_kn = (where(colinfo eq 'kidney',cnt_kn))[0]
                     ind_wt = (where(colinfo eq 'monitor'))[0] 
                     ind_te = (where(colinfo eq 'time'))[0]
                     ind_t1 = (where(colinfo eq 'temp',cnt_t1))[0]
                     ind_t2 = (where(colinfo eq 'temperaturecontrolreading',cnt_t2))[0]
                     ind_t3 = (where(colinfo eq 'temperaturesetpoint',cnt_t3))[0]
                     ind_hf = (where(colinfo eq 'magfield',cnt_hf))[0]
                     ind_ts = (where(colinfo eq 'timestamp',cnt_ts))[0]
                     if cnt_hf eq 0 then ind_hf = (where(colinfo eq 'hfield',cnt_hf))[0]
                     if cnt_hs gt 0 then hs = data[*,ind_hs] else hs = fltarr(n_data)
                     if cnt_ks gt 0 then ks = data[*,ind_ks] else ks = fltarr(n_data)
                     if cnt_ls gt 0 then ls = data[*,ind_ls] else ls = fltarr(n_data)
                     if cnt_en gt 0 then en = data[*,ind_en] else en = fltarr(n_data)
                     if cnt_a1 gt 0 then a1 = data[*,ind_a1] else a1 = fltarr(n_data)
                     if cnt_a2 gt 0 then a2 = data[*,ind_a2] else a2 = fltarr(n_data)
                     if cnt_a3 gt 0 then a3 = data[*,ind_a3] else a3 = fltarr(n_data)
                     if cnt_a4 gt 0 then a4 = data[*,ind_a4] else a4 = fltarr(n_data)
                     if cnt_a5 gt 0 then a5 = data[*,ind_a5] else a5 = fltarr(n_data)
                     if cnt_a6 gt 0 then a6 = data[*,ind_a6] else a6 = fltarr(n_data)
                     if cnt_pt gt 0 then pt = data[*,ind_pt] else pt = fltarr(n_data)
                     if cnt_df gt 0 then df = data[*,ind_df] else df = fltarr(n_data)
                     if cnt_mr gt 0 then mr = data[*,ind_mr] else mr = fltarr(n_data)
                     if cnt_mt gt 0 then mt = data[*,ind_mt] else mt = fltarr(n_data)
                     if cnt_ds gt 0 then ds = data[*,ind_ds] else ds = fltarr(n_data)
                     if cnt_mb gt 0 then mb = data[*,ind_mb] else mb = fltarr(n_data)
                     if cnt_bs gt 0 then bs = data[*,ind_bs] else bs = fltarr(n_data,21)
                     if cnt_as gt 0 then as = data[*,ind_as] else as = fltarr(n_data,20)
                     if cnt_sx gt 0 then sx = data[*,ind_sx] else sx = fltarr(n_data)
                     if cnt_sy gt 0 then sy = data[*,ind_sy] else sy = fltarr(n_data)
                     if cnt_sz gt 0 then sz = data[*,ind_sz] else sz = fltarr(n_data)
                     if cnt_sl gt 0 then sl = data[*,ind_sl] else sl = fltarr(n_data)
                     if cnt_su gt 0 then su = data[*,ind_su] else su = fltarr(n_data)
                     if cnt_dm gt 0 then dm = data[*,ind_dm] else dm = fltarr(n_data)
                     if cnt_fc gt 0 then fc = data[*,ind_fc] else fc = fltarr(n_data)
                     if cnt_b1 gt 0 then b1 = data[*,ind_b1] else b1 = fltarr(n_data)
                     if cnt_b2 gt 0 then b2 = data[*,ind_b2] else b2 = fltarr(n_data)
                     if cnt_vh gt 0 then vh = data[*,ind_vh] else vh = fltarr(n_data)
                     if cnt_vv gt 0 then vv = data[*,ind_vv] else vv = fltarr(n_data)
                     if cnt_kn gt 0 then kn = data[*,ind_kn] else kn = fltarr(n_data)
                     if cnt_hf gt 0 then hf = data[*,ind_hf] else hf = fltarr(n_data)
                     if cnt_t1 gt 0 then t1 = data[*,ind_t1] else t1 = fltarr(n_data)
                     if cnt_t2 gt 0 then t2 = data[*,ind_t2] else t2 = fltarr(n_data)
                     if cnt_t3 gt 0 then t3 = data[*,ind_t3] else t3 = fltarr(n_data)
                     if cnt_ts gt 0 then ts = data[*,ind_ts] else ts = fltarr(n_data)
                     qty = transpose([[[data[*,ind_qty[0,*]]]],[[data[*,ind_qty[1,*]]]]])
                     err = sqrt(qty)
                     if n_elements(moncnt) eq 0 then moncnt = data[0,ind_wt]
                     nstr = [['mg','be','pg']+'filmon']
                     cnt_cfx = 0
                     for j=0,n_elements(nstr)-1 do begin
                         ind_cfx = (where(colinfo eq nstr[j],count))[0]
                         if count ne 0 then begin
                            if finite(data[0,ind_cfx]) then begin
                               if round(data[0,ind_cfx]) then begin
                                  cnt_cfx = j+1
                                  break
                               endif
                            endif
                         endif
                     endfor
                     if cnt_cfx gt 0 then cfx = cnt_cfx*fix(round(data[*,ind_cfx])) else cfx = intarr(n_data)
                     if n_elements(x) eq 0 then begin
                        duration = moncnt 
                        ;figure out the monitor is time or monitor count
                        yn = stregex(header, '#reference',/boolean,/fold_case)
                        index = where(yn,count)
                        if count gt 0 then begin
                           tmp = strlowcase(strsplit(header[index[0]],' '+tab,/extract))
                           if n_elements(montype) eq 0 then begin
                              if tmp[n_elements(tmp)-1] eq 'time'    then montype = 0
                              if tmp[n_elements(tmp)-1] eq 'monitor' then montype = 1
                           endif
                           index = where(colinfo eq tmp[n_elements(tmp)-1],count)
                           if (count gt 0) and (tmp[n_elements(tmp)-1] eq 'time') then begin
                              ind_tm = where(colinfo eq 'time')
                              duration = -abs(data[0,ind_tm[0]]) ;negative value to denote time
                           endif
                        endif
                        x  = transpose([[hs],[ks],[ls],[en],[a3],[a4],[pt],[kn],[a1],[a2],[a5],[a6],[df],[mr],[mt],[ds],[mb],[bs],[as],[sx],[sy],[sz],[sl],[su],[dm],[fc],[b1],[b2],[vh],[vv],$
                              [hf],[t1],[t2],[t3],[ts],[cfx],[data[*,ind_te]],[data[*,ind_wt]]])
                        y  = qty
                        dy = err
                        (*(*pState).infoPtr)[1:12] = [latticeparm,latticeori]
                     endif else begin
                        x  = [[x],[transpose([[hs],[ks],[ls],[en],[a3],[a4],[pt],[kn],[a1],[a2],[a5],[a6],[df],[mr],[mt],[ds],[mb],[bs],[as],[sx],[sy],[sz],[sl],[su],[dm],[fc],[b1],[b2],[vh],$
                              [vv],[hf],[t1],[t2],[t3],[ts],[cfx],[data[*,ind_te]],[data[*,ind_wt]]])]]
                        y  = [[[y]],[[qty]]] 
                        dy = [[[dy]],[[err]]]
                     endelse
                     if (~ obj_valid(mesg)) then break
                 endfor
                 if obj_valid(mesg) then obj_destroy,mesg
                 if n_elements(x) eq 0 then break
                 ;check monitor counts
                 if (min(x[n_elements(x[*,0])-1,*]) lt 1000) and ((*pState).macs_normchoice eq 0)  and ~keyword_set(montype) then begin
                    ans = dialog_message(['Warning: Low monitor counts.','','Do you want to switch the intensity normalization choice from divided by monitor counts to divided by time?'],dialog_parent=(*pState).tlb,/question,/center)
                    if strupcase(ans) eq 'YES' then begin
                       (*pState).macs_normchoice = 1
                       dm_toggle_menubut,check=widget_info((*pState).tlb,find_by_uname='norm_time'),uncheck=widget_info((*pState).tlb,find_by_uname='norm_monitor')
                    endif
                 endif
                 if keyword_set(montype) and ((*pState).macs_normchoice eq 1) then begin
                    ans = dialog_message(['The intensity normalization choice is divided by time.','Do you want to switch to divided by monitor counts?'],dialog_parent=(*pState).tlb,/question,/center)
                    if strupcase(ans) eq 'YES' then begin
                       (*pState).macs_normchoice = 0
                       dm_toggle_menubut,check=widget_info((*pState).tlb,find_by_uname='norm_monitor'),uncheck=widget_info((*pState).tlb,find_by_uname='norm_time')
                    endif
                 endif
                 ;figure out scanning which device 
                 ind = where(stregex(header, '#scan +[0-9]+ +',/boolean,/fold_case),count)
                 if count eq 1 then begin
                    tmp = strsplit(header[ind],/extract)
                    tmp = strlowcase(tmp[n_elements(tmp)-1])
                    if  tmp eq 'kidney' then tmp = 'a4'  
                    wid = widget_info((*pState).tlb,find_by_uname='xaxislist')
                    widget_control,wid,get_value=list
                    list[n_elements(list)-1] = 'temp'
                    ind = where(strlowcase(list) eq tmp,count)
                    if (count eq 1) and ((*pState).xaxis ne ind) then begin
                       (*pState).xaxis = ind
                       dm_set_droplist,wid,select=ind
                    endif
                    wid = widget_info((*pState).tlb,find_by_uname='detlist')
                    ind = where(stregex(header, '#signal +[0-9]+ +monitor',/boolean,/fold_case),count)
                    tmp = strmid(tmp,0,3)
                    if (count eq 1) and ((tmp eq 'mon') or (tmp eq 'dfm')) then begin  ;monitor as the detector
                       dm_set_droplist,wid,select=2
                       *(*pState).detPtr = -1
                       (*pState).det_tit = 'det=monitor'
                    endif else begin          ;ptai as the detector #
                       dm_set_droplist,wid,select=3
                       *(*pState).detPtr = -2
                       (*pState).det_tit = (['spec ','diff ','spec+diff '])[(*pState).y_typ]+'det=ptai'
                    endelse
                    (*pState).title = (*pState).det_tit
                    dm_set_droplist,widget_info((*pState).tlb,find_by_uname='timelist'),select=([(*pState).y_typ,3])[(*(*pState).detPtr)[0] eq -1] 
                    widget_control,widget_info((*pState).tlb,find_by_uname='fitbut'),sensitive=(~(*pState).indvdet or (n_elements(*(*pState).detPtr) eq 1))
                 endif
                 end   
              'WAND': begin  ;WAND
                 for i=0L,n_files-1L do begin
                     file = open_path+sep+open_file[i]
                     if i eq 0 then $
                        mesg =  obj_new('dm_progress',title='Loading data',message='Loading '+file,group_leader=(*pState).tlb) $
                     else $
                        mesg->update,message='loading '+file,title='Loading...  '+strtrim(string(i+1),2)+'/'+strtrim(string(n_files),2)
                     ;read the wand file
                     dm_load_wand,file,data=data,psi=psi,temperature=temperature,error=error,group_leader=(*pState).tlb
                     if keyword_set(error) then begin
                        if obj_valid(mesg) then obj_destroy,mesg
                        if n_elements(x) eq 0 then return else break
                     endif
                     mask = where(data[*,0] lt 0,mskcount)
                     if mskcount gt 0 then data[mask,*] = 0.0
                     if i eq 0 then begin
                        x = transpose([[psi],[temperature]])
                        y = data 
                     endif else begin
                        x = [[x],[transpose([[psi],[temperature]])]]
                        y = [[y],[data]] 
                     endelse
                     if (~ obj_valid(mesg)) then break
                 endfor
                 if obj_valid(mesg) then obj_destroy,mesg
                 if n_elements(y) ne 0 then dy = sqrt(y)
                 end
              else:
          endcase
          if n_elements(x) eq 0 then begin
             ok = dialog_message(([open_file[0]+' contains','The files contain'])[n_files gt 1]+' no data.',dialog_parent=event.handler,/center,title='Empty data')
             return
          endif
          if n_elements(e_range) ne 0 then (*pState).e_ran = e_range
          *(*pState).xPtr  = temporary(x)
          *(*pState).yPtr  = temporary(y)
          *(*pState).dyPtr = temporary(dy)
          (*pState).filename[0] = filename
          if n_elements(duration) ne 0 then (*pState).duration = duration
          dcs_cryst_align_plot,pState
         end
       'loadbgfile': begin
          if ((*pState).t_chan eq 1) and (n_elements(*(*pState).xPtr) eq 0) then begin
             ok = dialog_message('Please load sample data files first to get the same energy range for empty can files.',dialog_parent=(*pState).tlb,/center)
             return
          endif
          t_chan = (*pState).t_chan
          e_ran  = (*pState).e_ran
          if (t_chan eq 1) and (total(finite(e_ran,/nan)) eq 0) and (n_elements( *(*pState).yPtr) ne 0) then t_chan = 2 
          dm_load_dcsdata,open_path,open_file,ang=x,qty=y,dqty=dy,t_chan=t_chan,e_range=e_ran,monrate=monitor,$
                      parent=(*pState).tlb,duration=duration,/raw,eadjust=1,montype=0,/fissionchamber,bitflip=(*pState).bitflip,ftpobj=ftpobj,ftpbufferdir=ftpbufferdir
          if n_elements(y) ne 0 then begin
             nfile = n_elements(y[0,*])
             (*pState).dcs_monitor[1] = monitor*nfile
             if nfile gt 1 then begin
                ind = where(y eq 0,count)
                if count ne 0 then begin
                   zero_error = min(abs(dy[ind]))
                   dy[ind] = 0
                endif
                y  = total(y,2)
                dy = sqrt(total(dy^2,2))  ;not quite accurate
                if count ne 0 then begin
                   ind = where(dy eq 0,count)
                   if count ne 0 then dy[ind] = zero_error
                endif
             endif
             *(*pState).bgPtr  = temporary(y)
             *(*pState).dbgPtr = temporary(dy)
             (*pState).filename[1] = filename
             widget_control,widget_info((*pState).mbar,find_by_uname='clearbgfile'),/sensitive
             if n_elements(*(*pState).yPtr) ne 0 then dcs_cryst_align_plot,pState
          endif 
         end  
       'clearbgfile': begin
          ptr_free,(*pState).bgPtr,(*pState).dbgPtr
          (*pState).bgPtr = ptr_new(/allocate_heap)
          (*pState).dbgPtr = ptr_new(/allocate_heap)
          widget_control,event.id,sensitive=0
          if n_elements(*(*pState).yPtr) ne 0 then dcs_cryst_align_plot,pState 
         end
       'optnbitflip': begin
          (*pState).bitflip = 1-(*pState).bitflip
          dm_set_button,event.id,(*pState).bitflip,onstring='Bitflip Error Correction Enabled',offstring='Bitflip Error Correction Disabled'
         end  
       'optnindvdet': begin
          (*pState).indvdet = 1-(*pState).indvdet
          dm_set_button,event.id,(*pState).indvdet,onstring='Show Individual Detector',offstring='Sum Detectors'
          widget_control,widget_info((*pState).tlb,find_by_uname='fitbut'),sensitive=(~(*pState).indvdet or (n_elements(*(*pState).detPtr) eq 1))
          dcs_cryst_align_plot,pState
         end
       'optnshowfit': begin
          (*pState).showfit = 1-(*pState).showfit
          dm_set_button,widget_info((*pState).tlb,find_by_uname='optnshowfit'),(*pState).showfit,onstring='Show Fitting Parameters',offstring='Hide Fitting Parameters'
          dcs_cryst_align_plot,pState
         end
       'optnfixwidth':begin
          (*pState).fixfitwidth = 1-(*pState).fixfitwidth
          dm_set_button,widget_info((*pState).tlb,find_by_uname='optnfixwidth'),(*pState).fixfitwidth,onstring='Fix Fitting Width',offstring='Not Fix Fitting Width'
          cnt  = widget_info((*pState).tlb,find_by_uname='center')
          csm0 = widget_info(cnt,find_by_uname='contextevent0')
          csm1 = widget_info(cnt,find_by_uname='contextevent1')
          kid0 = widget_info(csm0,/all_children)
          kid1 = widget_info(csm1,/all_children)
          for i=0,1 do begin
              kids = widget_info(([csm0,csm1])[i],/all_children)
              for j=0,n_elements(kids)-1 do begin
                  if kids[j] ne 0 then widget_control,kids[j],/destroy
              endfor
          endfor
          if (*pState).fixfitwidth then begin
             geom0 = widget_info(widget_info(cnt,find_by_uname='loadbut'),/geometry)
             geom1 = widget_info(cnt,/geometry)
             void = widget_label(csm0,value='Fixed Width (C2)',/align_center,scr_xsize=geom1.scr_xsize-2*geom1.xpad,uname='fixwidthlabel')
             void = widget_text(csm1,uname='fixwidth',/editable,scr_xsize=geom1.scr_xsize-2*geom1.xpad,frame=0)
             if !version.os_family eq 'unix' then begin
                wids = ['detlabel','detlist','timelabel','timelist','xaxislabel','xaxislist','fixwidthlabel',['load','fit','clpb','save','data','print']+'but']
                for i=0,n_elements(wids)-1 do widget_control,widget_info(cnt,find_by_uname=wids[i]),scr_ysize=geom0.scr_ysize
             endif
             if ((*pState).fitfunnterm eq 6) and ((*pState).fitfunchoice eq 0) then begin ;mpfit doesn't support nterm=6
                (*pState).fitfunnterm = 5
                dm_toggle_menubut,check=widget_info((*pState).tlb,find_by_uname='fitfun_5'),uncheck= widget_info((*pState).tlb,find_by_uname='fitfun_6')
                ok = dialog_message("Fixed width fitting currently doesn't work with nterm=6 for Gaussian. The Gaussian has been switched to nterm=5.",dialog_parent=event.handler,/center)
             endif
          endif else begin
             dcs_cryst_align_csm,cnt,pState,csms=[csm0,csm1]
          endelse
          widget_control,widget_info((*pState).tlb,find_by_uname='fitfun_6'),sensitive=(((*pState).fitfunchoice eq 0) and (~(*pState).fixfitwidth))
          dcs_cryst_align_plot,pState
         end   
       'optnshowfname': begin
          (*pState).showfname = 1-(*pState).showfname
          dm_set_button,event.id,(*pState).showfname,onstring='Show File Name',offstring='Hide File Name'
          if ~(*pState).showfname then filename = '' else begin
             filename = (*pState).filename[0]
             if n_elements(*(*pState).bgPtr) ne 0 then filename = '('+filename+')-('+(*pState).filename[1]+')'
          endelse
          (*pState).plotWin->setproperty,cornertxt=filename
         end  
       'optnshowcursor': begin
          (*pState).showcursor = 1-(*pState).showcursor
          dm_set_button,widget_info((*pState).tlb,find_by_uname='optnshowcursor'),(*pState).showcursor,onstring='Show Cursor Position',offstring='Hide Cursor Position'
          (*pState).plotWin->setproperty,showcursorpos=(*pState).showcursor
         end  
       'optnapplylamb2': begin
          (*pState).macs_lamb2 = 1-(*pState).macs_lamb2
          dm_set_button,event.id,(*pState).macs_lamb2,onstring='Apply Monitor Lambda/2 Correction',offstring='No Monitor Lambda/2 Correction'
          dcs_cryst_align_plot,pState
         end
       'norm_monitor':if (*pState).macs_normchoice ne 0 then begin
          (*pState).macs_normchoice = 0
          dm_toggle_menubut,check=event.id,uncheck=widget_info((*pState).tlb,find_by_uname='norm_time')
          dcs_cryst_align_plot,pState
         end
       'norm_time':if (*pState).macs_normchoice ne 1 then begin
           (*pState).macs_normchoice = 1
           dm_toggle_menubut,check=event.id,uncheck=widget_info((*pState).tlb,find_by_uname='norm_monitor')
           dcs_cryst_align_plot,pState
         end  
       'temp_samp': if (*pState).tchoice ne 0 then begin
          (*pState).tchoice = 0
          dm_toggle_menubut,check=event.id,uncheck=[widget_info((*pState).tlb,find_by_uname='temp_ctrl'),widget_info((*pState).tlb,find_by_uname='temp_setp')] 
          if (*pState).xaxis eq (n_elements(*(*pState).xtit)-1) then dcs_cryst_align_plot,pState
         end    
       'temp_ctrl': if (*pState).tchoice ne 1 then begin
          (*pState).tchoice = 1
          dm_toggle_menubut,check=event.id,uncheck=[widget_info((*pState).tlb,find_by_uname='temp_samp'),widget_info((*pState).tlb,find_by_uname='temp_setp')] 
          if (*pState).xaxis eq (n_elements(*(*pState).xtit)-1) then dcs_cryst_align_plot,pState
         end    
       'temp_setp': if (*pState).tchoice ne 2 then begin
          (*pState).tchoice = 2
          dm_toggle_menubut,check=event.id,uncheck=[widget_info((*pState).tlb,find_by_uname='temp_samp'),widget_info((*pState).tlb,find_by_uname='temp_ctrl')] 
          if (*pState).xaxis eq (n_elements(*(*pState).xtit)-1) then dcs_cryst_align_plot,pState
         end
       'fit_gaussian': if (*pState).fitfunchoice ne 0 then begin
          (*pState).fitfunchoice = 0
          dm_toggle_menubut,check=funcid[0],uncheck=funcid[1:2]
          for i=0,3 do widget_control,termid[i],set_value='C0*exp[-(x-C1)^2/(2*C2^2)]'+(['','+C3','+C3+C4*x','+C3+C4*x+C5*x^2'])[i],/sensitive
          widget_control,widget_info((*pState).tlb,find_by_uname='fitbut'),set_value='Gaussian Fit'
          dcs_cryst_align_plot,pState
         end  
       'fit_lorentzian': if (*pState).fitfunchoice ne 1 then begin
          (*pState).fitfunchoice = 1
          dm_toggle_menubut,check=funcid[1],uncheck=funcid[[0,2]]
          for i=0,3 do widget_control,termid[i],set_value='C0/(1+(x-C1)^2/C2^2)'+(['','+C3','+C3+C4*x','+C3+C4*x+C5*x^2'])[i],sensitive=(i ne 3)
          widget_control,widget_info((*pState).tlb,find_by_uname='fitbut'),set_value='Lorentzian Fit'
          if (*pState).fitfunnterm eq 6 then begin
             (*pState).fitfunnterm = 5
             dm_toggle_menubut,check=termid[2],uncheck=termid[3]
          endif
          dcs_cryst_align_plot,pState
         end    
       'fit_moffat': if (*pState).fitfunchoice ne 2 then begin
          (*pState).fitfunchoice = 2
          dm_toggle_menubut,check=funcid[2],uncheck=funcid[[0,1]]
          for i=0,3 do widget_control,termid[i],set_value='C0/(1+(x-C1)^2/C2^2)^C3'+(['','+C4','+C4+C5*x','+C4+C5*x+C6*x^2'])[i],sensitive=(i ne 3)
          widget_control,widget_info((*pState).tlb,find_by_uname='fitbut'),set_value='Moffat Fit'
          if (*pState).fitfunnterm eq 6 then begin
             (*pState).fitfunnterm = 5
             dm_toggle_menubut,check=termid[2],uncheck=termid[3]
          endif
          dcs_cryst_align_plot,pState
         end      
       'fitfun_3': if (*pState).fitfunnterm ne 3 then begin
          (*pState).fitfunnterm = 3
          dm_toggle_menubut,check=termid[0],uncheck=termid[1:3]
          dcs_cryst_align_plot,pState
         end       
       'fitfun_4': if (*pState).fitfunnterm ne 4 then begin
          (*pState).fitfunnterm = 4
          dm_toggle_menubut,check=termid[1],uncheck=termid[[0,2,3]]
          dcs_cryst_align_plot,pState
         end  
       'fitfun_5': if (*pState).fitfunnterm ne 5 then begin
          (*pState).fitfunnterm = 5
          dm_toggle_menubut,check=termid[2],uncheck=termid[[0,1,3]]
          dcs_cryst_align_plot,pState
         end  
       'fitfun_6': if (*pState).fitfunnterm ne 6 then begin
          (*pState).fitfunnterm = 6
          dm_toggle_menubut,check=termid[3],uncheck=termid[[0,1,2]]
          dcs_cryst_align_plot,pState
         end   
       'macs_eff_file':if (*pState).macs_eff ne 1 then begin 
          (*pState).macs_eff = 1
          dm_toggle_menubut,check=[event.id,widget_info((*pState).tlb,find_by_uname='macseffMenu')],uncheck=widget_info((*pState).tlb,find_by_uname='macs_eff_spec')
          dm_set_button,widget_info((*pState).tlb,find_by_uname='macs_eff_none'),0,/sensitive
          dcs_cryst_align_plot,pState
         end  
       'macs_eff_spec': begin 
          new = dm_dialog_listinput('det '+dm_to_string(1+indgen(20)),title='Please input new efficiencies:',default=(*pState).macs_deteff[2:3,*],$
                 label=['SPEC','DIFF'],info=['Intensity will be multiplied by these values.'],dialog_parent=(*pState).tlb)    
          ind =  where(finite(new,/nan),cnt)  
          if cnt ne 0 then begin
             ok = dialog_message((['Invalid input for '+(['SPEC ','DIFF '])[ind[0] mod 2]+dm_to_string(ind[0]/2+1)+'.','There are invalid inputs.'])[cnt gt 1],/error,dialog_parent=event.handler,/center)
             break
          endif
          (*pState).macs_deteff[2:3,*] = new
          (*pState).macs_eff = 2
          dm_toggle_menubut,check=[event.id,widget_info((*pState).tlb,find_by_uname='macseffMenu')],uncheck=widget_info((*pState).tlb,find_by_uname='macs_eff_file')
          dm_set_button,widget_info((*pState).tlb,find_by_uname='macs_eff_none'),0,/sensitive
          dcs_cryst_align_plot,pState
         end   
       'macs_eff_none':if (*pState).macs_eff ne 0 then begin 
          (*pState).macs_eff = 0
          dm_set_button,event.id,1,sensitive=0
          dm_toggle_menubut,uncheck=[widget_info((*pState).tlb,find_by_uname='macs_eff_file'),widget_info((*pState).tlb,find_by_uname='macs_eff_spec'),$
             widget_info((*pState).tlb,find_by_uname='macseffMenu')]
          dcs_cryst_align_plot,pState
         end
       'fitbut':  if n_elements(*(*pState).yPtr) gt (*pState).fitfunnterm then begin
          (*pState).fit = 1
          dcs_cryst_align_plot,pState
         end
       'savebut': if n_elements(*(*pState).xPtr) ne 0 then begin
          if dm_to_number(!version.release) ge 8.0 then $          
             ftypename = ['Postscript','PDF','JPEG','GIF','TIFF','BMP','PNG','PPM'] $
          else $
             ftypename = ['Postscript','JPEG','GIF','TIFF','BMP','PNG','PPM']
          ftype = dm_dialog_input('file type:',default=0,droplist_content=ptr_new(ftypename),is_droplist=[1],$
              /return_number,title='Please choose',dialog_parent=(*pState).tlb,xsize=80,cancel=cancel)
          if keyword_set(cancel) then return
          ftypename[0] = 'PS'    
          (*pState).plotWin->saveas,ftypename[ftype]
         end
       'databut': if n_elements(*(*pState).xPtr) ne 0 then begin
          path = (*pState).workDir
          file = dm_choose_file('txt',/write,dialog_parent=(*pState).tlb,title='Save plot data as an ASCII file.',path=path)
          if strlen(file) eq 0 then break
          if file_test(path,/directory) then (*pState).workDir=path
          openw,unit,file,/get_lun,error=openerr
          if openerr ne 0 then begin
             ok = dialog_message("Can't write in "+file[0],/error,dialog_parent=(*pState).tlb,/center)
          endif else begin
             dcs_cryst_align_plot,pState,/dataonly,xs=x,ys=y,dys=dy  ;get data
             widget_control,/hourglass  ;busy signal
             for i=0L,n_elements(x)-1L do $
                 printf,unit,x[i],' ',y[i],' ',dy[i]
             free_lun,unit
          endelse
         end
       'printbut': if n_elements(*(*pState).xPtr) ne 0 then (*pState).plotWin->saveas,'printer'
       'clpbbut':  if n_elements(*(*pState).xPtr) ne 0 then (*pState).plotWin->saveas,'clipboard'
       'exitbut': begin
          widget_control,event.handler,/destroy
          return
         end
       'detlist': begin
          old = *(*pState).detPtr
          (*pState).tth = [!values.f_nan,!values.f_nan]
          widget_control,event.id,get_value=det_list
          case tmp_value of
              0: begin  ;all
                 *(*pState).detPtr = indgen((*pState).ndet)
                 (*pState).det_tit = (['',(['spec ','diff ','spec+diff '])[(*pState).y_typ]])[(*pState).ftype eq 'MACS']+'det=all'     
                 end
              1: begin  ;specify
                 n_index = n_elements(*(*pState).detPtr)
                 maxdet  = max(*(*pState).detPtr,min=mindet)
                 if (n_index eq (maxdet-mindet+1)) and (n_index gt 3) then $
                    default = dm_to_string(mindet+1)+'-'+dm_to_string(maxdet+1) $
                 else begin
                    default = dm_to_string(*(*pState).detPtr+1,separator=',')
                    if ((*pState).ftype eq 'DCS') then begin
                       if ((*(*pState).detPtr)[0] eq 913) then default = 'FC2'
                       if ((*(*pState).detPtr)[0] eq 914) then default = 'FC3'
                       if n_elements((*(*pState).detPtr)) eq 294 then begin
                          if total(abs((*(*pState).detPtr)-dm_mask_detbanks(-1))) eq 0 then default = 'Lower Bank' $
                          else if total(abs((*(*pState).detPtr)-dm_mask_detbanks(1))) eq 0 then default = 'Upper Bank'
                       endif else if n_elements((*(*pState).detPtr)) eq 325 then begin
                          if total(abs((*(*pState).detPtr)-dm_mask_detbanks(0))) eq 0 then default = 'Central Bank'
                       endif
                    endif else if ((*pState).ftype eq 'MACS') then begin
                       if ((*(*pState).detPtr)[0] eq -1) then default = 'monitor'
                       if ((*(*pState).detPtr)[0] eq -2) then default = 'ptai'
                    endif
                 endelse
                 tmp = dm_dialog_input(dialog_parent=event.handler,'Det num:',default=default,title='Specifying detectors',$
                       info='Detector number starts from 1 to '+dm_to_string((*pState).ndet)+'. eg:'+(['371,374 or 370-374 c','6,7,8 or 6-8','120,121,122 or 120-122'])[iftype<2],xsize=200)
                 (*pState).det_tit = (['',(['spec ','diff ','spec+diff '])[(*pState).y_typ]])[(*pState).ftype eq 'MACS']+'det='+tmp
                 if strlen(tmp) ne 0 then begin
                    if ((*pState).ftype eq 'DCS') then begin
                       if strlowcase(tmp) eq 'central bank' then  tmp_det = 0
                       if strlowcase(tmp) eq 'lower bank'   then  tmp_det = -1
                       if strlowcase(tmp) eq 'upper bank'   then  tmp_det = 1
                       if strlowcase(tmp) eq 'fc2'          then  tmp_det = 913
                       if strlowcase(tmp) eq 'fc3'          then  tmp_det = 914
                       if n_elements(tmp_det) ne 0 then begin
                           (*pState).det_tit = (tmp_det gt 2)?((['FC2','FC3'])[tmp_det-913]):('det='+(['lower','central','upper'])[tmp_det+1])
                           dm_set_droplist,event.id,select=(tmp_det gt 2)?(5+tmp_det-913):(([4,2,3])[tmp_det+1])
                           *(*pState).detPtr = (tmp_det gt 2)?tmp_det:(dm_mask_detbanks(tmp_det))
                           break
                        endif
                    endif else if ((*pState).ftype eq 'MACS') then begin
                       if strlowcase(tmp) eq 'monitor' then  tmp_det = -1
                       if strlowcase(tmp) eq 'ptai'    then  tmp_det = -2
                       if n_elements(tmp_det) ne 0 then begin
                           (*pState).det_tit = ([(['spec ','diff ','spec+diff '])[(*pState).y_typ]+'det=ptai','det=monitor'])[2+tmp_det]
                           dm_set_droplist,event.id,select=n_elements(det_list)-23-tmp_det
                           *(*pState).detPtr = tmp_det
                           break
                        endif
                    endif
                    ;search for :- range detectors
                    len = [1]
                    det_tmp = -1L
                    while (len[0] gt 0) do begin
                        tmp1 = stregex(tmp,'([0-9]+)[ '+tab+']*[:-][ '+tab+']*([0-9]+)[ '+tab+']*([cCuUlL]*)',/subexpr,length=len)
                        if len[0] gt 0 then begin
                           tmp2 = dm_to_number(strmid(tmp,tmp1[1],len[1]),/long)
                           tmp3 = dm_to_number(strmid(tmp,tmp1[2],len[2]),/long)
                           if len[3] gt 0 then status = strlowcase(strmid(tmp,tmp1[3],1)) else status=''
                           tmp4 = min([tmp2,tmp3])+lindgen(abs(tmp3-tmp2)+1)
                           if status ne '' then $
                              tmp4 = dm_common(tmp4,dm_mask_detbanks((where(['l','c','u'] eq status))[0]-1)+1,/no_copy)
                           det_tmp = [det_tmp,tmp4]
                           tmp = strmid(tmp,0,tmp1[0])+strmid(tmp,tmp1[0]+len[0],strlen(tmp)-tmp1[0]-len[0])
                        endif
                    endwhile 
                    ;individual detectors
                    tmp1 = strsplit(tmp,'[a-zA-Z '+tab+',:-]',/regex,/extract)
                    tmp2 = dm_to_number(tmp1)
                    ind2 = where(finite(tmp2),count2)
                    if count2 gt 0 then det_tmp = [det_tmp,dm_to_number(tmp1[ind2],/long)]
                    index = where(det_tmp gt 0,count)
                    if count gt 0 then dets = det_tmp[index]-1
                 endif  
                 ndet = n_elements(dets)
                 if ndet gt 0 then dets = dets[uniq(dets,sort(dets))]
                 if n_elements(dets) ne ndet then (*pState).det_tit = (['',(['spec ','diff ','spec+diff '])[(*pState).y_typ]])[(*pState).ftype eq 'MACS']+'det='+dm_to_string(dets+1,separator=',')
                 ndet = n_elements(dets)
                 if (ndet eq 0) or (ndet eq (*pState).ndet) then begin
                    *(*pState).detPtr = indgen((*pState).ndet)
                    (*pState).det_tit = (['',(['spec ','diff ','spec+diff '])[(*pState).y_typ]])[(*pState).ftype eq 'MACS']+'det=all'
                    dm_set_droplist,event.id,select=0
                 endif else begin
                    *(*pState).detPtr = dets
                    maxdet = max(dets,min=mindet)
                    if (ndet eq (maxdet-mindet+1)) and (ndet gt 3) then $
                       (*pState).det_tit = (['',(['spec ','diff ','spec+diff '])[(*pState).y_typ]])[(*pState).ftype eq 'MACS']+'det='+dm_to_string(mindet+1)+'-'+dm_to_string(maxdet+1)
                    (*pState).tth[0] = mean((*(*pState).tthPtr)[dets])
                    tth = dm_to_string((*pState).tth[0],resolution=3)+dgsym
                    if ndet gt 1 then tth = '(<2th>='+tth+')' else tth='(2th='+tth+')'
                    (*pState).det_tit = (*pState).det_tit+tth
                 endelse
                 end
           else: begin
                 case (*pState).ftype of
                      'DCS':  begin
                              if tmp_value le 4 then *(*pState).detPtr = dm_mask_detbanks(([0,1,-1])[tmp_value-2]) $
                              else if tmp_value le 6 then  *(*pState).detPtr = ([913,914])[tmp_value-5] $
                              else *(*pState).detPtr = tmp_value-7
                              if tmp_value le 6 then (*pState).det_tit = (['det='+['central','upper','lower'],'FC2','FC3'])[tmp_value-2] $
                              else begin
                                 (*pState).tth[0]  = (*(*pState).tthPtr)[tmp_value-7]
                                 tth = dm_to_string((*pState).tth[0])+dgsym
                                 (*pState).det_tit = 'det='+strtrim(string(tmp_value-6),2)+'(2th='+tth+')'
                              endelse
                              end
                      'MACS': begin
                              if tmp_value lt 4 then begin
                                 *(*pState).detPtr = ([-1,-2])[tmp_value-2]  ;monitor and ptai
                                 (*pState).det_tit = (['det=monitor',(['spec ','diff ','spec+diff '])[(*pState).y_typ]+'det=ptai'])[tmp_value-2]
                              endif else begin
                                 *(*pState).detPtr = tmp_value-4
                                 (*pState).det_tit = (['spec ','diff ','spec+diff '])[(*pState).y_typ]+'det='+strtrim(string(tmp_value-3),2)
                              endelse 
                              end
                      else:   begin
                              *(*pState).detPtr = tmp_value-2
                              (*pState).tth[0]  = (*(*pState).tthPtr)[tmp_value-2]
                              tth = dm_to_string((*pState).tth[0])+dgsym
                              (*pState).det_tit = (['',(['spec ','diff ','spec+diff '])[(*pState).y_typ]])[(*pState).ftype eq 'MACS']+'det='+strtrim(string(tmp_value-1),2)+'(2th='+tth+')'
                              end
                 endcase
                 end
          endcase
          if (*pState).ftype eq 'DCS' then $
             (*pState).title = (*pState).det_tit+', '+(*pState).tch_tit $
          else begin
             (*pState).title = (*pState).det_tit
             if (*pState).ftype eq 'MACS' then begin
                dm_set_droplist,widget_info((*pState).tlb,find_by_uname='timelist'),select=([(*pState).y_typ,3])[(*(*pState).detPtr)[0] eq -1] 
             endif
          endelse
          n_old = n_elements(old)
          n_new = n_elements(*(*pState).detPtr)
          comm  = dm_common(old,*(*pState).detPtr)
          same  = (n_old eq n_new) and (n_old eq n_elements(comm)) and (comm[0] ne -1)
          old   = -1 & comm = -1
          widget_control,widget_info((*pState).tlb,find_by_uname='fitbut'),sensitive=(~(*pState).indvdet or (n_elements(*(*pState).detPtr) eq 1))
          if (n_elements(*(*pState).yPtr) ne 0) and (~ same) then dcs_cryst_align_plot,pState   ;reload data
         end
       'timelist': if (((*pState).ftype eq 'DCS') and (((*pState).t_chan ne tmp_value) or (tmp_value eq 2))) or ((*pState).ftype eq 'MACS') then begin
          if (*pState).ftype eq 'DCS' then begin
             (*pState).t_chan = tmp_value
             if (*pState).t_chan eq 0 then (*pState).tch_tit='tchan=all' $
             else if (*pState).t_chan eq 1 then (*pState).tch_tit='tchan=elastic' $
             else begin
                default = dm_to_string((*pState).e_ran)
                new     = dm_dialog_input(['from:','to:'],title='Energy range',xsize=100,default=default,/float,$
                          dialog_parent=(*pState).tlb,info='In units of meV')
                (*pState).t_chan = 1
                (*pState).e_ran[*] = !values.f_nan
                if total(finite(new)) eq 2 then $
                   if new[1] gt new[0] then begin
                      (*pState).e_ran   = new
                      (*pState).t_chan  = 2
                      (*pState).tch_tit = 'E=['+dm_to_string(new[0])+','+dm_to_string(new[1])+']meV'
                   endif
                if (*pState).t_chan eq 1 then begin
                   ok = dialog_message('Invalid energy range. Elastic peak option is used.',dialog_parent=(*pState).tlb,/center)
                   dm_set_droplist,event.id,select=1
                   (*pState).tch_tit = 'tchan=elastic'
                endif
             endelse
             (*pState).title = (*pState).det_tit+', '+(*pState).tch_tit
             (*pState).filesel->getproperty,file=open_file
             if n_elements(open_file) ne 0 then begin ;reload data
                tlb  = (*pState).tlb
                load = widget_info((*pState).tlb,find_by_uname='loadbut')
                dcs_cryst_align_event,{WIDGET_BUTTON,ID:load,TOP:tlb,HANDLER:tlb,SELECT:1},/nofilehistory
             endif
             if n_elements(*(*pState).bgPtr) ne 0 then begin
                ptr_free,(*pState).bgPtr,(*pState).dbgPtr
                (*pState).bgPtr = ptr_new(/allocate_heap)
                (*pState).dbgPtr = ptr_new(/allocate_heap)
                widget_control,widget_info((*pState).mbar,find_by_uname='clearbgfile'),sensitive=0
                ok = dialog_message('Please reload the empty can file.',dialog_parent=(*pState).tlb,/center)
             endif
          endif else if (*pState).ftype eq 'MACS' then begin
             if tmp_value eq 3 then begin
                *(*pState).detPtr = -1
                (*pState).det_tit = 'det=monitor'
                (*pState).title   = (*pState).det_tit
                dm_set_droplist,widget_info((*pState).tlb,find_by_uname='detlist'),select=2
             endif else begin
                (*pState).y_typ = tmp_value
                if (*(*pState).detPtr)[0] eq -1 then begin
                   *(*pState).detPtr = -2  ;ptai
                   (*pState).det_tit = (['spec ','diff ','spec+diff '])[(*pState).y_typ]+'det=ptai'
                   (*pState).title   = (*pState).det_tit
                   dm_set_droplist,widget_info((*pState).tlb,find_by_uname='detlist'),select=3
                endif else begin     
                   str1 = (['(spec *\+ *)*diff','spec( *\+ *diff)*','(spec )*(diff *)*'])[(*pState).y_typ]
                   str2 = (['spec','diff','spec+diff '])[(*pState).y_typ]
                   tmp  = stregex((*pState).title,str1,/fold_case,len=len)
                   if len gt 0 then (*pState).title = strmid((*pState).title,0,tmp)+str2+strmid((*pState).title,tmp+len)
                endelse
             endelse
             dcs_cryst_align_plot,pState
          endif
         endif
       'xaxislist':if (*pState).xaxis ne tmp_value then begin
          (*pState).xaxis = tmp_value
          (*pState).fit   = 0
          if n_elements(*(*pState).xPtr) ne 0 then dcs_cryst_align_plot,pState
         end
       'calctthbut':begin
          default = (float(*(*pState).infoPtr))[0:9]
          tmpname = [(['lambda ('+string('c5'XB)+')','E (meV)'])[(*pState).ftype eq 'MACS'],['a','b','c']+' ('+string('c5'XB)+')','alpha','beta','gamma','H','K','L']+':'
          if (*pState).ftype eq 'MACS' then begin
             if finite(default[0]) then default[0] = (81.8042d)/default[0]/default[0] ;macs, use energy
          endif
          info = dm_dialog_input(tmpname,default=default,/float,dialog_parent=(*pState).tlb,xsize=80,title='Calculate 2theta',e2lambda=((*pState).ftype eq 'MACS'),lambda2e=((*pState).ftype ne 'MACS'),cancel=cancel)
          if keyword_set(cancel) then return
          if total(finite(info)) eq n_elements(default) then begin
             if (*pState).ftype eq 'MACS' then begin
                kidlim  = dm_getkidneyrange(info[0])  ;kidney range
                info[0] = sqrt((81.8042d)/info[0])
             endif
             (*(*pState).infoPtr)[0:9] = double(info[0:9])
             Ei  = (81.8042d)/info[0]/info[0]
             q   = dcs_cryst_align_qlength1(info[7:9],info[1:6])
             tth = dcs_cryst_align_2theta1(Ei,info[7:9],info[1:6])
             if finite(tth) then begin
                if (*pState).ftype eq 'MACS' then begin
                   det_l = (([tth,-tth]-kidlim[0]+76.)/8)<19
                   det_u = (([tth,-tth]-kidlim[1]+76.)/8)>0 
                   ptais = [-1,-1]
                   if det_l[0] ge det_u[0] then ptais[0] = 1>(round((det_l[0]+det_u[0])/2)+1)<20
                   if det_l[1] ge det_u[1] then ptais[1] = 1>(round((det_l[1]+det_u[1])/2)+1)<20
                   ind = where(ptais gt 0,count)
                   if count eq 0 then begin
                      mesg = 'It is outside the detector range.'
                   endif else begin
                      mesg = ''
                      for i=0,count-1 do begin
                          ptai   = ptais[ind[i]]
                          kidney = ([tth,-tth])[ind[i]]+76.0-8*(ptai-1)
                          mesg   = [mesg,((count eq 2)?'For 2theta='+dm_to_string(([tth,-tth])[ind[i]],resolution=3)+string('b0'xb)+', ':'')+'PTAI='+dm_to_string(ptai)+', kidney='+$
                                   dm_to_string(kidney,resolution=3)+string('b0'xb)+', rate meter channel='+dm_to_string(2*(ptai-1))+' (SPEC) or '+dm_to_string(2*ptai-1)+' (DIFF).']
                      endfor
                      if ind[0] eq 1 then tth = -tth  ;negative side only
                   endelse
                endif else begin
                   idet = where(*(*pState).tthPtr ge tth,count)
                   if tth lt min(*(*pState).tthPtr) then count = 0
                   if count ne 0 then begin
                      if idet[0]-1 gt 0 then begin
                         if abs((*(*pState).tthPtr)[idet[0]-1]-tth) lt abs((*(*pState).tthPtr)[idet[0]]-tth) then $
                            idet[0] = idet[0]-1
                      endif
                      tmp = [idet[0],((n_elements(*(*pState).tthPtr)-1)<(idet[0]+1)),(0>(idet[0]-1))] & count = 0
                      if (*pState).ftype eq 'DCS' then idet = dm_common(tmp,dm_mask_detbanks(0),count)
                      if count eq 0 then begin
                         mesg = 'The corresponding detector number is '+dm_to_string(tmp[0]+1,/int)+'.'  
                      endif else begin
                         mesg = 'The corresponding central bank detector number is '+dm_to_string(idet[0]+1,/int)+'.'
                      endelse
                      if (*pState).ftype eq 'DCS' then mesg = [mesg,'(Detector number starts from 1. DCS liveplot detector number starts from 0.)']   
                   endif else $
                      mesg = 'It is outside the detector range.'
                endelse
                mesg = ['Two theta angle for ['+dm_to_string(info[7:9],separator=', ')+'] is '+dm_to_string(tth,resolution=3)+string('b0'xb)+'.',mesg]
             endif else begin
                lmax = 4.*!dpi/q
                if (*pState).ftype eq 'MACS' then begin
                   emin = 81.8042/lmax/lmax
                   mesg = 'Incident energy must be great than '+dm_to_string(emin,resolution=2)+' meV to view ['+dm_to_string(info[7:9],separator=', ')+'] reflection.'
                endif else begin
                   lmax = dm_to_string(lmax,resolution=3)+' '+string('c5'XB)
                   mesg = 'Incident wavelength must be smaller than '+lmax+' to view ['+dm_to_string(info[7:9],separator=', ')+'] reflection.'
                endelse
             endelse
             ok = dialog_message(mesg,/info,dialog_parent=(*pState).tlb,title='Result',/center)
          endif
         end
       'calclattbut':begin
          default = [abs((*pState).tth[(*pState).ftype eq 'MACS']),float((*(*pState).infoPtr)[[0,4,5,6,7,8,9]])]
          tmpname = ['2theta',(['lambda ('+string('c5'XB)+')','E (meV)'])[(*pState).ftype eq 'MACS'],'alpha','beta','gamma','H','K','L']+':'
          if (*pState).ftype eq 'MACS' then begin  ;macs, use energy 
             if finite(default[1]) then default[1] = (81.8042d)/default[1]/default[1]
          endif
          info = dm_dialog_input(tmpname,title='calculate',info='assuming a=b=c',default=default,/float,dialog_parent=(*pState).tlb,xsize=80,cancel=cancel)
          if keyword_set(cancel) then return
          if total(finite(info)) eq 8 then begin
             info = double(info)
             if (*pState).ftype ne 'MACS' then info[1] = (81.8042d)/info[1]/info[1]
             a  = dcs_cryst_align_latta(info[1],info[0],info[5:7],info[2:4])
             ok = dialog_message("The lattice paramter is "+dm_to_string(a,resolution=4)+' '+string('c5'xB)+' assuming a=b=c.',$
                   /info,dialog_parent=(*pState).tlb,title='Result',/center)
          endif
         end
       'calca2but':begin
          case (*pState).ftype of 
               'DCS':   info = psiname+['=90-theta, 2theta>0','=-(90+theta), 2theta<0']
               'MACS':  info = psiname+['=theta, 2theta>0','=180+theta, 2theta<0']
               'WAND':  info = psiname+'=theta-90'
               else:    info = ''
          endcase
          tth = dm_dialog_input('two theta:',default=(*pState).tth[(*pState).ftype eq 'MACS'],info=info,xsize=80,$
                dialog_parent=(*pState).tlb,title='Calculate '+psiname+' reset value',cancel=cancel)
          if keyword_set(cancel) then return
          if finite(tth) then begin
             if (*pState).ftype eq 'WAND' then tth = -tth  ;WAND
             if tth ge 0 then a2 = 90.0-tth/2. else a2=-(90.0+tth/2.)
             if (*pState).ftype eq 'MACS' then a2 = 90-a2  ;MACS
             mesg = 'Set '+psiname+' to '+dm_to_string(a2,resolution=3)+string('b0'xb)+'.'
             ok = dialog_message(mesg,/info,dialog_parent=(*pState).tlb,title='Result',/center)
          endif
         end
       'calca2rot':begin
          default = (float(*(*pState).infoPtr))[0:12]
          tmpname = [(['lambda ('+string('c5'XB)+')','E (meV)'])[(*pState).ftype eq 'MACS'],['a','b','c']+' ('+string('c5'XB)+')','alpha','beta','gamma','H1','K1','L1','H2','K2','L2']+':'
          if (*pState).ftype eq 'MACS' then begin  ;macs, use energy
             if finite(default[0]) then default[0] = (81.8042d)/default[0]/default[0]
          endif
          info = dm_dialog_input(tmpname,default=default,/float,dialog_parent=(*pState).tlb,xsize=80,title='Calculate '+psiname+' Rotation',info='1->2 counterclockwise',cancel=cancel)
          if keyword_set(cancel) then return
          if total(finite(info)) eq n_elements(default) then begin
             if (*pState).ftype eq 'MACS' then begin
                kidlim  = dm_getkidneyrange(info[0])  ;kidney range
                info[0] = sqrt((81.8042d)/info[0])
             endif
             (*(*pState).infoPtr)[0:12] = double(info)
             ki = 2.*!dpi/info[0]
             q1 = dcs_cryst_align_qlength1(info[7:9],info[1:6])
             q2 = dcs_cryst_align_qlength1(info[10:12],info[1:6])
             if max([q1,q2])/2. lt ki then begin
                mesg = ''
                tth1 = asin(q1/2./ki)*2/!dtor
                tth2 = asin(q2/2./ki)*2/!dtor
                if (*pState).ftype eq 'MACS' then begin
                   det_l = (([tth1,-tth1]-kidlim[0]+76.)/8)<19
                   det_u = (([tth1,-tth1]-kidlim[1]+76.)/8)>0
                   if det_l[0] lt det_u[0] then begin
                      if det_l[1] ge det_u[1] then tth1 = -tth1 $ 
                      else mesg = 'Two theta angle for ['+dm_to_string(info[7:9],separator=', ')+'] is outside the detector range.'
                   endif
                   if strlen(mesg) eq 0 then begin
                      det_l = (([tth2,-tth2]-kidlim[0]+76.)/8)<19
                      det_u = (([tth2,-tth2]-kidlim[1]+76.)/8)>0
                      if det_l[0] lt det_u[0] then begin
                         if det_l[1] ge det_u[1] then tth2 = -tth2 $ 
                         else mesg = 'Two theta angle for ['+dm_to_string(info[10:12],separator=', ')+'] is outside the detector range.'
                      endif  
                   endif 
                endif else begin
                   idet = where(*(*pState).tthPtr ge max([tth1,tth2]),count)
                   if count ne 0 then begin
                      if (*pState).ftype eq 'WAND' then begin
                         tth1 = -tth1 & tth2 = -tth2
                      endif
                   endif else $
                      mesg = 'The vectors are beyond the detector range.'
                endelse
                if strlen(mesg) eq 0 then begin
                   abet = dcs_cryst_align_angle(info[7:9],info[10:12],info[1:6]) ;angle between q1 and q2
                   if tth1 ge 0 then ang1 = 90.0-tth1/2. else ang1 = -(90.0+tth1/2.)
                   if tth2 ge 0 then ang2 = 90.0-tth2/2. else ang2 = -(90.0+tth2/2.)
                   ang2 = ang2-abet
                   if (*pState).ftype eq 'MACS' then begin
                      ang1 = 90-ang1
                      ang2 = 90-ang2
                   endif
                   mesg = psiname+' needs to rotate '+dm_to_string(ang2-ang1,resolution=2)+string('b0'xb)+'.'
                endif
             endif else begin
                lmax = 4.*!dpi/(max([q1,q2]))
                if (*pState).ftype eq 'MACS' then begin
                   mesg = 'Incident energy must be greater than '+dm_to_string(81.8042/lmax/lmax,resolution=3)+' meV.'
                endif else begin
                   mesg = 'Incident wavelength must be smaller than '+dm_to_string(lmax,resolution=3)+' '+string('c5'XB)+'.'
                endelse
             endelse
             ok = dialog_message(mesg,/info,dialog_parent=(*pState).tlb,title='Result',/center)
          endif
         end
       'e2lambdabut':   dcs_cryst_align_e2lambda,event,lambda=float((*(*pState).infoPtr)[0]),dcs=((*pState).ftype eq 'DCS'),macs=((*pState).ftype eq 'MACS')
       'kidlimitbut':   dm_macs_kidlimit,event,bgcolor=(*pState).bgcolor
       'macsalignment': begin ;20140103/24394-24425
          ind_dts = (where(stregex(*(*pState).xtit,'DFMDTS',/fold_case,/boolean)))[0] 
          ind_b11 = (where(stregex(*(*pState).xtit,'MonBlade11',/fold_case,/boolean)))[0] 
          ind_a2  = (where(stregex(*(*pState).xtit,'A2',/fold_case,/boolean)))[0]
          ind_mbt = (where(stregex(*(*pState).xtit,'MBTSlide',/fold_case,/boolean)))[0]
          for i=0,n_files-1 do begin
              selected[i] = where(dFiles eq open_file[i])
              (*pState).filesel->set_file,selected[i],/highlight
              event.id = widget_info((*pState).tlb,find_by_uname='loadbut')
              dcs_cryst_align_event,event,/nofilehistory         ;load data first        
              if ((*pState).xaxis eq ind_dts) or ((*pState).xaxis eq ind_b11) then begin
                 (*pState).fit = 1
                 dcs_cryst_align_plot,pState,params=params
                 if n_elements(angles) eq 0 then begin
                    angles   = (*(*pState).xPtr)[ind_a2+2,0] 
                    mbtslide = (*(*pState).xPtr)[ind_mbt+2,0]  
                    dfmdts   = (((*pState).xaxis eq ind_dts)?(params[1]):0.0)
                    blade11  = (((*pState).xaxis eq ind_b11)?(params[1]):0.0)
                 endif else begin
                    ind = where(abs(angles-(*(*pState).xPtr)[ind_a2+2,0]) le 0.1,cnt)
                    if cnt eq 0 then begin
                       angles   = [angles,(*(*pState).xPtr)[ind_a2+2,0]]
                       mbtslide = [mbtslide,(*(*pState).xPtr)[ind_mbt+2,0]]  
                       dfmdts   = [dfmdts,(((*pState).xaxis eq ind_dts)?(params[1]):0.0)]
                       blade11  = [blade11,(((*pState).xaxis eq ind_b11)?(params[1]):0.0)]
                    endif else begin
                       angles[ind[0]]   = (*(*pState).xPtr)[ind_a2+2,0]
                       mbtslide[ind[0]] = (*(*pState).xPtr)[ind_mbt+2,0]  
                       if ((*pState).xaxis eq ind_dts) then dfmdts[ind[0]] = params[1]
                       if ((*pState).xaxis eq ind_b11) then blade11[ind[0]] = params[1]
                    endelse 
                 endelse
              endif
              wait,0.5
          endfor 
          (*pState).filesel->set_file,selected,/highlight 
          (*pState).filename[0] = filename
          if ~(*pState).showfname then filename = ''
          dm_fit_macs_alignment,angles,mbtslide,dfmdts,blade11,plotobj=(*pState).plotWin,filename=filename,colr0=colr0,group_leader=(*pState).tlb
         end
       'macsalignmonchr': begin ;20140103/24452-24472
          a1center = fltarr(21)
          da1      = fltarr(21)
          ind_bld0 = (where(stregex(*(*pState).xtit,'MonBlade01',/fold_case,/boolean)))[0]
          for i=0,n_files-1 do begin
              selected[i] = where(dFiles eq open_file[i])
              (*pState).filesel->set_file,selected[i],/highlight
              event.id = widget_info((*pState).tlb,find_by_uname='loadbut')
              dcs_cryst_align_event,event,/nofilehistory         ;load data first        
              i_mon = (*pState).xaxis-ind_bld0
              if i_mon ge 0 and i_mon le 21 then begin
                 (*pState).fit = 1
                 dcs_cryst_align_plot,pState,params=params,sigma=sigma
                 a1center[i_mon] = params[1]
                 da1[i_mon]      = sigma[1]
              endif
              wait,0.5
          endfor
          (*pState).filesel->set_file,selected,/highlight
          (*pState).filename[0] = filename
          if ~(*pState).showfname then filename = ''
          (*pState).plotWin->setproperty,xdat=indgen(21)+1,xran=[0,22],ydat=a1center,yerr=da1,ytit='MonBlade Center (\deg)',xtit='MonBlade Number',/nodraw,$
              title='monochromator alignment',psym='circle',/hidelegend,linestyle='no line',color=colr0,cornertxt=filename
          (*pState).plotWin->draw  
          ok = dialog_message('Do you want to save the fit results?',title='Please confirm:',/question,dialog_parent=(*pState).tlb,/center)
          if ok eq 'Yes' then begin
             file = dm_choose_file('txt',/write,dialog_parent=(*pState).tlb,title='Please select a file name for saving the results',file='macs_monblade_'+date_str+'.txt')
             if strlen(file) eq 0 then break   
             openw,lun,file,/get_lun,error=openerr
             if openerr ne 0 then message,"Can't write in "+file+'.'
             for j=0,20 do printf,lun,j+1,a1center[j]
             free_lun,lun    
          endif 
          ;move to the center and reset
          ok = dialog_message('Do you want to move to the fit center and reset monochrmator blade positions in ICE?',title='Talk to ICE?',/question,dialog_parent=(*pState).tlb,/center)
          if ok eq 'Yes' then begin
             resetpos = dm_dialog_input('monblade reset position:',default=45.0,dialog_parent=(*pState).tlb,/return_number,cancel=cancel)
             if keyword_set(cancel) then break
             if finite(resetpos,/nan) then begin
                ok = dialog_message('The entered value is invalid. 45.0 is used instead.',dialog_parent=(*pState).tlb,/center)
                resetpos = 45.0
             endif       
             (*pState).plotWin->add_plot,[0,22],[resetpos,resetpos],linestyle='dashed',color='grey'
             (*pState).plotWin->draw
             dm_sendcom,'wait 66666'
             for nn=0,20 do begin
                 blades = 'monblade'+string(nn+1,format='(i02)') 
                 dm_move,blades,a1center[nn]
                 dm_sendcom,'device sethard '+blades + ' '+dm_to_string(resetpos)
             endfor
          endif
         end
       'macsalignanalyzer': begin
          ;ask for the a4 mask range
          mask = dm_dialog_input(['ignore A4 from:','to:'],default=[-12.0,12.0],dialog_parent=(*pState).tlb,/return_number,$
                    info=['Data with A4 within the ignored range will not be used in averaging','the analyzer center position.'],cancel=cancel)
          if keyword_set(cancel) then break
          if total(finite(mask,/nan)) gt 0 then begin
             ok = dialog_message('Invalid range. New detector efficiency is not calculated.',dialog_parent=(*pState).tlb,/center)
             break
          endif
          mask = [min(mask),max(mask)]
          a5center = fltarr(2,20)
          a5weight = fltarr(2,20)
          da5cnt   = fltarr(2,20)
          kidney   = fltarr(2)
          moncnt   = fltarr(2)
          a2       = fltarr(2)
          ind_det0 = 4
          ind_ana0 = (where(stregex(*(*pState).xtit,'AnalyzerTheta01',/fold_case,/boolean)))[0]
          ind_kid  = (where(stregex((*(*pState).xtit),'A4',/fold_case,/boolean)))[0]+2
          ind_a2   = (where(stregex((*(*pState).xtit),'A2',/fold_case,/boolean)))[0]+2
          (*pState).y_typ = 0
          dm_set_droplist,widget_info((*pState).tlb,find_by_uname='timelist'),select=0
          tmp_macs_eff = (*pState).macs_eff ;save efficiency method
          (*pState).macs_eff = 0            ;do not apply efficiency
          for i=0,1 do begin
              selected[i] = where(dFiles eq open_file[i])
              (*pState).filesel->set_file,selected[i],/highlight
              event.id = widget_info((*pState).tlb,find_by_uname='loadbut')
              dcs_cryst_align_event,event,/nofilehistory         ;load data first
              moncnt[i] = (*(*pState).xPtr)[n_elements((*(*pState).xPtr)[*,0])-1,0]
              kidney[i] = (*(*pState).xPtr)[ind_kid,0]
              a2[i]     = (*(*pState).xPtr)[ind_a2,0]
              for j=0,19 do begin
                  (*pState).det_tit = 'spec det='+dm_to_string(j+1)
                  (*pState).title   = (*pState).det_tit
                  (*pState).xaxis   = ind_ana0+j
                  (*pState).fit     = 1 
                  *(*pState).detPtr = j
                  dm_set_droplist,widget_info((*pState).tlb,find_by_uname='detlist'),select=ind_det0+j
                  dm_set_droplist,widget_info((*pState).tlb,find_by_uname='xaxislist'),select=ind_ana0+j
                  dcs_cryst_align_plot,pState,params=params,sigma=sigma,xs=xs,ys=ys,dy=dy
                  a5center[i,j] = params[1]
                  a5weight[i,j] = sqrt(2*!PI)*params[0]*params[2]*((i eq 1)?(moncnt[0]/moncnt[1]):1.0)
                  da5cnt[i,j]   = sigma[1]
                  wait,0.5
              endfor
          endfor
          (*pState).filesel->set_file,selected,/highlight
          (*pState).filename[0] = filename
          if ~(*pState).showfname then filename = ''
          (*pState).macs_eff = tmp_macs_eff ;restore efficiency method
          a4     = kidney[0]-76.0+findgen(20)*8.0
          ign_k0 = where((a4 le mask[1]) and (a4 ge mask[0]),cnt0)
          a4     = kidney[1]-76.0+findgen(20)*8.0
          ign_k1 = where((a4 le mask[1]) and (a4 ge mask[0]),cnt1)
          a5_ave = reform((a5center[0,*]+a5center[1,*])/2.)
          da5    = reform(sqrt(da5cnt[0,*]^2+da5cnt[1,*]^2)/2.0) 
          if cnt0 gt 0 then begin
             a5_ave[ign_k0]     = a5center[1,ign_k0]
             da5[ign_k0]        = da5cnt[1,ign_k0]
             a5weight[0,ign_k0] = !values.f_nan 
          endif
          if cnt1 gt 0 then begin
             a5_ave[ign_k1]     = a5center[0,ign_k1]
             da5[ign_k1]        = da5cnt[0,ign_k1]
             a5weight[1,ign_k1] = !values.f_nan 
          endif
          wgh_ave = total(a5weight,1,/nan)/(1>total(finite(a5weight),1)) ;average of two file
          ind     = where(wgh_ave gt 0,cnt,complement=ind1,ncomplement=cnt1)
          (*pState).plotWin->setproperty,xdat=indgen(20)+1,ydat=a5_ave,yerr=da5,ytit='Average Center (\deg)',xtit='Analyzer Number',/nodraw,xran=[0,21],$
              title='A5 alignment @ kidney=('+dm_to_string(kidney,sep=dgsym+', ')+dgsym+')',psym='circle',/hidelegend,linestyle='no line',color=colr0,cornertxt=filename  
          (*pState).plotWin->add_text,'ignore!d1!n = ('+dm_to_string(ign_k0+1,sep=',')+')!Cignore!d2!n = ('+dm_to_string(ign_k1+1,sep=',')+')',0.05,0.93,fontsize=10
          (*pState).plotWin->draw  
          ok = dialog_message('Do you want to save the fit results?',title='Please confirm:',/question,dialog_parent=(*pState).tlb,/center)
          if ok eq 'Yes' then begin
             file = dm_choose_file('txt',/write,dialog_parent=(*pState).tlb,title='Please select a file name for saving the results',file='macs_aligna5_'+date_str+'.txt')
             if strlen(file) eq 0 then break   
             openw,lun,file,/get_lun,error=openerr
             if openerr ne 0 then message,"Can't write in "+file+'.'
             det_eff = fltarr(20)+1.0
             det_eff[ind] = 1.0/wgh_ave[ind]
             det_eff[ind] = det_eff[ind]*cnt/total(det_eff[ind])
             if cnt ne 20 then det_eff = det_eff*20./total(det_eff)
             printf,lun,'det#'+tab+'center1'+tab+'     center2'+tab+'center_average'+tab+tab+'int_int1'+tab+'int_int2'+tab+'int_average'+tab+'det_eff'
             for j=0,19 do printf,lun,string(j+1,format='(i02)'),a5center[0,j],a5center[1,j],a5_ave[j],a5weight[0,j],a5weight[1,j],wgh_ave[j],det_eff[j],format='(i2, 7g)'
             free_lun,lun    
          endif 
          ;move to the center and reset
          ok = dialog_message('Do you want to move to the fit center and reset analyzer positions in ICE?',title='Talk to ICE?',/question,dialog_parent=(*pState).tlb,/center)
          if ok eq 'Yes' then begin
             a5pos = mean(a2)/2.0
             resetpos = dm_dialog_input('analyzer reset position:',default=a5pos,dialog_parent=(*pState).tlb,/return_number,cancel=cancel)
             if keyword_set(cancel) then break
             if finite(resetpos,/nan) then begin
                ok = dialog_message('The entered value is invalid. '+dm_to_string(a5pos)+' is used instead.',dialog_parent=(*pState).tlb,/center)
                resetpos = a5pos
             endif  
             (*pState).plotWin->add_plot,[0,21],[a5pos,a5pos],linestyle='dashed',color='grey'
             (*pState).plotWin->draw     
             dm_sendcom,'wait 66666'
             for i=1,20 do begin
                 analyzer = 'analyzertheta'+string(i,format='(i02)')
                 dm_move,analyzer,a5_ave[i-1]
                 dm_sendcom,'device set '+analyzer+' '+dm_to_string(resetpos)
             endfor
             if cnt1 ne 0 then begin
                ok = dialog_message('Detector '+dm_to_string(ind1+1,sep=', ')+([' are',' is'])[cnt1 eq 1]+' reset even though '+(['they are','it is'])[cnt1 eq 1]+ ' outside the masking range.',dialog_parent=(*pState).tlb,/center)
             endif
          endif
         end
       'macsalignkidney': begin
          event.id = widget_info((*pState).tlb,find_by_uname='loadbut')
          dcs_cryst_align_event,event,/nofilehistory         ;load data first
          *(*pState).detPtr = indgen((*pState).ndet)
          (*pState).det_tit = (['spec ','diff ','spec+diff '])[(*pState).y_typ]+'det=all'
          (*pState).title   = (*pState).det_tit
          (*pState).xaxis   = (where(stregex(*(*pState).xtit,'^A4',/fold_case,/boolean)))[0] 
          dm_set_droplist,widget_info((*pState).tlb,find_by_uname='detlist'),select=0
          dm_set_droplist,widget_info((*pState).tlb,find_by_uname='xaxislist'),select=(*pState).xaxis
          tmp_macs_eff = (*pState).macs_eff ;save efficiency method
          (*pState).macs_eff = 1            ;apply efficiency from file
          dcs_cryst_align_plot,pState,xs=x,ys=y,dys=dy
          (*pState).macs_eff = tmp_macs_eff ;restore efficiency method
          if n_elements(x) eq 0 then break
          ind  = sort(x) & x = x[ind] & y = y[ind]& dy = dy[ind]
          ind1 = where(x le -30,count1)
          ind2 = where(x ge 30,count2)
          dm_gaussfit,y[ind1],x=x[ind1],params=params1,nterms=4,measure_errors=dy[ind1]
          dm_gaussfit,y[ind2],x=x[ind2],params=params2,nterms=4,measure_errors=dy[ind2]
          yy = [params1[0]*exp(-0.5*((x[ind1]-params1[1])/params1[2])^2)+params1[3],params2[0]*exp(-0.5*((x[ind2]-params2[1])/params2[2])^2)+params2[3]]
          (*pState).plotWin->add_plot,[x[ind1],x[ind2]] ,yy,linestyle='solid',color=(['red','cyan'])[total(abs([255,0,0]-colr0)) eq 0],legend='gauss fit',layer=0,thick=2
          (*pState).plotWin->add_text,'peak1 = '+dm_to_string(params1[1],res=3)+dgsym+'!Cpeak2 = '+dm_to_string(params2[1],res=3)+dgsym+'!Ccenter = '+dm_to_string((params1[1]+params2[1])/2.0,res=3)+dgsym,0.05,0.93,$
             fontsize=(*pState).textposfsize[2]
          (*pState).plotWin->draw
          ok = dialog_message('Do you want to redefine kidney angle in ICE?',title='Talk to ICE?',/question,dialog_parent=(*pState).tlb,/center)
          if ok eq 'Yes' then begin  
             dm_sendcom,'wait 66666'
             kidney = dm_dev_pos('kidney',/number)
             dm_move,'kidney',kidney
             dm_sendcom,'device set kidney '+dm_to_string(kidney-(params1[1]+params2[1])/2.0,res=3)
          endif
         end
       'macsflipratio': begin
          moncnt = fltarr(2)
          f_numb = strarr(2)
          fixfitwidth = (*pState).fixfitwidth
          showfit = (*pState).showfit
          (*pState).fixfitwidth = 1
          (*pState).showfit = 1
          for i=0,1 do begin
              selected[i] = where(dFiles eq open_file[i])
              tmp = stregex(open_file[i],'([0-9]{5})\.ng0$',/extract,/subexpr)
              if n_elements(tmp) eq 2 then f_numb[i] = tmp[0] else f_numb[i] = open_file[i] 
              (*pState).filesel->set_file,selected[i],/highlight
              event.id = widget_info((*pState).tlb,find_by_uname='loadbut')
              dcs_cryst_align_event,event,/nofilehistory         ;load data first
              moncnt[i] = (*(*pState).xPtr)[n_elements((*(*pState).xPtr)[*,0])-1-(*pState).macs_normchoice,0]
              (*pState).fit = 1
              if i eq 0 then $
                 dcs_cryst_align_plot,pState,params=params1,sigma=sigma1,xs=xs1,ys=ys1,dy=dy1,fixedwidth=!values.f_nan,peaktime=peaktime1 $
              else $
                 dcs_cryst_align_plot,pState,params=params2,sigma=sigma2,xs=xs2,ys=ys2,dy=dy2,fixedwidth=params1[2],peaktime=peaktime2     
              wait,1
          endfor
          (*pState).fixfitwidth = fixfitwidth
          (*pState).showfit = showfit
          (*pState).filesel->set_file,selected,/highlight
          if (n_elements(params1) eq 0) or (n_elements(params2) eq 0) then break
          fr  = params1[0]/params2[0]*moncnt[1]/moncnt[0]
          if fr lt 1 then fr = 1./fr
          dfr = fr*sqrt((sigma1[0]/params1[0])^2+(sigma2[0]/params2[0])^2)
          (*pState).plotWin->setproperty,xdat=xs1,ydat=ys1,yerr=dy1,/nodraw,psym='circle',/hidelegend,linestyle='no line',color='black'
          dcs_cryst_align_fitline,pState,params1,xs1,xx=xx,yy=yy
          (*pState).plotWin->add_plot,xx,yy,linestyle='solid',color=(['red','cyan'])[total(abs([255,0,0]-colr0)) eq 0],layer=0,thick=2
          (*pState).plotWin->add_plot,xs2,ys2,yerr=dy2,linestyle='no line',color='black',psym='circle'
          dcs_cryst_align_fitline,pState,params2,xs2,xx=xx,yy=yy
          (*pState).plotWin->add_plot,xx,yy,linestyle='solid',color=(['red','cyan'])[total(abs([255,0,0]-colr0)) eq 0],layer=2,thick=2
          xmin  = min([xs1,xs2],max=xmax) & ymin = min([ys1-dy1,ys2-dy2]) & ymax = max([ys1+dy1,ys2+dy2])
          title = 'flipping ratio='+dm_to_string(fr,resolution=2)+'\pm'+dm_to_string(dfr,resolution=2)
          if n_elements(peaktime1) ne 0 then text  = '!C!D'+f_numb[0]+': '+dm_to_string(peaktime1,/date,/est) else text = ''
          if n_elements(peaktime2) ne 0 then begin
             text  = text+'!C!D'+f_numb[1]+': '+dm_to_string(peaktime2,/date,/est)
             text  = text+'!C!D'+'average : '+dm_to_string(mean([peaktime2,peaktime1],/double),/date,/est)
          endif
          (*pState).plotWin->add_text,text,0.04,0.94,fontsize=(*pState).textposfsize[2]
          (*pState).plotWin->setproperty,xran=[xmin,xmax]+[-1,1],yran=[ymin,ymax]+[-0.1,0.1]*(ymax-ymin),ytit='Neutron Counts',title=title,cornertxt=f_numb[0]+','+f_numb[1]
         end  
       'clearhistory': begin
          ptr_free,(*pState).file_hist
          (*pState).n_history = 0
          dcs_cryst_align_setfilehistory,pState
         end
       'dm_filesel_newfilter': begin
          tlb  = (*pState).tlb
          if strmatch(event.filter,'.dcs',/fold_case) then dcs_cryst_align_changeftype,{WIDGET_BUTTON,ID:widget_info(tlb,find_by_uname='ftype_DCS'),TOP:tlb,HANDLER:tlb,SELECT:1} $
          else if strmatch(event.filter,'.ng0,.bt9',/fold_case) then dcs_cryst_align_changeftype,{WIDGET_BUTTON,ID:widget_info(tlb,find_by_uname='ftype_MACS'),TOP:tlb,HANDLER:tlb,SELECT:1} $
          else if strmatch(event.filter,'.wand',/fold_case) then dcs_cryst_align_changeftype,{WIDGET_BUTTON,ID:widget_info(tlb,find_by_uname='ftype_WAND'),TOP:tlb,HANDLER:tlb,SELECT:1}
         end 
       else: begin
          choose = dm_to_number(strmid(uname,6,strlen(uname)-6),/int) 
          uname  = strmid(uname,0,5) 
          if uname eq 'ftype' then dcs_cryst_align_changeftype,event                     ;change file type
          if uname eq 'fhist' then dcs_cryst_align_setfilehistory,pState,choose=choose   ;file history
         end
    endcase
end

pro dcs_cryst_align_Exit,tlb
    widget_control,tlb,get_uvalue=pState,/no_copy
    obj_destroy,(*pState).filesel
    obj_destroy,(*pState).plotWin
    cd,(*pState).root
    ptr_free,(*pState).fname,(*pState).fextn,(*pState).psiname,(*pState).xtit,(*pState).file_hist
    ptr_free,(*pState).detPtr,(*pState).xPtr,(*pState).yPtr,(*pState).dyPtr,(*pState).bgPtr,(*pState).dbgPtr,(*pState).tthPtr,(*pState).infoPtr
    ptr_free,pState
    widget_control,tlb,/destroy
end

;initparms=[a,b,c,alpha,beta,gamma,uh,uk,ul,vh,vk,vl]
pro dcs_cryst_align,event,initparms,workdir=workdir,datadir=datadir,wand=wand,bgcolor=bgcolor,macs=macs
    state={group_leader:                 0L, $  ;group leader
           tlb:                          0L, $  ;top level base
           mbar:                         0L, $  ;menu bar
           ftype:                     'DCS', $  ;instrument type 'DCS', 'MACS', or 'WAND'
           fname:   ptr_new(/allocate_heap), $  ;instrument name 
           fextn:   ptr_new(/allocate_heap), $  ;file extension
           psiname: ptr_new(/allocate_heap), $  ;'A2' for DCS, 'A3' for MACS, 'Psi' for WAND
           geom:                    [0L,0L], $  ;geometry
           filesel:               obj_new(), $  ;file selector
           file_hist:            ptrarr(10), $  ;save the latest 10 file selections
           filename:                ['',''], $  ;save file name cornertxt string
           n_history:                    0s, $  ;number of histories
           infoPtr: ptr_new(/allocate_heap), $  ;store the info for calculating 2theta, [lambda,a,b,c,alpha,beta,gamma,h1,k1,l1,h2,k2,l2]
           fit:                          0b, $  ;flag for fit
           tchoice:                      0b, $  ;temperature choice 0-sample 1-control 2-setpoint
           fitfunname:           ['','',''], $  ;
           fitfunchoice:                 0b, $  ;0-gaussian, 1-lorentzian, 2-moffat
           fitfunnterm:                  4b, $  ;fit function nterms gaussian-3,4,5,6 lorentzian-3,4,5 mofat-4,5,6
           mpfit_exist:                  0b, $  ;flag for mpfit function existing or not
           showfit:                      0b, $  ;flag for showing fitting parameters
           textposfsize:          fltarr(3), $  ;save the info text position and font size
           fixfitwidth:                  0b, $  ;flag for fix fitting width
           showfname:                    1b, $  ;flag for showing file name at the plot corner
           showcursor:                   0b, $  ;flag for showing cursor position
           ndet:                         0L, $  ;number of detectors
           bitflip:                      1b, $  ;flag for checking the bitflip error
           indvdet:                      0b, $  ;flag for showing individual detectors
           macs_eff:                     0b, $  ;flag for apply macs detector efficiency 0:do not apply 1:apply from file 2:apply specified
           macs_deteff:        fltarr(4,20), $  ;save the macs detector efficiency [0:1,*] file [2:3,*] specify
           macs_lamb2:                   0b, $  ;flag for applying monitor lamda/2 correction
           macs_normchoice:              0b, $  ;flag for normalizing choice for macs, 0-divided by monitor count, 1-divided by time
           macs_maspacing:        fltarr(2), $  ;[monochromator,analyzer] spacing
           tthPtr:  ptr_new(/allocate_heap), $  ;save the two theta value
           detPtr:  ptr_new(/allocate_heap), $  ;pointer to detector array
           tth:                     [0e,0e], $  ;save the tth vaule [det list 2th, macs actual 2th]
           t_chan:                       1s, $  ;0-all 1-elastic 2-specify
           e_ran:                   [0e,0e], $  ;energy range for t_chan=2
           xaxis:                        0s, $  ;xaxis number
           y_typ:                        0s, $  ;for macs, 0-spec 1-diff 2-spec+diff
           xtit:    ptr_new(/allocate_heap), $  ;xtit strings
           xPtr:    ptr_new(/allocate_heap), $  ;pointer to x data
           yPtr:    ptr_new(/allocate_heap), $  ;pointer to y data
           dyPtr:   ptr_new(/allocate_heap), $  ;pointer to dy data
           bgPtr:   ptr_new(/allocate_heap), $  ;pointer to background data, DCS only
           dbgPtr:  ptr_new(/allocate_heap), $  ;pointer to background error data, DCS only
           bgcolor:                      '', $  ;save plot window background color
           plotWin:               obj_new(), $  ;draw widget
           duration:                     0e, $  ;duration of measurement in secs
           dcs_monitor:             [0e,0e], $  ;dcs monitor counts, [sample,background]
           det_tit:                      '', $  ;output detector string
           tch_tit:                      '', $  ;output time channel string
           title:                        '', $  ;plot title
           root:                         '', $  ;starting directory
           workDir:                      '', $  ;work directory
           datDir:                       '', $  ;data directory
           enotebook:                    0b  $  ;use Rob's enotebook
    }
    registerName = 'dcs_cryst_align'
    cd,current = current
    state.root = current
    state.workDir = current
    state.datDir  = current
    if n_elements(workdir) ne 0 then state.workDir=workdir[0]
    if n_elements(datadir) ne 0 then state.datDir=datadir[0]
    if n_elements(bgcolor) ne 0 then state.bgcolor=bgcolor[0] else state.bgcolor = 'white'

    *state.infoPtr = dblarr(13) & (*state.infoPtr)[*] = !values.d_nan
    if (n_elements(initparms) le 12) and (n_elements(initparms) ne 0) then (*state.infoPtr)[1:n_elements(initparms)] = initParms
    
    ;handle instrument choice. To add a new instrument, modify dcs_cryst_align_changeftype
    if keyword_set(macs) then state.ftype = 'MACS' $
    else if keyword_set(wand) then state.ftype = 'WAND' $
    else if xregistered('dcs_mslice',/noshow) then begin  ;if started on IDL
         FORWARD_FUNCTION LookupManagedWidget
         id = LookupManagedWidget('dcs_mslice')
         if id ne 0 then begin
            widget_control,id,get_uvalue=obj_Mslice
            if obj_valid(obj_Mslice) then begin
               obj_Mslice->getproperty,ftype=ftype,latticeparm=latticeparm & ftype = strupcase(ftype)
               if ftype eq 'MACS' or ftype eq 'WAND' then state.ftype = ftype
               if finite((*state.infoPtr)[1],/nan) then (*state.infoPtr)[1:12] = latticeparm
            endif
         endif
    endif
    
    state.macs_deteff[2:3,*] = !values.f_nan
    
    pState = ptr_new(state,/no_copy)
    
    dcs_cryst_align_changeftype,/init,pState=pState,det_info=det_info,x_info=x_info

    defsysv,'!DAVE_AUXILIARY_DIR',exists=exists
    if exists then icon = !DAVE_AUXILIARY_DIR+'align.ico' $
    else icon = file_which('align.ico',/include_current_dir)
    if dm_to_number(!version.release) ge 6.4 then icon_extra={bitmap:icon}
    
    if n_elements(event) ne 0 then (*pState).group_leader = event.top
    (*pState).tlb  = widget_base(title=(*pState).ftype+([' Single Crystal',''])[(*pState).ftype eq 'MACS']+' Alignment Utility',/column,kill_notify='dcs_cryst_align_Exit',$
          /tlb_size_event,group_leader=(*pState).group_leader,_extra=icon_extra,map=0,mbar=bar,uname='tlb')
    (*pState).mbar = bar      
    
    ;check if mpfitpeak routine exists
    (*pState).mpfit_exist = 1b
    catch, myerror
    if myerror ne 0 then begin
       catch,/cancel
       (*pState).mpfit_exist = 0b
       (*pState).fitfunchoice = 0
    end
    if (*pState).mpfit_exist then resolve_routine,'mpfitpeak',/no_recompile,/is_function  ;check if mpfitpeak function exists, for Lorentzian and Moffat fit

    ;file menu
    filemenu    = widget_button(bar,value='File',/menu)
    ftypemenu   = widget_button(filemenu,value='File Type',/menu,uname='ftypeMenu')
    fhistmenu   = widget_button(filemenu,value='Recent Files',/menu,uname='fhistMenu',sensitive=0,/separator)
    clearhist   = widget_button(filemenu,value='Clear File History',uname='clearHistory',sensitive=0)
    fileExit    = widget_button(filemenu,value='Exit',uname='exitbut',/separator)
    for i=0,n_elements(*(*pState).fname)-1 do $
        void    = dm_widget_button(ftypemenu,value=(*(*pState).fname)[i],uname='ftype_'+(*(*pState).fname)[i],set_button=strmatch((*(*pState).fname)[i],(*pState).ftype),/fold_case)
    
    ;option menu
    optnmenu    = widget_button(bar,value='Options',/menu,uname='optnMenu')   
    optnBitflip = dm_widget_button(optnmenu,value='Check Bitflip Error',set_button=(*pState).bitflip,onstring='Bitflip Error Correction Enabled',$
                  offstring='Bitflip Error Correction Disabled',uname='optnbitflip',sensitive=((*pState).ftype eq 'DCS'))
    optnIndvDet = dm_widget_button(optnmenu,value='Show Individual Detector',set_button=(*pState).indvdet,onstring='Show Individual Detector',$
                  offstring='Sum Detectors',uname='optnindvdet')   
    optnShowFnm = dm_widget_button(optnmenu,value='Show File Name',set_button=(*pState).showfname,onstring='Show File Name',$
                  offstring='Hide File Name',uname='optnshowfname')    
    optnShowCur = dm_widget_button(optnmenu,value='Show Cursor Position',set_button=(*pState).showcursor,onstring='Show Cursor Position',$
                  offstring='Hide Cursor Position',uname='optnshowcursor')
    optnShowFit = dm_widget_button(optnmenu,value='Show Fitting Parameters',set_button=(*pState).showfit,onstring='Show Fitting Parameters',$
                  offstring='Hide Fitting Parameters',uname='optnshowfit')
    optnFixWid = dm_widget_button(optnmenu,value='Fix Fitting Width',set_button=(*pState).fixfitwidth,onstring='Fix Fitting Width',$
                  offstring='Not Fix Fitting Width',uname='optnfixwidth')                                 
    optnAppLamb = dm_widget_button(optnmenu,value='Apply Monitor Lambda/2 Correction',set_button=(*pState).macs_lamb2,onstring='Apply Monitor Lambda/2 Correction',$
                  offstring='No Monitor Lambda/2 Correction',uname='optnapplylamb2',sensitive=((*pState).ftype eq 'MACS'))                                                        
    tempmenu    = widget_button(optnmenu,value='Temperature Choice',uname='tempMenu',/menu,/separator,sensitive=((*pState).ftype ne 'WAND'))
    void        = dm_widget_button(tempmenu,value='Sample',uname='temp_samp',set_button=((*pState).tchoice eq 0)) 
    void        = dm_widget_button(tempmenu,value='Control',uname='temp_ctrl',set_button=((*pState).tchoice eq 1))
    void        = dm_widget_button(tempmenu,value='Setpoint',uname='temp_setp',set_button=((*pState).tchoice eq 2))  
    fitfcmenu   = widget_button(optnmenu,value='Fitting Function',/menu)
    void        = dm_widget_button(fitfcmenu,value='Gaussian',uname='fit_gaussian',set_button=((*pState).fitfunchoice eq 0))
    void        = dm_widget_button(fitfcmenu,value='Lorentzian',uname='fit_lorentzian',set_button=((*pState).fitfunchoice eq 1),sensitive=(*pState).mpfit_exist) 
    void        = dm_widget_button(fitfcmenu,value='Moffat',uname='fit_moffat',set_button=((*pState).fitfunchoice eq 2),sensitive=(*pState).mpfit_exist)
    (*pState).fitfunname = ['C0*exp[-(x-C1)^2/(2*C2^2)]','C0/(1+(x-C1)^2/C2^2)','C0/(1+(x-C1)^2/C2^2)^C3']
    void        = dm_widget_button(fitfcmenu,value=(*pState).fitfunname[(*pState).fitfunchoice],uname='fitfun_3',set_button=((*pState).fitfunnterm eq 3),/separator)     
    void        = dm_widget_button(fitfcmenu,value=(*pState).fitfunname[(*pState).fitfunchoice]+(['+C3','+C4'])[(*pState).fitfunchoice eq 2],uname='fitfun_4',set_button=((*pState).fitfunnterm eq 4))     
    void        = dm_widget_button(fitfcmenu,value=(*pState).fitfunname[(*pState).fitfunchoice]+(['+C3+C4*x','+C4+C5*x'])[(*pState).fitfunchoice eq 2],uname='fitfun_5',set_button=((*pState).fitfunnterm eq 5))   
    void        = dm_widget_button(fitfcmenu,value=(*pState).fitfunname[(*pState).fitfunchoice]+(['+C3+C4*x+C5*x^2','+C4+C5*x+C6*x^2'])[(*pState).fitfunchoice eq 2],uname='fitfun_6',set_button=((*pState).fitfunnterm eq 6),sensitive=((*pState).fitfunchoice eq 0))
    if (*pState).ftype eq 'MACS' then begin  ;MACS
       normmenu = widget_button(optnmenu,value='Intensity Normalization Choice',uname='normMenu',/menu)
       void     = dm_widget_button(normmenu,value='Divided by Monitor Count',uname='norm_monitor',set_button=((*pState).macs_normchoice eq 0))
       void     = dm_widget_button(normmenu,value='Divided by Counting Time',uname='norm_time',set_button=((*pState).macs_normchoice eq 1))
       macseffmenu = dm_widget_button(optnmenu,value='Apply Detector Efficiency',set_button=((*pState).macs_eff ne 0),uname='macseffMenu',/menu)
       void     = dm_widget_button(macseffmenu,value='From File',uname='macs_eff_file',set_button=((*pState).macs_eff eq 1))
       void     = dm_widget_button(macseffmenu,value='Specify...',uname='macs_eff_spec',set_button=((*pState).macs_eff eq 2))
       void     = dm_widget_button(macseffmenu,value='None',uname='macs_eff_none',set_button=((*pState).macs_eff eq 0),sensitive=((*pState).macs_eff ne 0),/separator)
    endif                 
             
    ;tools menu
    iftype      = (where(*(*pState).fname eq (*pState).ftype))[0]
    toolmenu    = widget_button(bar,value='Tools',/menu,uname='toolMenu')  
    calctthBut  = widget_button(toolmenu,value='Calculate 2theta',uname='calctthbut')
    calclattBut = widget_button(toolmenu,value='Calculate Lattice Parameter',uname='calclattbut')
    calca2But   = widget_button(toolmenu,value='Calculate '+(*(*pState).psiname)[iftype]+' Reset Value',uname='calca2but')
    calca2Rot   = widget_button(toolmenu,value='Calculate '+(*(*pState).psiname)[iftype]+' Rotation Angle',uname='calca2rot')
    e2lambdaBut = widget_button(toolmenu,value='E to Wavelength Conversion',/separator,uname='e2lambdabut')
    if (*pState).ftype eq 'MACS' then begin  ;MACS
       void     = widget_button(toolmenu,value='Kidney Angle Limit',uname='kidlimitbut')
       void     = widget_button(toolmenu,value='Fit and Plot Instrument Alignment Files',uname='macsalignment',/separator)
       void     = widget_button(toolmenu,value='Fit and Plot Monochromator Alignment Files',uname='macsalignmonchr')
       void     = widget_button(toolmenu,value='Fit and Plot Analyzer Alignment Files',uname='macsalignanalyzer')
       void     = widget_button(toolmenu,value='Fit and Plot Kidney Alignment File',uname='macsalignkidney')
       void     = widget_button(toolmenu,value='Calculate Flipping Ratio',uname='macsflipratio',/separator)
    endif
    
    ;background menu
    if (*pState).ftype eq 'DCS' then begin
       backmenu = widget_button(bar,value='Background',/menu,uname='backgroundMenu')
       void     = widget_button(backmenu,value='Load Empty Can File(s)',uname='loadbgfile')
       void     = widget_button(backmenu,value='Clear Background',uname='clearbgfile',/separator,sensitive=0)
    endif
    
    if !version.os_family eq 'Windows' then $
       mbar_sep = widget_label((*pState).tlb,sensitive=0,value=' ',/dynamic_resize,scr_ysize=4)
    
    tmpbase     = widget_base((*pState).tlb,/row)   
    left        = widget_base(tmpbase,/col)
    (*pState).filesel = obj_new('dm_filesel',parent=left,xsize=160,ysize=20,/frame,filter=(*(*pState).fextn)[iftype],path=(*pState).datDir,group_leader=(*pState).tlb,$
       ncnrftp=file_test((*pState).workDir,/directory,/write),bufferdir=(*pState).workDir)
    if !version.os_family eq 'unix' then $
       center   = widget_base(tmpbase,/col,/align_bottom,xpad=0,ypad=0,space=0,uname='center') $
    else $
       center   = widget_base(tmpbase,/grid_layout,/col,/align_bottom,uname='center')

    void        = widget_label(center,value='Det. Number',uname='detlabel')
    detList     = dm_widget_droplist(center,value=det_info,uname='detlist')
    void        = widget_label(center,value=(['Time Channel','Det. Type'])[(*pState).ftype eq 'MACS'],uname='timelabel',sensitive=((*pState).ftype ne 'WAND'))
    timeList    = dm_widget_droplist(center,value=([['All','Elastic','Specify',' '],['SPEC','DIFF','SPEC+DIFF','Monitor']])[0:([2,3])[(*pState).ftype eq 'MACS'],((*pState).ftype eq 'MACS')],$
                  select=([(*pState).t_chan,(*pState).y_typ])[(*pState).ftype eq 'MACS'],sensitive=((*pState).ftype ne 'WAND'),uname='timelist')
    void        = widget_label(center,value='X-axis',uname='xaxislabel')
    xaxisList   = dm_widget_droplist(center,value=x_info,uname='xaxislist')
    ;for context sensitive menu
    dcs_cryst_align_csm,center,pState,csm=csm
    loadBut     = widget_button(center,value='Load Data',uname='loadbut')
    fitBut      = widget_button(center,value=(['Gaussian','Lorentzian','Moffat'])[(*pState).fitfunchoice]+' Fit',uname='fitbut')
    clpbBut     = widget_button(center,value='Copy to Clipboard',uname='clpbbut')
    saveBut     = widget_button(center,value='Save Plot',uname='savebut')
    dataBut     = widget_button(center,value='Save Data',uname='databut')
    printBut    = widget_button(center,value='Print',uname='printbut')
    if dm_to_number(!version.release) ge 5.6 then begin
       widget_control,fitBut,tooltip='Fit the highest peak or zoom in to fit a local peak.'
       widget_control,dataBut,tooltip='Save the plot data in an ascii file.'
    endif
    
    geom  = widget_info(printBut,/geometry)
    widget_control,detList,xsize=geom.scr_xsize
    widget_control,timeList,xsize=geom.scr_xsize
    widget_control,xaxisList,xsize=geom.scr_xsize

    geom  = widget_info(left,/geometry)
    ysize = geom.scr_ysize
    xsize = fix(1.4*ysize,type=3)
    (*pState).plotWin = obj_new('dm_plot',xsize=xsize,ysize=ysize,widgetbase=tmpbase,group_leader=(*pState).tlb,/compound,background=(*pState).bgcolor,/isolatin1,ctxtfsize=8,showcursorpos=(*pState).showcursor)
    
    geom  = widget_info(center,/geometry)
    widget_control,center,xsize=geom.scr_xsize
       
    dm_center_kid, (*pState).tlb   ;centering the top level base
    widget_control,(*pState).tlb,/realize,/map
    (*pState).plotWin->draw,/view
    (*pState).plotWin->totop
    
    geom  = widget_info((*pState).tlb,/geometry)
    (*pState).geom = [geom.scr_xsize,geom.scr_ysize]
    
    widget_control,(*pState).tlb,set_uvalue=pState

    if xregistered('dave',/noshow) and (n_elements(event) ne 0) then begin
       (*pState).enotebook = 1b
       RET = DAVE_SET_FOCUS((*pState).tlb)
    endif
    
    xmanager,registerName,(*pState).tlb,cleanup='dcs_cryst_align_Exit',/no_block
end