; $Id: dm_dcspatch.pro,v 1.6.2.5 2015/06/05 13:58:00 ymqiu Exp $
;#######################################################################
;
; NAME:
;  dm_dcspatch
;
; PURPOSE:
;  this program patches the wavelength info of a dcs file, adapted from
;  John Copley's dcs_patchrawdatafile.pro
;
; CATEGORY:
;  dcs tools
;
; AUTHOR:
;  Yiming Qiu
;  NIST Center for Neutron Research
;  100 Bureau Drive, Gaithersburg, MD 20899-6102
;  United States
;  yiming.qiu@nist.gov
;  June, 2015
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;#######################################################################

;clear all objects and widget hierarchy
pro dm_dcspatch_Exit,tlb
    widget_control,tlb,get_uvalue=state,/no_copy
    obj_destroy,[state.open_filesel,state.save_filesel]
    widget_control,tlb,/destroy
end

;event handler
pro dm_dcspatch_event,event
    compile_opt IDL2    ;,strictarrsubs
                        ;idl2=defint32,strictarr
                        ;strictarr:   [] to index array
                        ;strictarrsubs: error when out-of-range indices,IDL5.6 or after
    WIDGET_CONTROL,/HOURGLASS
    widget_control,event.handler,get_uvalue=state,/no_copy
    state.open_filesel->getproperty,path=open_path,file=open_file,ftpobj=ftpobj,dFiles=dfiles
    state.save_filesel->getproperty,path=save_path

    while(strmid(open_path,0,1,/reverse_offset) eq  ([state.pathsep,'/'])[obj_valid(ftpobj)]) do  open_path = strmid(open_path,0,strlen(open_path)-1)
    while(strmid(save_path,0,1,/reverse_offset) eq  state.pathsep) do  save_path = strmid(save_path,0,strlen(save_path)-1)
    
    ;catch and ignore all errors in this program
    catch, myerror
    if myerror ne 0 then begin
       ok = dialog_message(dialog_parent=state.tlb,!error_state.msg,/error,/center)
       catch,/cancel
       widget_control,event.handler,set_uvalue=state,/no_copy
       if obj_valid(mesg) then obj_destroy,mesg
       return
    end
    
    if (event.id eq state.alcommBut) or (event.id eq state.a2onlyBut) or (event.id eq state.tconlyBut) or (event.id eq state.jponlyBut) then widget_control,state.tlb,update=0
    if strlowcase(tag_names(event,/structure)) eq 'dm_filesel_select' then event.id = -999
    case (event.id) of
       state.tlb:   widget_control,event.id,scr_xsize=state.geom[0],scr_ysize=state.geom[1]
       state.doneBut:begin
          widget_control,event.handler,set_uvalue=state,/no_copy
          widget_control,event.handler,/destroy
          return
         end
       state.tsdminBut:begin
          state.tsdminonly = (~state.tsdminonly)
          dm_set_button,event.id,state.tsdminonly,onstring='Patch Tsdmin and Rotate Time Channel Only',offstring='Patch All'
          tmp = [state.l_mspd,state.r_mspd,state.l_wvln,state.r_wvln,state.r_resl,state.r_srmode,state.l_denom,state.r_denom]
          for i=0,n_elements(tmp)-1 do widget_control,tmp[i],sensitive=(~state.tsdminonly)
          tmp = 'lb_'+['mspd','wvln','resl','srmode','denom']
          for i=0,n_elements(tmp)-1 do begin
              wid = widget_info(state.tlb,find_by_uname=tmp[i])
              if wid ne 0 then widget_control,wid,sensitive=(~state.tsdminonly)
          endfor
          tmp = state.p_strtlett
          state.p_strtlett = state.s_strtlett
          state.s_strtlett = tmp
          id  = widget_info(state.strtlett,find_by_uname=state.p_strtlett)
          ind = where(state.lett_ids eq id,cnt)
          if cnt eq 1 then begin
             dm_toggle_menubut,uncheck=state.lett_ids[fix(byte(tmp))-fix(byte('A'))],check=id
             widget_control,state.patchBut,set_value='Patch --> ('+state.p_strtlett+')'
             if float(!version.release) ge 5.6 then $
                widget_control,state.patchBut,tooltip=state.tooltip[0]+state.p_strtlett+'.'
          endif
         end  
       state.alcommBut:begin
          state.allowcomment = (~state.allowcomment)
          dm_set_button,event.id,state.allowcomment,onstring='Allow Patching Comments',offstring='Disallow Patching Comments'
          geom = widget_info(state.patchBut,/geometry)
          widget_control,state.clabel,set_value=(['  ','Comment'])[state.allowcomment]
          if state.allowcomment then begin
             state.comment = widget_text(state.aline,value='',/editable,/all_events,scr_xsize=geom.scr_xsize)
             if n_elements(open_file) gt 0 then begin
                davePtr = dm_define_pointer() & omithisto=2 & printinfo=0 & filetype='binary'
                file = open_path+([state.pathsep,'/'])[obj_valid(ftpobj)]+open_file[0]
                if obj_valid(ftpobj) then begin
                   ok = ftpobj->GetFileContent(file,localfilename=save_path+state.pathsep+'ftptmp'+open_file[0])
                   file = save_path+state.pathsep+'ftptmp'+open_file[0]
                endif
                dcs_readdatafile,file,omithisto,printinfo,filetype,davePtr=davePtr
                if obj_valid(ftpobj) then file_delete,file,/ALLOW_NONEXISTENT,/NOEXPAND_PATH,/QUIET
                this = dm_locate_datastrptr(davePtr)
                state.p_comment = strtrim((*(*this).specificPtr).comments,2) 
                heap_free,davePtr
             endif else state.p_comment = ''
             widget_control,state.comment,set_value=state.p_comment
          endif else widget_control,state.comment,/destroy
         end 
       state.a2onlyBut:begin
          if state.allowcomment then begin
             state.allowcomment = 0b
             dm_set_button,state.alcommBut,state.allowcomment,onstring='Allow Patching Comments',offstring='Disallow Patching Comments'
             widget_control,state.comment,/destroy
          endif
          state.a2only = (~state.a2only)
          dm_set_button,event.id,state.a2only,onstring='Patch A2 Only',offstring='Disallow Patching A2'
          widget_control,state.alcommBut,sensitive=(~state.a2only)
          widget_control,state.tsdminBut,sensitive=(~state.a2only)
          widget_control,state.jponlyBut,sensitive=(~state.a2only)
          widget_control,state.tconlyBut,sensitive=(~state.a2only)
          tmp = [state.l_mspd,state.r_mspd,state.l_wvln,state.r_wvln,state.r_resl,state.r_srmode,state.l_denom,state.r_denom]
          for i=0,n_elements(tmp)-1 do widget_control,tmp[i],sensitive=((~state.tsdminonly) and (~state.a2only))
          tmp = [state.l_tsdmin,state.r_tsdmin]
          for i=0,n_elements(tmp)-1 do widget_control,tmp[i],sensitive=(~state.a2only)
          widget_control,state.r_tsdmin,sensitive=(~state.a2only)
          tmp = 'lb_'+['mspd','wvln','resl','srmode','denom']
          for i=0,n_elements(tmp)-1 do begin
              wid = widget_info(state.tlb,find_by_uname=tmp[i])
              if wid ne 0 then widget_control,wid,sensitive=((~state.tsdminonly) and (~state.a2only))
          endfor
          wid = widget_info(state.tlb,find_by_uname='lb_tsdmin')
          if wid ne 0 then widget_control,wid,sensitive=(~state.a2only)
          wid = widget_info(state.tlb,find_by_uname='lb_tsdmin')
          if wid ne 0 then widget_control,wid,sensitive=(~state.a2only)
          widget_control,state.tlb,tlb_set_title=(['Patch DCS file wavelength info','Patch DCS file A2 angles'])[state.a2only]
          if state.a2only then begin
             if n_elements(open_file) gt 0 then begin
                davePtr = dm_define_pointer() & omithisto=2 & printinfo=0 & filetype='binary'
                file = open_path+([state.pathsep,'/'])[obj_valid(ftpobj)]+open_file[0]
                if obj_valid(ftpobj) then begin
                   ok = ftpobj->GetFileContent(file,localfilename=save_path+state.pathsep+'ftptmp'+open_file[0])
                   file = save_path+state.pathsep+'ftptmp'+open_file[0]
                endif
                dcs_readdatafile,file,omithisto,printinfo,filetype,davePtr=davePtr
                if obj_valid(ftpobj) then file_delete,file,/ALLOW_NONEXISTENT,/NOEXPAND_PATH,/QUIET
                this = dm_locate_datastrptr(davePtr)
                a2 = dm_to_string(((*(*this).specificPtr).motor_pos)[1],resolution=3)
             endif else a2 = ''
             tmp = widget_info(state.l_wvln,/geometry)
             state.l_a2 = widget_text(state.aline,value=a2,xsize=tmp.xsize)
             state.r_a2 = widget_text(state.aline,value=dm_to_string(state.p_a2),xsize=tmp.xsize,/editable,/all_events)
          endif else begin
             widget_control,state.l_a2,/destroy
             widget_control,state.r_a2,/destroy
             state.a2offset = !values.f_nan
             state.p_a2 = !values.f_nan
          endelse
          widget_control,state.clabel,set_value=(['  ','A2'])[state.a2only]
         end
       state.jponlyBut:begin
           state.jumpyonly = ~state.jumpyonly
           dm_set_button,event.id,state.jumpyonly,onstring='Patch Jumpy File Only',offstring='Disallow Patching Jumpy File'
           widget_control,state.tsdminBut,sensitive=(~state.jumpyonly)
           widget_control,state.a2onlyBut,sensitive=(~state.jumpyonly)
           widget_control,state.tconlyBut,sensitive=(~state.jumpyonly)
           tmp = [state.l_mspd,state.r_mspd,state.l_wvln,state.r_wvln,state.r_resl,state.r_srmode,state.l_denom,state.r_denom]
           for i=0,n_elements(tmp)-1 do widget_control,tmp[i],sensitive=((~state.tsdminonly) and (~state.jumpyonly))
           tmp = [state.l_tsdmin,state.r_tsdmin]
           for i=0,n_elements(tmp)-1 do widget_control,tmp[i],sensitive=(~state.jumpyonly)
           widget_control,state.r_tsdmin,sensitive=(~state.jumpyonly)
           tmp = 'lb_'+['mspd','wvln','resl','srmode','denom']
           for i=0,n_elements(tmp)-1 do begin
               wid = widget_info(state.tlb,find_by_uname=tmp[i])
               if wid ne 0 then widget_control,wid,sensitive=((~state.tsdminonly) and (~state.jumpyonly))
           endfor
           wid = widget_info(state.tlb,find_by_uname='lb_tsdmin')
           if wid ne 0 then widget_control,wid,sensitive=(~state.jumpyonly)
           wid = widget_info(state.tlb,find_by_uname='lb_tsdmin')
           if wid ne 0 then widget_control,wid,sensitive=(~state.jumpyonly)
           widget_control,state.tlb,tlb_set_title=(['Patch DCS file wavelength info','Patch DCS jumpy files'])[state.jumpyonly]
           state.open_filesel->set_filter,(['.dcs','.dcs,.sht'])[state.jumpyonly],/sortbyname
           if float(!version.release) ge 5.6 then $
              widget_control,state.patchBut,tooltip=state.tooltip[state.jumpyonly]+state.p_strtlett+'.'
         end    
       state.tconlyBut:begin
           if state.allowcomment then begin
              state.allowcomment = 0b
              dm_set_button,state.alcommBut,state.allowcomment,onstring='Allow Patching Comments',offstring='Disallow Patching Comments'
              widget_control,state.comment,/destroy
           endif
           state.tconly = (~state.tconly)
           dm_set_button,event.id,state.tconly,onstring='Patch a Single Time Channel Only',offstring='Disallow Patching a Single Time Channel'
           widget_control,state.alcommBut,sensitive=(~state.tconly)
           widget_control,state.tsdminBut,sensitive=(~state.tconly)
           widget_control,state.jponlyBut,sensitive=(~state.tconly)
           widget_control,state.a2onlyBut,sensitive=(~state.tconly)
           tmp = [state.l_mspd,state.r_mspd,state.l_wvln,state.r_wvln,state.r_resl,state.r_srmode,state.l_denom,state.r_denom]
           for i=0,n_elements(tmp)-1 do widget_control,tmp[i],sensitive=((~state.tsdminonly) and (~state.tconly))
           tmp = [state.l_tsdmin,state.r_tsdmin]
           for i=0,n_elements(tmp)-1 do widget_control,tmp[i],sensitive=(~state.tconly)
           widget_control,state.r_tsdmin,sensitive=(~state.tconly)
           tmp = 'lb_'+['mspd','wvln','resl','srmode','denom']
           for i=0,n_elements(tmp)-1 do begin
               wid = widget_info(state.tlb,find_by_uname=tmp[i])
               if wid ne 0 then widget_control,wid,sensitive=((~state.tsdminonly) and (~state.tconly))
           endfor
           wid = widget_info(state.tlb,find_by_uname='lb_tsdmin')
           if wid ne 0 then widget_control,wid,sensitive=(~state.tconly)
           wid = widget_info(state.tlb,find_by_uname='lb_tsdmin')
           if wid ne 0 then widget_control,wid,sensitive=(~state.tconly)
           widget_control,state.tlb,tlb_set_title=(['Patch DCS file wavelength info','Patch DCS file'])[state.tconly]
           if state.tconly then begin
              geom = widget_info(state.patchBut,/geometry)
              state.energy = widget_text(state.aline,value='',/editable,/all_events,scr_xsize=geom.scr_xsize)
           endif else begin
              widget_control,state.energy,/destroy
              state.p_energy = !values.f_nan
           endelse
           widget_control,state.clabel,set_value=(['  ','E (meV)'])[state.tconly]
         end   
       state.helpBut: begin
          info = ['To change the wavelength info in a DCS file, select the file in the DCS raw file directory first. The original wavelength info will appear in '+$
                  'the center left column. Enter new values in the center right column. Click Patch button to save the patched file in the patched file directory. '+$
                  'The name of the patched file will start with a letter which can be changed in the option menu.','',$
                  'The comment in a DCS file can also be changed by setting the "Allow Patching Comments" option.','',$
                  'If only the tsdmin value needs to be changed, set the "Patch Tsdmin and Rotate Time Channel Only" option.','',$
                  'To patch the A2 (sample rotation) angle, set the "Patch A2 Only" option. If multiple files are selected, the same offset value calculated '+$
                  'from the A2 value in the first file and the entered new A2 value will be applied to the other files.','',$
                  'To patch a jumpy file from .sht files, set the "Patch Jumpy File Only" option. Select the jumpy .dcs.gz file, and click Patch button. '+$
                  'The .sht files need to be present in the raw file directory.']
          ok = dialog_message(info,/center,/info,dialog_parent=event.handler,title='About Patching DCS Files')
         end     
       state.patchBut:begin
          n_files = n_elements(open_file)
          if n_files eq 0 then begin
             ok = dialog_message('Please select DCS raw files first.',/error,/center,dialog_parent=event.handler)
             break
          endif
          if ~(file_test(save_path,/write)) then begin
             ok = dialog_message('The pathced file directory is unwritable.',/error,/center,dialog_parent=event.handler)
             break
          endif
          if state.tconly then begin
             if ~finite(state.p_energy) then errmsg = 'Please enter the corresponding time channel energy value.'
          endif else if state.jumpyonly then begin
             jumpy_file = ['']
             sht_count  = [0]
             for i=0,n_files-1 do begin
                 if stregex(open_file[i],'(\.dcs\.gz|\.dcs)$',/fold_case,/boolean) then begin
                    jumpy_file=[jumpy_file,open_file[i]]
                    tmp = strsplit(open_file[i],'.',/extract)
                    ind = where(stregex(dfiles,'^'+tmp[0]+'_[0-9]{4}\.sht$',/boolean),count)
                    sht_count = [sht_count,count]
                 endif             
             endfor 
             ind = where(sht_count ne 0,count)
             if count eq 0 then begin
                ok = dialog_message('No .sht files are found for the selected raw data file'+(['','s'])[n_files gt 1]+'.',/error,/center,dialog_parent=event.handler)
                break
             endif else begin
                open_file = jumpy_file[ind]
                sht_count = sht_count[ind]
                n_files   = count 
             endelse
          endif else if state.a2only then begin
             if ~finite(state.p_a2) then $
                errmsg = 'Please enter A2 value.' $
             else if n_files gt 1 then begin
                ok = dialog_message('The same A2 offset value of '+dm_to_string(state.a2offset)+' will be applied to all files.',/center,dialog_parent=event.handler)
             endif
          endif else begin
             if (~state.tsdminonly) then begin  
                if state.p_mspd eq 0 then errmsg = 'Incorrect master speed.'
                if ~finite(state.p_wvln) then begin
                   if n_elements(errmsg) eq 0 then errmsg = 'Incorrect wavelength.' $
                   else errmsg = [errmsg,'Incorrect wavelength.']
                endif
                if state.p_denom eq 0 then begin
                   if n_elements(errmsg) eq 0 then errmsg = 'Incorrect speed ratio denominator.' $
                   else errmsg = [errmsg,'Incorrect speed ratio denominator.']
                endif
             endif
             if state.p_tsdmin eq 0 then begin
                if n_elements(errmsg) eq 0 then errmsg = 'Incorrect tsdmin.' $
                else errmsg = [errmsg,'Incorrect tsdmin.']
             endif
          endelse
          if n_elements(errmsg) gt 0 then begin
             ok = dialog_message(errmsg,/error,/center,dialog_parent=event.handler)
             break
          endif
          ;Define scalar quantities
          if state.tsdminonly then begin
             squans   = ["tsdmin"]
             v_squans = double([state.p_tsdmin])
          endif else begin
             squans   = ["ch_ms","ch_res","ch_srdenom","ch_srmode","ch_wl","tsdmin"]
             v_squans = double([state.p_mspd,state.p_resl,state.p_denom,state.p_srmode,state.p_wvln,state.p_tsdmin])
          endelse
          for i=0L,n_files-1 do begin
              if i eq 0 then $
                 mesg = obj_new('dm_progress',title='Patching...',message='Patching '+open_file[i],group_leader=state.tlb) $
              else begin
                 if (~ obj_valid(mesg)) then break
                 mesg->update,message='Patching '+open_file[i],title='Patching...  '+dm_to_string(i+1)+'/'+dm_to_string(n_files)
              endelse
              if state.jumpyonly then begin ; load the .sht datas first
                 sum = 0L & temp_ind = 0L
                 tmp = strsplit(open_file[i],'.',/extract)
                 ind = where(stregex(dfiles,'^'+tmp[0]+'_[0-9]{4}\.sht$',/boolean),count)
                 max_ind = 1
                 for j=0,count-1 do begin
                     infile  = open_path+([state.pathsep,'/'])[obj_valid(ftpobj)]+dfiles[ind[j]]
                     if obj_valid(ftpobj) then begin
                        ok = ftpobj->GetFileContent(infile,localfilename=save_path+state.pathsep+'ftptmp'+dfiles[ind[j]])
                        infile = save_path+state.pathsep+'ftptmp'+dfiles[ind[j]]
                     endif
                     openr,unit,infile,/get_lun
                     data = ulonarr(1024,934) 
                     readu,unit,data & free_lun,unit
                     if obj_valid(ftpobj) then file_delete,infile,/ALLOW_NONEXISTENT,/NOEXPAND_PATH,/QUIET
                     ;now check if it is jumpy
                     qty  = total(data[*,0:912],1)
                     qty_max = max(qty) & qty_mean = mean(qty)
                     m_ind   = where(qty ge 10*qty_mean,count1)
                     if count1 ne 0 then qty[m_ind] = qty_max
                     tmp = a_correlate(qty,[31,248])
                     tmp1 = dm_to_number(strmid(infile,7,4,/reverse_offset))
                     max_ind = max_ind>(tmp1)
                     if min(tmp) gt 0.3 then continue $
                     else begin
                          sum = sum+data
                          temp_ind = [temp_ind,tmp1-1]
                     endelse
                 endfor 
                 if n_elements(sum) eq 1 then begin   ;all jumpy data
                    if n_elements(ajfile) eq 0 then ajfile = open_file[i] else ajfile = [ajfile,open_file[i]]
                    continue
                 endif
                 sum = double(sum)
              endif
              infile  = open_path+([state.pathsep,'/'])[obj_valid(ftpobj)]+open_file[i] 
              outfile = save_path+state.pathsep+state.p_strtlett+open_file[i]
              if obj_valid(ftpobj) then begin
                 ok = ftpobj->GetFileContent(infile,localfilename=save_path+state.pathsep+'ftptmp'+open_file[i])
                 infile = save_path+state.pathsep+'ftptmp'+open_file[i]
              endif
              ;check if the file is compressed
              compress = (strlowcase(strmid(infile,2,3,/reverse_offset)) eq '.gz')
              ;Read header to determine byte ordering ("endianness")
              openr,unit,infile,compress=compress,/get_lun
              header = bytarr(11)
              readu,unit,header
              free_lun,unit
              continued = 0b
              case string(header[9]) of
                "B": begin
                     openr,unitr,infile,/swap_if_little_endian,compress=compress,/get_lun
                     openw,unitw,outfile,/swap_if_little_endian,compress=compress,/get_lun
                     end
                "L": begin
                     openr,unitr,infile,/swap_if_big_endian,compress=compress,/get_lun
                     openw,unitw,outfile,/swap_if_big_endian,compress=compress,/get_lun
                     end
               else: begin
                     ok = dialog_message("Cannot determine byte ordering of file "+open_file[i]+'. This file is not patched.',/center,dialog_parent=state.tlb)
                     continued = 1b
                     end
              endcase
              if continued then begin
                 if obj_valid(ftpobj) then file_delete,infile,/ALLOW_NONEXISTENT,/NOEXPAND_PATH,/QUIET
                 state.save_filesel->set_path      ;update the save directory
                 continue
              endif
              readu,unitr,header
              writeu,unitw,header             
              ; Define the types of some variables
              nlgth   = 0l
              dldocgf = bytarr(5)
              dt      = 0b
              padding = 0b
              scalar  = 0.0d
              n1      = 0l
              n2      = 0l
              nstrgs  = 0l
              nchars  = 0l
              while (~eof(unitr)) do begin
                 readu,unitr,nlgth              ; read name_length
                 name = bytarr(nlgth)
                 readu,unitr,name               ; read name
                 readu,unitr,dldocgf            ; read 5 bytes of doc_length, doc, and global flag
                 readu,unitr,dt                 ; read data type
                 writeu,unitw,nlgth
                 writeu,unitw,name
                 writeu,unitw,dldocgf
                 writeu,unitw,dt
                 name = strlowcase(string(name))
                 case dt of
                      1: begin                  ; read scalar
                         readu,unitr,padding
                         writeu,unitw,padding
                         readu,unitr,scalar
                         ; Patchable scalars
                         if (~state.a2only) and (~state.jumpyonly) and (~state.tconly) then begin
                            if state.tsdminonly then begin ;save masterspeed and speed denominator info
                               case name of
                                    'ch_ms':      ch_ms0 = scalar
                                    'ch_srdenom': ch_srdenom0 = scalar
                                    'tsdmin':     tsdmin0 = scalar
                                    else:
                               endcase
                            endif
                            index = where(squans eq name,cnt)
                            if (cnt eq 1) then scalar = v_squans[index]
                         endif else if state.jumpyonly then begin
                            case name of 
                                 'duration':  scalar = scalar/max_ind*(n_elements(temp_ind)-1)
                                 'ncycles':   scalar = double(n_elements(temp_ind)-1)
                                 else:
                            endcase
                         endif else if state.tconly then begin
                            case name of 
                                 'tsdmin':    tsdmin0 = scalar
                                 'ch_wl':     ch_wl0  = scalar
                                 else:
                            endcase
                         endif
                         writeu,unitw,scalar
                         end
                      2: begin                  ; read matrix
                         readu,unitr,n1,n2
                         nhi = (n1>n2)
                         nlo = (n1<n2)
                         if state.jumpyonly and ((name eq 'temp_control') or (name eq 'temp_sample')) then begin
                            if n1 eq 1 then n2 = n_elements(temp_ind)-1 $
                            else n1 = n_elements(temp_ind)-1
                         endif
                         writeu,unitw,n1,n2
                         readu,unitr,padding
                         writeu,unitw,padding
                         if (nlo eq 1) then data=dblarr(nhi) else data=dblarr(n1,n2) 
                         readu,unitr,data
                         if state.jumpyonly then begin
                            case name of 
                                 'temp_control': data = data[temp_ind[1:*]]
                                 'temp_sample':  data = data[temp_ind[1:*]]
                                 'histodata':    data = sum[*,0:912]
                                 'histohigh':    data = sum[*,913:930]
                                 'timsum':       data = total(sum[*,0:930],2)
                                 'detsum':       data = total(sum[*,0:930],1)
                                 else:
                            endcase
                         endif else if state.a2only then begin
                            if name eq 'motor_pos' then data[1] = state.a2offset+data[1]
                         endif else if state.tconly then begin
                            case name of 
                                 'tchanlook':    tchanlook = data
                                 'histodata':    begin
                                                 times = tsdmin0+[0,reform(total(tchanlook[0,*]*0.05,/cumulative))]
                                                 conversion = !dcs_hsq2mn/(!dcs_hom/!dcs_dsd)^2
                                                 if (ch_wl0 eq 0.0) then energies = -conversion/times^2 $
                                                 else energies = !dcs_hsq2mn/ch_wl0^2-conversion/times^2
                                                 tmptmp = min(abs(energies-state.p_energy),tmp_imin)
                                                 i_prev = tmp_imin-1 
                                                 i_next = tmp_imin+1
                                                 if i_prev lt 0 then i_prev = 999
                                                 if i_next eq 1000 then i_next = 0 
                                                 data[tmp_imin,*] = (data[i_prev,*]+data[i_next,*])/(2d)
                                                 end 
                                 else:
                            endcase
                         endif else begin
                            if state.tsdminonly and ((name eq 'histohigh') or (name eq 'histodata') or (name eq 'timesum')) then begin ;needs to shift data
                               tchwid = 6e4/ch_ms0*ch_srdenom0               ;assuming equal width, in usec
                               nshift = round((tsdmin0-state.p_tsdmin)/tchwid)
                               if (name eq 'histodata') or (name eq 'histohigh') then $
                                  data=[shift(data[0:999,*],nshift,0),data[1000:1023,*]] $
                               else $
                                  data=[shift(data[0:999],nshift),data[1000:1023]] ;timesum
                            endif
                         endelse
                         writeu,unitw,data
                         end
                      5: begin                  ; read string
                         readu,unitr,nchars     ; read number of characters in string element
                         if (nchars gt 0) then begin
                            str = bytarr(nchars)
                            readu,unitr,str     ; read string element
                         endif
                         if (name eq 'comments') and state.allowcomment then begin
                            nchars = long(strlen(state.p_comment)) 
                            str = byte(state.p_comment)
                         endif 
                         writeu,unitw,nchars
                         if (nchars gt 0) then writeu,unitw,str
                         end
                      7: begin                  ; read string array
                         readu,unitr,nstrgs     ; read number of strings in array
                         writeu,unitw,nstrgs
                         for j=0L,nstrgs-1 do begin
                             readu,unitr,nchars ; read number of characters in string element
                             writeu,unitw,nchars
                             str = bytarr(nchars)
                             readu,unitr,str    ; read string element
                             writeu,unitw,str
                         endfor
                         end
                   else:
                 endcase
              endwhile
              free_lun,unitr
              free_lun,unitw
              if obj_valid(ftpobj) then file_delete,infile,/ALLOW_NONEXISTENT,/NOEXPAND_PATH,/QUIET
              state.save_filesel->set_path      ;update the save directory
          endfor
          if n_elements(ajfile) ne 0 then $
             ok = dialog_message(["There is no recoverable data in the following file"+(['','s'])[n_elements(ajfile) gt 1]+":",ajfile,'', (["This file is","These files are"])[n_elements(ajfile) gt 1]+" not patched."],/center,dialog_parent=state.tlb)
          state.open_filesel->set_path,ftp=obj_valid(ftpobj)
          state.open_filesel->clear_file,fileloc=open_file[0]
          if obj_valid(mesg) then obj_destroy,mesg
         end  
       state.r_mspd:begin
          widget_control,event.id,get_value=tmp
          state.p_mspd = dm_to_number(tmp,/int)
         end
       state.r_wvln:begin
          widget_control,event.id,get_value=tmp
          state.p_wvln = dm_to_number(tmp,/float)
         end  
       state.r_tsdmin:begin
          widget_control,event.id,get_value=tmp
          state.p_tsdmin = dm_to_number(tmp,/int)
         end  
       state.r_resl:begin
          widget_type = widget_info(event.id,/type)
          if widget_type eq 8 then tmp_value=widget_info(event.id,/droplist_select) $
          else if widget_type eq 12 then begin
             tmp_value=event.index
             if dm_to_string((['low','medium'])[tmp_value]) ne event.str then break
          endif else tmp_value=0
          state.p_resl = tmp_value+1
         end  
       state.r_srmode:begin
          widget_type = widget_info(event.id,/type)
          if widget_type eq 8 then tmp_value=widget_info(event.id,/droplist_select) $
          else if widget_type eq 12 then begin
             tmp_value = event.index
             if dm_to_string((['1/m','(m-1)/m'])[tmp_value]) ne event.str then break
          endif else tmp_value = 0
          state.p_srmode = tmp_value+1
         end    
       state.r_denom:begin
          widget_control,event.id,get_value=tmp
          state.p_denom = dm_to_number(tmp,/int)
         end    
       state.r_a2: begin
          widget_control,event.id,get_value=tmp
          state.p_a2 = dm_to_number(tmp,/float)
          widget_control,state.l_a2,get_value=tmp
          tmp = dm_to_number(tmp,/double)
          if finite(tmp) and finite(state.p_a2) then state.a2offset=state.p_a2-tmp
         end
       state.comment:begin
          widget_control,event.id,get_value=tmp
          state.p_comment = strtrim(tmp,2)
         end
       state.energy:begin
          widget_control,event.id,get_value=tmp
          state.p_energy = dm_to_number(tmp,/float)
         end   
       else: begin
          if strlowcase(tag_names(event,/structure)) eq 'dm_filesel_select' then begin  ;let file selector click event
             if event.object eq state.open_filesel then begin
                davePtr = dm_define_pointer() & omithisto=2 & printinfo=0 & filetype='binary'
                file = open_path+([state.pathsep,'/'])[obj_valid(ftpobj)]+open_file[0]
                if obj_valid(ftpobj) then begin
                   ok = ftpobj->GetFileContent(file,localfilename=save_path+state.pathsep+'ftptmp'+open_file[0])
                   file = save_path+state.pathsep+'ftptmp'+open_file[0]
                endif
                dcs_readdatafile,file,omithisto,printinfo,filetype,davePtr=davePtr
                if obj_valid(ftpobj) then file_delete,file,/ALLOW_NONEXISTENT,/NOEXPAND_PATH,/QUIET
                this = dm_locate_datastrptr(davePtr)
                widget_control,state.l_mspd,set_value=dm_to_string((*(*this).specificPtr).ch_ms)
                widget_control,state.l_wvln,set_value=dm_to_string((*(*this).specificPtr).ch_wl,resolution=3)
                widget_control,state.l_tsdmin,set_value=dm_to_string((*(*this).specificPtr).tsdmin)
                dm_set_droplist,state.l_resl,select=(*(*this).specificPtr).ch_res-1
                dm_set_droplist,state.l_srmode,select=(*(*this).specificPtr).ch_srmode-1
                widget_control,state.l_denom,set_value=dm_to_string((*(*this).specificPtr).ch_srdenom)
                if state.allowcomment then begin
                   state.p_comment = strtrim((*(*this).specificPtr).comments,2) 
                   widget_control,state.comment,set_value=state.p_comment
                endif  
                if state.a2only then begin
                   a2 = round(((*(*this).specificPtr).motor_pos)[1]*1000.0d)/(1000.0d) ;accurate to 0.001 degree
                   widget_control,state.l_a2,set_value=dm_to_string(a2)
                   if finite(state.p_a2) then begin
                      if finite(state.a2offset) then begin  
                         state.p_a2 = a2+state.a2offset
                         widget_control,state.r_a2,set_value=dm_to_string(state.p_a2)
                      endif else begin
                         state.a2offset = state.p_a2-a2
                      endelse
                   endif
                endif  
                heap_free,davePtr
             endif
          endif else begin  ;file name starting letter menu event
             ind = where(state.lett_ids eq event.id,cnt)
             if cnt eq 1 then begin
                dm_toggle_menubut,uncheck=state.lett_ids[fix(byte(state.p_strtlett))-fix(byte('A'))],check=event.id
                state.p_strtlett = string(byte('A')+byte(ind))
                widget_control,state.patchBut,set_value='Patch --> ('+state.p_strtlett+')'
                if float(!version.release) ge 5.6 then $
                   widget_control,state.patchBut,tooltip=state.tooltip[state.jumpyonly]+state.p_strtlett+'.'
             endif
          endelse
         end
    endcase
    if (event.id eq state.alcommBut) or (event.id eq state.a2onlyBut) or (event.id eq state.tconlyBut) or (event.id eq state.jponlyBut) then widget_control,state.tlb,/update
    widget_control,event.handler,set_uvalue=state,/no_copy
end

;main program
pro dm_dcspatch,event,dataDir=dataDir,workDir=workDir
    state={ group_leader:    0L, $   ;group leader
         tlb:                0L, $   ;top level base
         open_filesel: obj_new(),$   ;open file selector
         save_filesel: obj_new(),$   ;save file selector
         l_mspd:             0L, $   ;raw master speed text box
         r_mspd:             0L, $   ;patch master speed text box
         p_mspd:         20000s, $   ;master speed 
         l_wvln:             0L, $   ;raw wavelength text box
         r_wvln:             0L, $   ;patch wavelength text box
         p_wvln:  !values.f_nan, $   ;wavelength
         l_tsdmin:           0L, $   ;raw tsdmin text box
         r_tsdmin:           0L, $   ;patch tsdmin text box
         p_tsdmin:           0s, $   ;tsdmin
         l_resl:             0L, $   ;raw resolution mode selection droplist
         r_resl:             0L, $   ;patch resolution mode selection droplist
         p_resl:             1s, $   ;resolution mode    1-low 2-medium
         l_srmode:           0L, $   ;raw speed ratio mode selection droplist
         r_srmode:           0L, $   ;patch speed ratio mode selection droplist
         p_srmode:           1s, $   ;speed ratio mode   1-1/m 2-(m-1)/m
         l_denom:            0L, $   ;raw speed ratio denominator text box
         r_denom:            0L, $   ;patch speed ratio denominator text box
         p_denom:            0s, $   ;speed ratio denominator
         clabel:             0L, $   ;comment/a2 label
         aline:              0L, $   ;a2 line
         l_a2:               0L, $   ;raw a2 text box
         r_a2:               0L, $   ;patch a2 text box
         p_a2:    !values.f_nan, $   ;a2 value
         a2offset:!values.f_nan, $   ;save a2 offset value
         strtlett:           0L, $   ;starting letter menu
         lett_ids:   lonarr(26), $
         p_strtlett:        'P', $   ;starting letter
         s_strtlett:        'R', $   ;buffer 
         comment:            0L, $   ;comment text box
         p_comment:          '', $   ;comment string
         energy:             0L, $   ;energy text box
         p_energy:!values.f_nan, $   ;energy value
         patchBut:           0L, $   ;convert button
         doneBut:            0L, $   ;done button
         tsdminBut:          0L, $   ;tsdmin and rotate only option button
         tsdminonly:         0b, $   ;tsdmin and rotate only flag
         alcommBut:          0L, $   ;allow comment option button
         allowcomment:       0b, $   ;allow comment
         a2onlyBut:          0L, $   ;A2 only option button
         a2only:             0b, $   ;A2 only flag
         jponlyBut:          0L, $   ;jumpy only option button
         jumpyonly:          0b, $   ;Jumpy only flag
         tconlyBut:          0L, $   ;single time channel only button
         tconly:             0b, $   ;single time channel only flag
         helpBut:            0L, $   ;Help button
         pathsep:            '', $   ;path seperator
         tooltip:       ['',''], $   ;tool tip for the patch button
         geom:    [0L,0L,0L,0L]  $   ;xsize,ysize,xoffset,yoffset
    }
    registerName    ='dm_dcspatch'
    if xregistered(registerName) then return   ;only allow one copy to be running at one time
    ;get the correct path separator
    if float(!version.release) ge 5.5 then state.pathsep=call_function("path_sep") $
    else begin
       defsysv,'!delimiter',exists=exists
       if (~ exists) then begin
          case !version.os_family of
            'Windows':  defsysv,'!delimiter','\'
            'unix':     defsysv,'!delimiter','/'
            'MacOS':    defsysv,'!delimiter',':'
            else:       defsysv,'!delimiter','/'
          endcase
       endif
       state.pathsep = !delimiter
    endelse
    
    if n_elements(event) ne 0 then begin
       state.group_leader = event.top
       state.tlb = widget_base(title='Patch DCS file wavelength info',/col,kill_notify='dm_dcspatch_Exit',group_leader=event.top,/tlb_size_event,mbar=mbar,xpad=0,ypad=0)
    endif else $
       state.tlb = widget_base(title='Patch DCS file wavelength info',/col,kill_notify='dm_dcspatch_Exit',/tlb_size_event,mbar=mbar,xpad=0,ypad=0)
    
    ;menu bar
    filemenu = widget_button(mbar,value='File',/menu)
    optnmenu = widget_button(mbar,value='Options',/menu)
    helpmenu = widget_button(mbar,value='Help',/menu)   
    
    state.doneBut   = widget_button(filemenu,value='Exit')
    state.alcommBut = dm_widget_button(optnmenu,value='Allow Patching Comments',set_button=state.allowcomment,onstring='Allow Patching Comments',offstring='Disallow Patching Comments')
    state.tsdminBut = dm_widget_button(optnmenu,value='Patch Tsdmin and Rotate Time Channel Only',set_button=state.tsdminonly,onstring='Patch Tsdmin and Rotate Time Channel Only',offstring='Patch All')
    state.a2onlyBut = dm_widget_button(optnmenu,value='Patch A2 Only',set_button=state.a2only,onstring='Patch A2 Only',offstring='Disallow Patching A2')
    state.jponlyBut = dm_widget_button(optnmenu,value='Patch Jumpy File Only',set_button=state.jumpyonly,onstring='Patch Jumpy File Only',offstring='Disallow Patching Jumpy File')
    state.tconlyBut = dm_widget_button(optnmenu,value='Patch a Single Time Channel Only',set_button=state.tconly,onstring='Patch a Single Time Channel Only',offstring='Disallow Patching a Single Time Channel')
    state.strtlett  = widget_button(optnmenu,value='Patched File Name Starts With',/separator,/menu)
    for i=0,25 do begin
        tmp = string(byte('A')+byte(i))
        state.lett_ids[i] = dm_widget_button(state.strtlett,value=tmp,set_button=(tmp eq state.p_strtlett),uname=tmp)
    endfor
    state.helpBut  = dm_widget_button(helpmenu,value='Help')
    
    ;menu bar separator for windows system
    if !version.os_family eq 'Windows' then begin
       mbar_sep    = widget_label(state.tlb,sensitive=0,/dynamic_resize,value=' ',scr_ysize=5)
       tmp         = 0
    endif else tmp = 4
    xsize          = 9
    
    row            = widget_base(state.tlb,/row,ypad=0)
    openCol        = widget_base(row,/col,ypad=0)
    cmdCol         = widget_base(row,/col,/align_center,ypad=0,xpad=0)
    saveCol        = widget_base(row,/col,ypad=0)

    label          = widget_label(openCol,value='DCS raw file directory')
    label          = widget_label(saveCol,value='Patched file directory')

    state.open_filesel = obj_new('dm_filesel',parent=openCol,ysize=18+tmp,/frame,filter='.dcs',path=dataDir,/selectevent,group_leader=state.tlb,$
                     ncnrftp=file_test(n_elements(workDir) eq 0?'':workDir,/directory,/write),bufferdir=workDir)
    state.save_filesel = obj_new('dm_filesel',parent=saveCol,ysize=18+tmp,/frame,filter='.dcs',path=workDir)

    label          = widget_label(cmdcol,value='Master Speed',uname='lb_mspd')
    line           = widget_base(cmdcol,/row,xpad=0,space=0,ypad=0,/align_center)
    state.l_mspd   = widget_text(line,value='',xsize=xsize)
    state.r_mspd   = widget_text(line,value=((state.p_mspd gt 0)?dm_to_string(state.p_mspd):' '),xsize=xsize,/editable,/all_events)
    geom           = widget_info(state.r_mspd,/geometry)
    label          = widget_label(cmdcol,value='Wavelength',uname='lb_wvln')
    line           = widget_base(cmdcol,/row,xpad=0,space=0,ypad=0,/align_center)
    state.l_wvln   = widget_text(line,value='',xsize=xsize)
    state.r_wvln   = widget_text(line,value=dm_to_string(state.p_wvln),xsize=xsize,/editable,/all_events) 
    label          = widget_label(cmdcol,value='Tsdmin',uname='lb_tsdmin')
    line           = widget_base(cmdcol,/row,xpad=0,space=0,ypad=0,/align_center)
    state.l_tsdmin = widget_text(line,value='',xsize=xsize)
    state.r_tsdmin = widget_text(line,value=((state.p_tsdmin gt 0)?dm_to_string(state.p_tsdmin):' '),xsize=xsize,/editable,/all_events) 
    label          = widget_label(cmdcol,value='Resolution',uname='lb_resl')
    line           = widget_base(cmdcol,/row,xpad=0,space=0,ypad=0,/align_center)
    state.l_resl   = dm_widget_droplist(line,value=['low','medium'],xsize=geom.scr_xsize,sensitive=0)
    state.r_resl   = dm_widget_droplist(line,value=['low','medium'],xsize=geom.scr_xsize) 
    label          = widget_label(cmdcol,value='Speed Ratio Mode',uname='lb_srmode')
    line           = widget_base(cmdcol,/row,xpad=0,space=0,ypad=0,/align_center)
    state.l_srmode = dm_widget_droplist(line,value=['1/m','(m-1)/m'],xsize=geom.scr_xsize,sensitive=0)
    state.r_srmode = dm_widget_droplist(line,value=['1/m','(m-1)/m'],xsize=geom.scr_xsize) 
    label          = widget_label(cmdcol,value='Speed Ratio Denominator',uname='lb_denom')
    line           = widget_base(cmdcol,/row,xpad=0,space=0,ypad=0,/align_center)
    state.l_denom  = widget_text(line,value='',xsize=xsize)
    state.r_denom  = widget_text(line,value=((state.p_denom gt 0)?dm_to_string(state.p_denom):' '),xsize=xsize,/editable,/all_events) 
    state.clabel   = widget_label(cmdcol,value=(['  ','Comment','A2'])[state.allowcomment+2*state.a2only],/dynamic_resize)
    state.aline    = widget_base(cmdcol,/row,xpad=0,space=0,ypad=0,/align_center)
    if state.allowcomment then $
    state.comment  = widget_text(state.aline,value='',/editable,/all_events)
    if state.a2only then begin
       state.l_a2  = widget_text(state.aline,value='',xsize=xsize)
       state.r_a2  = widget_text(state.aline,value='',xsize=xsize)
    endif    
    state.patchBut = widget_button(cmdCol,value='Patch --> ('+state.p_strtlett+')')
    state.tooltip  = ['Select raw files, enter patch values in the center right column, then click this button to patch files.','Select raw .dcs.gz files, then click this button to patch. Sht files need to be present.']+$
                      ' Patched file name starts with '
    if float(!version.release) ge 5.6 then $
       widget_control,state.patchBut,tooltip=state.tooltip[0]+state.p_strtlett+'.'

    dm_center_kid, state.tlb   ;centering the top level base
    widget_control,state.tlb,/realize
    geom = widget_info(state.tlb,/geometry)
    state.geom = [geom.scr_xsize,geom.scr_ysize,geom.xoffset,geom.yoffset]
    tlb  = state.tlb
    widget_control,state.tlb,set_uvalue=state,/no_copy
    xmanager,registerName,tlb,cleanup='dm_dcspatch_Exit',/no_block
end